;;;  -*- mode: LISP; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : John R. Anderson & Andrea Stocco
;;; Copyright   : (c) 2005 John R. Anderson
;;; Address     : Department of Psychology 
;;;             : Carnegie Mellon University
;;;             : Pittsburgh, PA 15213-3890
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : AL-buffers.lisp
;;; Version     : 0.1a. (largely) Based on the previous "Box-buffers" code by JRA.
;;; 
;;; Description : Describes the additional modules used by Algebra model
;;;
;;; Date        : Oct 14th, 2005.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Describes the modules that are available for the Algebra model. These modules
;;; let the model interact with tutor interface through a set of specific API.
;;;
;;; All of the modules may be roughly divided as input or output modules.
;;; In input modules, information is taken from tutor and conveyed to the model
;;; in an accessible format.  In output modules, information is sent to the
;;; tutor in form of request to a module.  Some module are used both
;;; as an input and an output module.   In the following list, values in
;;; square brackets indicate the chunk-types that are used by each module.
;;;
;;; The set of INPUT modules comprises:
;;;    - The SCREEN module         [SCREEN]
;;;    - The VERBAL module         [VERBAGE]
;;;    - The LABEL module          [ASSOC]
;;;
;;; The set of OUTPUT modules comprises one only module:
;;;    - The DIGITAL module        [DIGITAL]
;;;      used for trasforming finger-movements into the corresponding tutor events
;;;
;;; The set of input/output modules:
;;;    - The GRAPHICAL module      [BOX]
;;;    - The MATHEMATICAL module   [VERBAGE]
;;;
;;; In the Math module, requests to this module are VERBAGE chunks
;;; that are interpreted as simple arithmetic calculations (additions and
;;; multiplications, mostly).  The arguments are processed by Lisp code, and
;;; the result is stored into the RESULT slot of the VERBAGE chunk in the 
;;; mathematical buffer.  In a subsequent step, the model can read such a value
;;; and make use of it.
;;;
;;; Note that if a module is input-only, then you should not find any
;;;  +buffer> commands (as is the case of SCREEN) and that, conversely, if a
;;; module is output-only, you should not have any =buffer> requests.  There
;;; are, however, minor exceptions within this code.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; To do's
;;;
;;; * Specify the Label module query function (currently undefined, even if
;;; a parameter for it exists).  
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; GENERAL FUNCTIONS AND PARAMETERS
;;; An input module for retrieving hint information.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *zvar* nil)               ;; A var for storing infor for debugging purposes
(defparameter *instructions* nil)
(defparameter *hint* nil)

(defvar *state*)                        ;; A variable holding current interaction state
(defvar *ignore-error*)
(defvar *assess-success*)


;; Gets the value of the slot ELEM in the
;; slot definition LIS.
;;
(defun get-value (lis elem) 
  (second (member elem lis)))


;; CREATE-CHUNK
;; Creates a new chunk in the specified buffer, according to the specified
;; description. It also set the BUSY parameter to nil
;;
(defun create-chunk (buffer-name chunk-description busy)
  ;  (format t "~%Called CREATE-CHUNK in buffer ~A and with description = ~A~%" buffer-name chunk-description)
  (set busy nil)
  (create-new-buffer-chunk buffer-name chunk-description))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; THE VERBAL MODULE
;;; An input module for retrieving hint information.  The module is used the 
;;; read task information from special areas of the screen.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar *comprehension*)
(defparameter *verbal-busy* nil)        ;; Whether the verbal module is busy or not
(defvar *verbal-error*)                 ;; If an error has occurred in the verbal module

;; The QUERY function for the VERBAL module
;;
(defun verbal-query (instance buffer-name slot value)
  (declare (ignore buffer-name instance))
  (case slot
    (state
     (case value
       (busy *verbal-busy*)
       (free (not *verbal-busy*))
       (error-free (not *verbal-error*))
       (error *verbal-error*)
       (t (print-warning "Unknown state query ~S to goal module" value)
          nil)))))


;; The REQUEST function
;;
(defun request-verbal-chunk (instance buffer-name chunk-spec)
  (declare (ignore instance))
  (let* ((specs  (chunk-spec-to-chunk-def chunk-spec))
         (description
          (case (get-value  specs 'command)
          (read-instructions   (load-up '*instructions*))
          (advance (advance (get-value specs 'arg1) (car (no-output (buffer-chunk verbal)))))
          (correct-error (correct-error))
          (read-opening (load-up '*instructions*))           
          (read-special (read-opening (get-value specs 'arg1)))
          (read-hint  (if (not (cdr *hint*)) (setf *hint* (cons 'hint  (follow-help))))  (load-up '*hint*))))
         (delay  (if (equal (get-value  specs 'command) 'advance) (* (get-value specs 'arg1) .2) .2)))
    (update-state)
    (setf *verbal-busy* t)
    (setf *verbal-error* nil)
    (cond (description (schedule-event-relative delay 'create-verbal-chunk 
                                                :module buffer-name
                                                :params (list buffer-name description)))
          (t (schedule-event-relative 1 'create-verbal-error 
                                      :module buffer-name
                                      :priority -100)))))

(defun advance (number chunk)
  (cond ((numberp number)
         (do ((new '(first second third fourth fifth sixth seventh eighth ninth tenth eleventh twelfth) (cdr new))
              (old (nthcdr number '(first second third fourth fifth sixth seventh eighth ninth tenth eleventh twelfth)) (cdr old))
              (result '(isa sentence) (append result (list (car new) (chunk-slot-value-fct chunk (car old))))))
             ((null (chunk-slot-value-fct chunk (car old))) result)))
        ((equal number 'rest) '(isa sentence))))


(defun load-up (instructions)
  (let ((words (second (eval instructions))))
    (if (listp (second words)) (setf words (append (list (car words) (eval (second words))) (cddr words))))
    (set instructions (cons (car (eval instructions)) (cddr (eval instructions))))
  (do ((temp words (cdr temp))
       (slots '(first second third fourth fifth sixth seventh eighth ninth tenth eleventh twelfth) (cdr slots))
       (result '(isa sentence) (append result (list (car slots) (car temp)))))
      ((null temp) result))))

;; Sets the appropriate parameter in case o a verbal error
;;
(defun create-verbal-error ()
  (setf *verbal-busy* nil)
  (setf *verbal-error* t))

;; Resets the parameter the their normal values after
;; a verbal error has occurred.
;;
(defun free-verbal-error ()
  (setf *verbal-error* nil))

;; Creates a new chunk to be put in the BUFFER-NAME buffer.
;; The chunk is created according to the CHUNK-DESCRIPTION
;;
(defun create-verbal-chunk (buffer-name chunk-description)
  (setf *verbal-busy* nil)
  (create-new-buffer-chunk buffer-name chunk-description))

(defun execute-verbal (lis)
  ; (format t "~%Executed EXECUTE-VERBAL with list: ~A" lis)
  (do ((temp lis (cdr temp)))
      ((null temp) nil)
    (cond ((chunk-p-fct (caar temp))
           (set-buffer-chunk 'retrieval (caar temp))
           (clear-buffer 'retrieval))
          (t (eval `(add-dm ,(car temp)))))))

;; READ-HINT
;; Reads an hint 
;;
(defun read-hint (arg)
  (let ((instruction (cond ((find-matching *hint* arg))
                           (t (setf *hint* (follow-help)) (find-matching *hint* arg)))))
    (cond ((not instruction)
           (setf *instructions* nil)(perceive-instructions) (setf *hint* nil)
           (let ((instruction (find-matching (cdr *instructions*) arg)))
             (cond ((equal (chunk-slot-value-fct (car instruction) 'pre) arg) 
                    (list   (list 'isa 'verbage 'command 'read-instruction 'result (car instruction))
                            (append instruction (list 'pre arg))))
                   (t (setf *instructions* (list (car *instructions*))) nil))))
          ;                      (list   (list 'isa 'verbage 'command 'read-hint 'result nil))))))
          (t  (setf *instructions* (list (car *instructions*)))
              (list   (list 'isa 'verbage 'command 'read-hint 'result (car instruction))
                      (append instruction (list 'pre arg)))))))

;; READ-INSTRUCTIONS
;;
(defun read-instructions (arg)
  (let ((instruction (find-matching (cdr *instructions*) arg)))
    (cond ((or (not instruction) (equal (car (last instruction)) 'unknown))
           (setf *instructions* (list (car *instructions*)))
           (mod-buffer-chunk 'screen '(instructions nil)) (setf *hint* (follow-help)) (update-state)
           (list (list 'isa 'verbage 'command 'read-instructions 'result nil)))
          (t  (setf *instructions* (remove instruction *instructions*))
              (list   (list 'isa 'verbage 'command 'read-instructions 'result (car instruction))
                      (append instruction (list 'pre arg)))))))


;; READ-OPENING
;;
(defun read-opening (arg)
  (if (cdr *instructions*)
    (append (list (list 'isa 'verbage 'command 'read-instructions 'result (car (second *instructions*)))
                  (if (get-value (second *instructions*) 'pre) (second *instructions*)
                      (append (second *instructions*) (list 'pre arg))))
            (cddr *instructions*))
    '((isa verbage command read-instructions result nothing))))




;; CORRECT-ERROR
;;
(defun correct-error ()
  (let ((text (cdr (assoc ':text (dump-screen))))
       (title (second (assoc :title (second (assoc ':instructions (dump-screen)))))))
    (cond ((and (listp (car text)) (equal text '(("Your answer is incorrect." "The result of subtracting something from 10 is 2.
To find out the value of that something, subtract 2 from 10.
Click the green box and type 10 - 2."))))
           (execute-command :click (retrieve-label 'box))
           (update-state)
           (perceive-instructions) (read-instructions (perceive-state)))
          ((or (equal (find-section) 27) (equal title "More On Invert")) (execute-command :click :first-mistake)
           (execute-command :click :back)
           (update-state)
           (cons (list   'isa 'verbage 'command 'read-instructions 'result 'op27-1)
                 instructions27))
          (t (execute-command :click :first-mistake)
             (execute-command :click :back)
             (update-state)
             (list (list   'isa 'verbage 'command 'try-again))))))

(defun retrieve-label (index) 
  (no-output (chunk-slot-value-fct (car (buffer-chunk label)) index)))

(defun chunk-slot-value-fct (chunk-name slot-name)
  "Return the value of a slot for the named chunk"
  (let ((c (get-chunk chunk-name)))
    (when c
      (if (valid-slot-name slot-name (act-r-chunk-chunk-type c))
          (chk-slot-value c slot-name) nil))))


;; Modified, AS, Oct 13, 2005
;; Added a format directive for debugging purposes.
;; Just to control if the model can properly follow the sections
;;
(defun find-section () 
  ;(format t "~%Found section: ~A" *section*)
  *section*)

(defun update-state () 
  (cond (*assess-success* (setf *assess-success* nil) 
                          (mod-focus step assess)))
  (create-new-buffer-chunk 'screen (list 'isa 'screen 'state (perceive-state) 'hint (perceive-hint) 'instructions nil
                                         'error (perceive-error) 'equation (perceive-equation)))
  (setf *state* nil))


;; Modified, AS, Oct 13, 2005
;; Added two format instruction for debugging purposes
;;
(defun perceive-instructions ()
  ;(format t "Called perceiving instructions")
  (let* ((task (second (assoc ':title (second (assoc ':instructions (dump-screen))))))
         (instruction (cond ((equal task "More On Invert") '((junk isa operator pre junk)))
                            ((= (find-section) 11) instructions11)
                            ((= (find-section) 12) instructions12)
                            ((= (find-section) 17) Instructions17)
                            ((= (find-section) 26) instructions26)
                            ((= (find-section) 27) (if (and (cdr *instructions*)
                                                            (eq (car (second *instructions*)) 'junk)) 
                                                     (setf *instructions* nil))
                             Instructions27)
                            ((= (find-section) 31) instructions31)
                            ((= (find-section) 34) instructions34)
                            ((= (find-section) 43) instructions43))))
    ;(print instruction)  ;; for debug
    ;; Modified, AS, Oct20, 2005
    ;; Instrcutions need to be filtered because operators are tagged
    ;;
    (setf instruction (extract-proper-operators instruction))
    (cond (*start* (setf *start* nil)
                   (setf *instructions*  (cons task instruction))
                   'new)
          ((and (equal task "Invert Special Case")
                (null *instructions*))
           (setf *instructions*  (cons task instruction)) 'change)
          (instruction (if (not (equal (car *instructions*) task))
                         (setf *instructions*  (cons task instruction)))
                       (if (cdr *instructions*) t)))))


(defun find-plus-box (arg)
   (declare (ignore arg))
  (do ((choices (reverse (cddr (assoc ':equation (dump-screen)))) (cdr choices)))
      ((null choices) nil)
    (if (and (listp (second (car choices))) (member (second (second (car choices))) '(+ -))
             (find-times-below (caar choices)))
      (return (list 'isa 'box 'val (caar choices))))))

(defun find-matching-boxes (arg)
   (declare (ignore arg))
  (do ((choices (reverse (cddr (assoc ':equation (dump-screen)))) (cdr choices)))
      ((null choices) nil)
    (if (and (listp (second (car choices))) 
             (or (numberp (first (second (car choices)))) (numberp (third (second (car choices)))))
             (matching-below (caar choices)))
      (return (list 'isa 'box 'val (caar choices))))))



(defun matching-below (box)
  (let* ((set (cddr (assoc ':equation (dump-screen))))
        (target (assoc box set))
        (op (second (second target))))
  (do ((choices set (cdr choices)))
      ((null choices) nil)
    (let ((expr (second (car choices))))
      (if (and (listp expr) 
               (or (numberp (first expr)) (numberp (third expr)))
               (or (equal (second expr) op) (equal (second expr) (inverse-operator op)))
               (or (equal (first expr) box) (equal  (third expr) box)))
        (return (list  'isa 'box 'val (caar choices))))))))

(defun find-collect-boxes (arg)
   (declare (ignore arg))
   (find-collect-boxes-help (cddr (assoc ':equation (dump-screen)))))

(defun find-collect-boxes-help (choices)
  (let* ((x-box (car (find :x choices :key 'second)))
        (first (find-it x-box choices))
        (second (find-it x-box (remove first choices :test 'equal))))
    (if (and x-box first second (not (eq first second)))
      (cond ((and (times-test (second first)) (times-test (second second))) (merge-collect (first first) (first second) choices))
            ((times-test (second first))(merge-collect (first first) x-box choices))
            ((times-test (second second))(merge-collect x-box (first second) choices))))))


(defun times-test (box)
  (and (eq (second box) '*)
       (numberp (first box))))


(defun merge-collect (a b choices)
  (let* ((a-child (find-plus a choices))
         (b-child (find-plus  b choices)))
    (cond ((and (member (second (second a-child)) '(+ -))(member (second (second b-child)) '(+ -))) 
      (cond ((equal a-child b-child) (list 'isa 'box 'val (car a-child))))
      (cond ((or (numberp (car (second a-child)))(numberp (third (second a-child))))
             (setf a-child (find-plus (car a-child) choices))))
      (cond ((or (numberp (car (second b-child)))(numberp (third (second b-child))))
             (setf b-child (find-plus (car b-child) choices))))
      (if (and (member (second (second a-child)) '(+ -)) (equal a-child b-child))
        (list 'isa 'box 'val (car a-child)))))))

(defun find-it (x lis)
  (do ((temp lis (cdr temp)))
      ((null temp) nil)
    (if (and (listp (second (car temp)))
             (or (eq x (first (second (car temp))))  (eq x (third (second (car temp)))))) (return (car temp)))))

(defun find-plus (x lis)
  (do ((temp lis (cdr temp)))
      ((null temp) nil)
    (if (and (listp (second (car temp))) (member (second (second (car temp))) '(+ -))
             (or (eq x (first (second (car temp))))  (eq x (third (second (car temp)))))) (return (car temp)))))


(defun inverse-operator (op)
  (case op
    (+ '-)
    (- '+)
    (* '/)
    (/ '*)))


(defun find-three (arg)
   (declare (ignore arg))
  (do ((choices (cdr (assoc ':equation (dump-screen))) (cdr choices)))
      ((null choices) nil)
    (if (equal (cdar choices) '((NIL NIL NIL) (:BOX-EDITABLE))) (return (list 'isa 'box 'val (caar choices))))))

(defun find-threea (arg)
   (declare (ignore arg))
  (do ((choices (reverse (cdr (assoc ':equation (dump-screen)))) (cdr choices)))
      ((null choices) nil)
    (if (equal (cdar choices) '((NIL NIL NIL) (:BOX-EDITABLE))) (return (list 'isa 'box 'val (caar choices))))))


(defun input-number (arg)
   (declare (ignore arg)) 
  (let ((lis  (assoc ':equation (dump-screen))))
    (if (> (length lis) 2)
    (do ((temp lis (cdr temp)))
        ((null temp) nil)
      (if (and (listp (car temp)) (numberp (second (car temp))))
        (return (list 'isa 'box 'val (caar temp))))))))

(defun find-bottom-equal (arg)
   (declare (ignore arg)) 
  (let* ((lis  (cdr (assoc ':equation (dump-screen))))
         (contents (second (car lis))))
    (if (and (listp contents) (not (numberp (first contents))) 
             (not (numberp (third contents))) (eq (second contents) '=))
      (list 'isa 'box 'val (caar lis)))))

(defun green-little-box-p ()
  (do ((temp (cdr (assoc :equation (dump-screen))) (cdr temp)))
      ((null temp) nil)
    (cond ((or (equal (last (car temp)) '((:ARG1-EDITABLE)))
               (equal (last (car temp)) '((:ARG2-EDITABLE)))
               (equal (last (car temp)) '((:ANSWER-EDITABLE)))) (return (caar temp))))))

(defun green-little-box ()
  (do ((temp (cdr (assoc :equation (dump-screen))) (cdr temp)))
      ((null temp) nil)
    (cond ((or (equal (last (car temp)) '((:ARG1-EDITABLE)))
               (equal (last (car temp)) '((:ARG2-EDITABLE)))
               (equal (last (car temp)) '((:ANSWER-EDITABLE))))
           (return (if (not (listp (second (car temp)))) (second (car temp))
                       (if (numberp (first (second (car temp)))) 
                                (if (listp (third (second (car temp)))) (car (third (second (car temp)))) (third (second (car temp))))
                                (if (listp (first (second (car temp)))) (car (first (second (car temp)))) (first (second (car temp)))))))))))

(defun type-in-box-p ()
  (second (assoc :INPUT-DIALOG (dump-screen))))

(defun BOX1-CLICKED-P ()
  (let* ((box (bottom-number-box))
        (choices (cdr (assoc :equation (dump-screen))))
        (status (assoc box choices)))
    (equal (car (third status)) :SELECTED)))

(defun BOX2-CLICKED-P ()
  (let* ((box (BOX-ABOVE-BOTTOM))
        (choices (cdr (assoc :equation (dump-screen))))
        (status (assoc box choices)))
    (equal (car (third status)) :SELECTED)))

(defun boxes-reordered-p ()
  (let ((temp (cdr (assoc :equation (dump-screen)))))
    (and (member '(:OP-EDITABLE) temp :key 'third :test 'equal)
         (member '(:BOX-EDITABLE) temp :key 'third :test 'equal)
         (member '(:ARG-OP-EDITABLE) temp :key 'third :test 'equal))))

(defun boxes-combined-p ()
  (let ((temp (cdr (assoc :equation (dump-screen)))))
    (and (member '(:OP-EDITABLE) temp :key 'third :test 'equal)
         (member '(:BOX-EDITABLE) temp :key 'third :test 'equal)
         (not (member '(:ARG-OP-EDITABLE) temp :key 'third :test 'equal)))))


(defun boxes-inverted-p ()
  (do ((temp (cdr (assoc :equation (dump-screen))) (cdr temp)))
      ((null temp) nil)
    (cond ((or (equal (last (car temp)) '((:BOX-EDITABLE)))) (return (caar temp))))))

(defun free-screen-error ()
  (setf *ignore-error* t))

(defun perceive-error ()
  (let ((text (second (assoc :text (dump-screen)))))
   (and (not *ignore-error*) (or (and (listp text) (equal (car text) "Your answer is incorrect."))
                                 (equal text "Your answer is incorrect.
Use the <- and -> buttons (or the left and right arrow keys)
and the First Mistake button
to review your work and correct the mistake.")
                                 (equal text "Your answer is incorrect.
Use the <- and -> buttons (or the left and right arrow keys)
to review your work and correct the mistake."  )))))


(defun perceive-hint ()
;  (setf *hint* nil)
  (if  (and (assoc :equation (dump-screen))
            (second (assoc ':text (dump-screen))))
    t
    nil))


(defun perceive-equation ()
  (if  (assoc :equation (dump-screen))
    t
    nil))


(defun boxes-subtract-p ()
  (and (member '(:OP-EDITABLE) (cdr (assoc :equation (dump-screen))) :key 'third :test 'equal)
   (find-bottom-equal nil)))

 
(defun boxes-collect-p ()
  (and (member '(:BOX-EDITABLE) (cdr (assoc :equation (dump-screen))) :key 'third :test 'equal)
   (find-collect-boxes-help (cddr (assoc ':old-equation (dump-screen))))))

(defun test-lone-number (arg)
   (declare (ignore arg))
  (let* ((chunk (new-name-fct 'box))
         (bottom (second (assoc ':equation (dump-screen)))))
    (if (numberp (second bottom))
      (list chunk 'isa 'box 'val 'success))))

(defun dead-endp ()
  (if (not (or (just-numbers nil) (test-lone-number nil) (input-number nil) (find-bottom nil)))
    (not (find-below (fourth (find-plus-box nil))))))


(defun check-errorp () 
  (equal (car (last (check-error nil))) 'error))


(defun continuep () 
  (equal (second (assoc :text (dump-screen)))
         "Take a break. Click Continue when you're ready to go on."))

(defun red-box-p ()
  (do ((temp (cdr (assoc :equation (dump-screen))) (cdr temp)))
      ((null temp) nil)
    (cond ((and (not (and (listp (second (car temp))) (equal (second (second (car temp))) '=)))
                (equal (caar (last (car temp))) ':SELECTED)) (return (caar temp))))))

(defun read-number (arg) 
  (read-from-box arg 'numberp))

(defun read-op (arg) 
  (read-from-box arg 'operatorp))

(defun READ-SIGNED-NUMBERA (arg) 
  (let* ((expr (second (assoc arg (cdr (assoc ':old-equation (dump-screen)))))))
    (list 'isa 'box 'val(cond ((numberp (first expr)) (first expr))
                              ((eq (second expr) '+) (third expr))
                              (t (- (third expr)))))))

(defun read-signed-number (arg1 arg2) 
  (let* ((number1 (first (second (assoc arg1 (cdr (assoc ':old-equation (dump-screen)))))))
         (number (if number1 number1 1))
         (box2 (second (assoc arg2 (cdr (assoc ':old-equation (dump-screen))))))
         (op (if (or (eq arg1 (third box2))
                     (and (numberp (first box2)) (equal arg1 'empty)))
               (second (second (assoc arg2 (cdr (assoc ':old-equation (dump-screen)))))) '+))
         (val (if (eq op '-) (- number) number)))
     (list 'isa 'box 'val val)))

(defun read-from-box (box predicate)
  (let* ((contents (second (assoc box (cdr (assoc ':old-equation (dump-screen))))))
         (number  (cond ((apply predicate (list (first contents))) (first contents))
                       ((apply predicate (list (second contents))) (second contents))
                       ((apply predicate (list (third contents))) (third contents)))))
    (list 'isa 'box 'val number)))

;; Returns whether a symbol is an operator
;;
(defun operatorp (arg) 
  (member arg '(+ - / *)))
           

(defun read-op1 (box1 box2)
  (let* ((exp1 (second (assoc box1 (cdr (assoc ':old-equation (dump-screen))))))
         (exp2 (second (assoc box2 (cdr (assoc ':old-equation (dump-screen))))))
         (op1 (second exp1))
         (op2 (second exp2))
         (term (third exp1)))
    (if  (or (not (numberp term))
             (and (eq op1 '*) 
                  (member op2 '(+ -))))
      (list 'isa 'box 'val op2)
      (if (equal op1 op2)
        (list 'isa 'box 'val (if (member op2 '(+ -)) '+ '*))
        (list 'isa 'box 'val (if (member op2 '(+ -)) '- '/))))))

(defun read-op2 (box1 box2)
  (let* ((exp1 (second (assoc box1 (cdr (assoc ':old-equation (dump-screen))))))
         (exp2 (second (assoc box2 (cdr (assoc ':old-equation (dump-screen))))))
         (op1 (second exp1))
         (op2 (second exp2))
         (term1 (car exp1))
         (term2 (car exp2)))
    (if (not (and (numberp term1) (numberp term2)))
      (list 'isa 'box 'val op1)
      (if (equal op1 op2)
      (list 'isa 'box 'val (if (member op1 '(+ -)) '+ '*))
      (list 'isa 'box 'val (if (member op1 '(+ -)) '- '/))))))

(defun assign-left (box2)
  (let* ((desc (second (assoc box2 (cdr (assoc ':old-equation (dump-screen)))))))
    (cond ((numberp (car desc)) (list  'isa 'box 'val box2))
          (t  (list 'isa 'box 'val (car desc))))))

(defun assign-right (box2)
  (let* ((desc (second (assoc box2 (cdr (assoc ':old-equation (dump-screen)))))))
    (cond ((numberp (car desc)) (list 'isa 'box 'val (third desc)))
          (t (list 'isa 'box 'val box2)))))

(defun check-invert-op (box)
  (let* ((contents (second (assoc box (cdr (assoc ':old-equation (dump-screen)))))))
    (if (and (numberp (car contents)) (equal (second contents) '-)) 
      (list 'isa 'box 'val 'failure)
      (list 'isa 'box 'val 'success))))

(defun evaluable (lis)
  (and (listp lis) (or (and (equal (second lis) '*) (eq (first lis) 0))
                       (and (numberp (first lis))
                            (member (second lis) '(* / + -)) (numberp (third lis))))))

(defun just-numbers (arg)
   (declare (ignore arg)) 
  (let ((lis  (assoc ':equation (dump-screen))))
    (do ((temp lis (cdr temp)))
        ((null temp) nil)
      (if (and (listp (car temp)) (evaluable (second (car temp)))) 
        (return (list 'isa 'box 'val (first (car temp))))))))

(defun perceive-state ()
  (cond (*state*)
        ((and (Box1-Clicked-p)(Box2-Clicked-p))
         (if (equal (bottom-number-box) (car (second (assoc :equation (dump-screen))))) 'box12-clicked-invert 
             (if (find-collect-boxes nil) 'boxes-to-combine
                 (if (find-matching-boxes nil) 'box12-clicked-combine 'box12-clicked-reorder))))
        ((Box2-Clicked-p) 'Box2-Clicked)
        ((Box1-Clicked-p) 'Box1-Clicked)
        ((red-box-p) 'red-box)
        ((and (boxes-collect-p)(type-in-box-p) (Boxes-Inverted-p)) 'green-clicked-collect )
        ((and (boxes-reordered-p)(type-in-box-p) (Boxes-Inverted-p)) 'green-clicked-reorder )
        ((and (boxes-combined-p)(type-in-box-p) (Boxes-Inverted-p)) 'green-clicked-combine )
        ((and (type-in-box-p) (Boxes-Inverted-p)) 
         (nth *input-length* '(green-clicked green-clicked1 green-clicked2 green-clicked3)))
        ((boxes-subtract-p) 'boxes-subtracted)
        ((boxes-collect-p) 'boxes-collected)
        ((boxes-combined-p) 'boxes-combined)
        ((boxes-reordered-p) 'boxes-reordered)
        ((Boxes-Inverted-p) 'Boxes-Inverted)
        ((type-in-box-p) (if (variable-subp) 'type-in-boxa 'type-in-box))
        ((green-little-box-p) (if (variable-subp) 'green-little-boxb 'green-little-boxa))
        ((continuep) 'continue)
        ((just-numbers nil) 'box-to-evaluate)
        ((find-bottom-equal nil) 'bottom-equal)
        ((find-collect-boxes nil) 'collect-boxes)
        ((find-matching-boxes nil) 'matching-boxes)
        ((dead-endp) 'dead-end)
        ((input-number nil) 'input-number)
        ((and (find-bottom nil) (< (length (assoc :equation (dump-screen))) 4)) 'done)
        ((find-bottom nil) 'bottom-number)
        ((find-plus-box nil) 'plus-box)
        ((< (length (assoc :equation (dump-screen))) 3) 'done)
        (t 'start)))

(defun evaluable-box () (fourth (or (just-numbers nil)(input-number nil))))
(defun bottom-number-box () (fourth (find-bottom nil)))
(defun box-above-bottom () (fourth (find-above (bottom-number-box))))

(defun variable-subp ()
  (do ((temp (cdr (assoc ':old-equation (dump-screen))) (cdr temp)))
      ((null temp) nil)
    (if (numberp (second (car temp))) (return t))))



(defun find-twoa (arg)
   (declare (ignore arg))
  (do ((choices (cdr (assoc ':equation (dump-screen))) (cdr choices)))
      ((null choices) nil)
    (if (equal (caddar choices) '(:OP-ARG-EDITABLE)) (return (list 'isa 'box 'val (caar choices))))))

(defun find-oneb (arg)
   (declare (ignore arg))
  (do ((choices (cdr (assoc ':equation (dump-screen))) (cdr choices)))
      ((null choices) nil)
    (if (equal (caddar choices) '(:ARG2-EDITABLE)) (return (list 'isa 'box 'val (third (cadar choices)))))))

(defun find-onea (arg)
   (declare (ignore arg))
  (do ((choices (cdr (assoc ':equation (dump-screen))) (cdr choices)))
      ((null choices) nil)
    (if (equal (caddar choices) '(:OP-EDITABLE)) (return (list 'isa 'box 'val (second (cadar choices)))))))

(defun find-one (arg)
   (declare (ignore arg))
  (do ((choices (reverse (cdr (assoc ':equation (dump-screen)))) (cdr choices)))
      ((null choices) nil)
    (if (equal (caddar choices) '(:OP-EDITABLE)) (return (list 'isa 'box 'val (second (cadar choices)))))))

(defun find-left* (bot)
  (let* ((expr (second (assoc bot (cdr (assoc ':equation (dump-screen))))))
         (left (if (not (numberp (first expr))) (first expr) (third expr)))
         (left-box (assoc left (cdr (assoc ':equation (dump-screen)))))
         (left-expr (second left-box)))
    (if (listp left-expr)
      (if (eq (second left-expr) '*) (list  'isa 'box 'val left)
          (find-left* left))
      (list  'isa 'box 'val 'failure))))

(defun find-left+ (bot)
  (let* ((left (first (second (assoc bot (cdr (assoc ':equation (dump-screen)))))))
         (left-box (assoc left (cdr (assoc ':equation (dump-screen)))))
         (left-expr (second left-box)))
    (if (listp left-expr)
      (if (member (second left-expr) '(+ -)) (list  'isa 'box 'val (first left-box))
          (list  'isa 'box 'val 'failure))
      (list  'isa 'box 'val 'failure))))

(defun find-right* (bot)
  (let* ((expr (second (assoc bot (cdr (assoc ':equation (dump-screen))))))
         (right (if (not (numberp (third expr))) (third expr) (first expr)))
         (right-box (assoc right (cdr (assoc ':equation (dump-screen)))))
         (right-expr (second right-box)))
    (if (listp right-expr)
      (if (eq (second right-expr) '*) (list  'isa 'box 'val (first right-box))
          (find-right* (first right-box)))
      (list  'isa 'box 'val 'failure))))

(defun find-right+ (bot)
  (let* ((right (third (second (assoc bot (cdr (assoc ':equation (dump-screen)))))))
         (right-box (assoc right (cdr (assoc ':equation (dump-screen)))))
         (right-expr (second right-box)))
    (if (listp right-expr)
      (if (member (second right-expr) '(+ -)) (list  'isa 'box 'val (first right-box))
          (list  'isa 'box 'val 'failure))
      (list  'isa 'box 'val 'failure))))

(defun find-bottom (arg)
   (declare (ignore arg))
  (let* ((expr (cdr (assoc ':equation (dump-screen))))
         (bottom (car expr))
         (contents (second bottom))
         (box (first bottom))
         (above (car (last (find-above box))))
         (rest (second (assoc above expr))))
    (if (and (listp contents) (or (numberp (first contents)) (numberp (third contents)))
             (listp rest) (or (numberp (first rest)) (numberp (third rest))))             
      (list 'isa 'box 'val box))))

(defun find-number-old (arg)
  (let* ((expr (cdr (assoc ':old-equation (dump-screen))))
         (bottom (assoc arg expr))
         (contents (second bottom)))
    (if (not (listp contents)) (list 'isa 'box 'val contents)
        (if (numberp (first contents)) 
          (list 'isa 'box 'val (first contents))
          (list 'isa 'box 'val (third contents))))))

(defun find-above (bot)
  (let* ((expr (second (assoc bot (cdr (assoc ':equation (dump-screen))))))
         (above (if (listp expr) (if (numberp (car expr)) (third expr) (car expr)))))
    (list  'isa 'box 'val above)))

(defun find-operator-above (above)
  (let* ((expr (second (assoc above (cdr (assoc ':old-equation (dump-screen)))))))
    (list  'isa 'box 'val (second expr))))

(defun find-right-above (bot)
  (let* ((expr (second (assoc bot (cdr (assoc ':equation (dump-screen)))))))
    (list  'isa 'box 'val (third expr))))

(defun find-times-below (box)
  (do ((choices (cddr (assoc ':equation (dump-screen))) (cdr choices)))
      ((null choices) nil)
    (let ((expr (second (car choices))))
      (if (and (listp expr) (equal (second expr) '*) 
               (or (equal (first expr) box) (equal  (third expr) box)))
        (return (list  'isa 'box 'val (caar choices)))))))

(defun find-below (box)
  (do ((choices (cddr (assoc ':equation (dump-screen))) (cdr choices)))
      ((null choices) nil)
    (let ((expr (second (car choices))))
      (if (and (listp expr) 
               (or (equal (first expr) box) (equal  (third expr) box)))
        (return (list  'isa 'box 'val (caar choices)))))))

(defun find-green (arg)
   (declare (ignore arg))
  (let* ((box (caar (member '(:BOX-EDITABLE) (cdr (assoc ':equation (dump-screen))) 
                            :key 'third :test 'equal))))
    (if box (list 'isa 'box 'val box))))

(defun green-box () (fourth (find-green nil)))



(defun find-matching (lis arg)
  (do ((temp lis (cdr temp)))
      ((null temp) nil)
    (let ((state (or (chunk-slot-value-fct (caar temp) 'pre) (get-value (car temp)  'pre))))
    (cond ((not state) (return (car temp)))
          ((equal state arg) (return (car temp)))))))


;; The RESET function
;;
(defun reset-verbal-2 (instance)
  (declare (ignore instance))
  (specify-compilation-buffer-type verbal perceptual))


;; VERBAL MODULE DEFINITION
;; The formal definition of the VERBAL module
;;
(define-module-fct 'verbal
  '(verbal) 
  nil
  :version "0.2a"
  :documentation "An input model for reading text"
  :query 'verbal-query
;  :reset '(nil reset-verbal-2)
  :request 'request-verbal-chunk)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; THE conceptual MODULE
;;; The conceptual module represents the output of comprehension
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *conceptual-busy* nil)   ;; Whether the module is busy (e.g., executing a command) or not.
(defvar       *conceptual-error*)      ;; THe error state of the module  


(defun conceptual-query (instance buffer-name slot value)
  (declare (ignore buffer-name instance))
  (case slot
    (state
     (case value
       (busy *conceptual-busy*)
       (free (not *conceptual-busy*))
       (empty (equal '(nil) (no-output (buffer-chunk conceptual))))
       (error *conceptual-error*)
       (t (print-warning "Unknown state query ~S to goal module" value)
          nil)))))

(defun request-conceptual-chunk (instance buffer-name chunk-spec)
   (declare (ignore instance))
  (let* ((specs  (chunk-spec-to-chunk-def chunk-spec))
         (delay .5))
    (setf *conceptual-busy* t)  (setf *conceptual-error* nil)
    (cond (specs (schedule-event-relative delay 'create-chunk 
                                                :params (list buffer-name specs '*conceptual-busy*)))
          (t (schedule-event-relative delay 'create-conceptual-error 
                                      :module buffer-name
                                      :priority -100)))))
(defun reset-conceptual-2 (instance)
  (declare (ignore instance))
  (specify-compilation-buffer-type conceptual goal))


(define-module-fct 'conceptual
  '(conceptual) 
  nil
  :version "0.2a"
  :documentation "A model for for representing understanding"
  :query 'conceptual-query
  :reset '(nil reset-conceptual-2 )
  :request #'goal-style-request)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; THE DIGITAL MODULE
;;; The digital module is an output module: Its requests are interpreted as
;;; finger movements to be executed on the tutor panels.
;;; 
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *digital-busy* nil)   ;; Whether the module is busy (e.g., executing a command) or not.
(defvar       *digital-error*)      ;; THe error state of the module  

;; The QUERY function
;; The module is FREE when *DIGITAL-BUSY* is NIL, BUSY otherwise.
;; It is in Error state iff *DIGITAL-ERROR* is T.
;;
(defun digital-query (instance buffer-name slot value)
  (declare (ignore buffer-name instance))
  (case slot
    (state
     (case value
       (busy *digital-busy*)
       (free (not *digital-busy*))
       (error *digital-error*)
       (t (print-warning "Unknown state query ~S to goal module" value)
          nil)))))


;; The REQUEST function
;;
;; This functions handles the request specification as a command to execute 
;; an output motor function on the screen.  the motor functions is usually a
;; a mouse or a key press---hence the name 'digital'.
;;
;; Before the execution, the module is set to BUSY and its error state is NIL.
;; The course of execution wiull reset this value properly.
;;
;; As usual, execution is achieved by scheduling an event (with a specified delay)
;; in the ACT-R scheduler
;;
(defun request-digital-chunk (instance buffer-name chunk-spec)
   (declare (ignore instance))
   (let* ((specs  (chunk-spec-to-chunk-def chunk-spec))
          (arg (get-value specs 'item))
          (state (get-value specs 'state))
          (description
           (case (get-value specs 'command) 
             (click     (note-state state)
                        (encode-click arg))
             (submit    (note-state state) 
                        (type-string "" t))
             (copy-box  (note-state state)
                        (type-string (copy-box arg) t))
             (enter      (note-state state) (setf *input-length* 0)
                        (type-string (format nil  "~a" (or (retrieve-label arg) arg) ) t))
             (key (incf *input-length*) (type-string (format nil  "~a" arg) nil))))
          (delay (* (length description) 0.5)))
     ;;
     ;; Sets the module state as BUSY and without ERROR,
     ;; Until the scheduled events are executed
     ;;
     (setf *digital-busy* t)
     (setf *digital-error* nil)
     ;;
     ;; Executions: Events are scheduled according to the value of the DESCRIPTION
     ;; local var. When not-nil, it is interpreted as a command. If NIl, an error state is created
     ;;
     (cond (description (schedule-event-relative delay 'perform-digital
                                                 :module buffer-name
                                                 :priority -100
                                                 :params (list description)))
           (t (schedule-event-relative .3 'create-digital-error 
                                       :module buffer-name
                                       :priority -100)))))

;; COPY-BOX
;; Original JRA code. 
;;
(defun copy-box (arg) 
  (do ((temp (second (assoc (retrieve-label arg) (cdr (assoc :old-equation (dump-screen))))) (cdr temp))
       (result "" (cond ((numberp (car temp)) (concatenate 'string result (format nil  "~a" (car temp))))
                     ((member (car temp) '(+ - * /)) (concatenate 'string result (string (car temp))))
                     (t result))))
      ((null temp) result)))

;; TYPE-STRING
;; Types a string of test by creating a list
;; of EXECUTE-COMMAND functions whose arguments are the
;; characters of the string STR.
;; 
(defun type-string (str flag)
  (if (equal str "NIL") (setf str ""))
  (if str
  (do ((i 0 (1+ i))
       (result nil (cons (list 'execute-command ':type (aref str i)) result)))
      ((= i (length str)) 
       (if flag (reverse (cons (list 'execute-command ':submit nil) result))
             (reverse result))))))

;; This is probably the ugliest piece of code in the model
;; Basically, it uses a state name to set the value of
;; the *STATE* var to... the name itself. 
;;
(defun note-state (state)
  (case state
    (state13 (setf *state* 'state13))
    (state14 (setf *state* 'state14))
    (state18 (setf *state* 'state18))
    (state19 (setf *state* 'state19))
    (state21 (setf *state* 'state21))
    (state22 (setf *state* 'state22))
    (state30 (setf *state* 'state30))
    (state31 (setf *state* 'state31))
    (state34 (setf *state* 'state34))
    (state38 (setf *state* 'state38))
    (state42 (setf *state* 'state42))
    (state46 (setf *state* 'state46))
    (state50 (setf *state* 'state50))
    (state52 (setf *state* 'state52))
    (state53 (setf *state* 'state53))
    (state58 (setf *state* 'state58))
    (state60 (setf *state* 'state60))
    (state61 (setf *state* 'state61))
    (state32 (cond ((and (present-test 'box3)
                         (present-test 'box5)) 
                    (setf *state* 'both-combine))
                   ((present-test 'box3)
                    (setf *state* (if (find-two nil) 'left-combine1 'left-combine2)))
                   ((present-test 'box5)
                    (setf *state* (if (find-two nil) 'right-combine1 'right-combine2)))))))

(defun present-test (box)
  (not (equal 'empty 
              (retrieve-label box))))

(defun find-two (arg)
   (declare (ignore arg))
  (do ((choices (cdr (assoc ':equation (dump-screen))) (cdr choices)))
      ((null choices) nil)
    (if (equal (caddar choices) '(:ARG-OP-EDITABLE)) (return (list 'isa 'box 'val (caar choices))))))


;; The ENCODE-CLICK function
;; This functions creates a list that is an unavaluated EXECUTE-COMMAND
;; functions.  The command is a  click, and the target is the button whose
;; name is ITEM (if item is a string) or the associated value of a list,
;; if ITEM is nota  string.
;;
(defun encode-click (item)
  (setf *zvar* item)         ;; Temporarily sets the ZVAR var --- for debugging only
  ;(format t "~A" item)       ;; Prints the value of item on the screen --- for debugging only
  (list (list 'execute-command 
              ':click
              (if (stringp item) 
                (map-string item) item))))

;; MAP-STRING
;; Another ugly piece of code.  Basically, it maps a string to the action corresponding
;; to the button with the same string as a label. It is used to mapvalue of operators
;; and parts of instructions over the corresponding buttons to press. 
;; 
(defun map-string (item)
  (cond ((equal item "evaluate") ':evaluate)
        ((equal item "invert") ':invert)
        ((equal item "continue") ':continue)
        ((equal item "back") :back)
        ((equal item "reorder") :reorder)
        ((equal item "combine") :combine)
        ((equal item "subtract") :subtract)
        ((equal item "hint") :hint)
        ((equal item "submit") :submit)
        ;; Added for Tutor 2
        ((equal item "unwind") :unwind)
        ((equal item "distribute") :distribute)
        ((equal item "collect") :collect)
        ;; end 
        ((equal item "next") :next-problem)))


;; CREATE-DIGITAL-ERROR
;; Setes the parameters for a digital error
;; When queried, the module will result FREE and ERROR.
;;
(defun create-digital-error ()
  (setf *digital-busy* nil)
  (setf *digital-error* t))


;; PERFORM-DIGITAL
;; The function that (finally) updates the system state
;; after the execution of a digital command.
;;
(defun perform-digital (description)
  (setf *digital-busy* nil) 
  (if (equal (second (car description)) :click)
    (setf *ignore-error* nil))
  (if (equal (third (car description)) :NEXT-PROBLEM)
    (setf *assess-success* t))
  (ignore-errors (mapcar 'eval description))
  (clear-buffer 'screen)
  (schedule-event-relative .3 
                           'update-state  
                           :module 'screen
                           :priority -100))


;; The DIGITAL MODULE definition
;;
(define-module-fct 'digital
  '(digital) 
  nil
  :version       "0.2a"
  :documentation "A module for executing finger movements (cliks) on a screen" 
  :query         'digital-query
  :request       'request-digital-chunk)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; THE GRAPHICAL MODULE
;;; The graphical module is both and input and an output module for the system.
;;; It is mainly used to output operators by requesting the execution of an 
;;; operator and collecting the subsequent result.  Operator commands are
;;; passed thorough a BOX chunk type, and they are collected by creating
;;; a similar chunk type in a subsequent phase.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *graphical-busy* nil)
(defvar *graphical-error*)

;; The REQUEST Function
;; Gets an operator specification, and interprets it as command.
;; The execution of the command is stored in the description variable
;; 
(defun request-graphical-chunk (instance buffer-name chunk-spec)
   (declare (ignore instance))
   (let* ((specs       (chunk-spec-to-chunk-def chunk-spec))
          (arg         (get-value specs 'val))
          (arg1        (get-value specs 'val1))
          (function    (get-value specs 'command) )
          (description (if arg1 
                         (apply function (list arg arg1))
                         (apply function (list arg))))
          (delay       .5))         ;; End of let*
     ;;
     ;; Set the buffer busy and error-free before starting processing
     ;;
     (setf *graphical-busy*  t)
     (setf *graphical-error* nil)
     ;;
     ;; The schedules thre creation of a new chunk in the buffer, or the
     ;; setting up of an error condition, depending on the value of DESCRIPTION
     ;;
     ; (format t "~%Executing a request from a graphical module~%")
     (cond (description (schedule-event-relative delay 
                                                 'create-chunk 
                                                 :params (list buffer-name description '*graphical-busy*)))
           (t (schedule-event-relative delay 
                                       'create-graphical-error 
                                       :module buffer-name
                                       :priority -100)))))

;; CREATE-GRAPHICAL-ERROR
;; When an error occurs processing a request, the parameters are updated
;; so that module results Free, but responds positively to Error queries
;; 
(defun create-graphical-error ()
  (setf *graphical-busy* nil)
  (setf *graphical-error* t))


;; The QUERY Function
;; The module supports four (4) states: Busy if *graphical-busy* is non-nil,
;; Free otherwise, Error-Free if *graphical-error* is not-nil, and Error
;; otherwise.
;;
(defun graphical-query (instance buffer-name slot value)
  (declare (ignore buffer-name instance))
  (case slot
    (state
     (case value
       (busy       *graphical-busy*)
       (free       (not *graphical-busy*))
       (error-free (not  *graphical-error*))
       (error      *graphical-error*)
       (t (print-warning "Unknown state query ~S to goal module" value)
          nil)))))

;; THE RESET function
;;
(defun reset-graphical-2 (instance)
  (declare (ignore instance))
  (specify-compilation-buffer-type graphical perceptual))


;;; GRAPHICAL
;;; The graphical module is an input module
;;;
(define-module-fct 'graphical
  '(graphical) 
  nil
  :version "0.2a"
  :documentation "A module for retrieving visual information about the screen"
  :query 'graphical-query
  :reset '(nil reset-graphical-2)
  :request 'request-graphical-chunk)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; THE LABEL MODULE
;;; An input module for retrieving hint information.
;;; Note that label query is not specifid yet.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *label-busy* nil)

(defun label-query (instance buffer-name slot value)  
  (declare (ignore instance buffer-name slot value)) 
  nil)

(defun reset-label-2 (instance)
	(declare (ignore instance))
	(specify-compilation-buffer-type label goal))



;;; LABEL
;;; One of the input modules of the model. 
;;;
(define-module-fct 'label
 '(label) 
 nil
 :version "0.2a"
 :documentation "Used for tracking references"
 :query 'label-query
  :reset '(nil reset-label-2)
 :request #'goal-style-request)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; THE MATHEMATICAL MODULE
;;; An output module---the more questionable one. May be removed as
;;; further improvements are made. This module substitutes the calculator
;;; on the tutor. Basically, its requests are intepreted as operations
;;; to be performed over specified numbers.    
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defparameter *mathematical-busy* nil)           ;; Whether the module is busy (e.g., computing) or not
(defvar       *mathematical-error*)              ;; Whether an error has occurred in the model or not

;; MATHEMATICAL-QUERY
;; This is the request function of the mathematical buffer.
;; When a request is performed, if the command is EVALUATE, 
;; The arguments in the ARG1 slot are evaluated by the EVALUATE-BOX
;; function, which stores the value in the VAL slot of the requested
;; chunk
;;
(defun request-mathematical-chunk (instance buffer-name chunk-spec)
  (declare (ignore instance))
  (let* ((specs  (chunk-spec-to-chunk-def chunk-spec))
         (description
          (case (get-value specs 'command) 
            (Evaluate-box (evaluate-box  (get-value specs 'arg1)))))
         (delay .5))
    (setf *mathematical-busy* t)  (setf *mathematical-error* nil)
    (cond (description (schedule-event-relative delay 'create-chunk 
                                                :params (list buffer-name description '*mathematical-busy*)))
          (t (schedule-event-relative delay 'create-mathematical-error 
                                      :module buffer-name
                                      :priority -100)))))

(defun evaluate-box (box)
  (let* ((expr (second (assoc box (cdr (assoc ':equation (dump-screen))))))
         (val (cond ((atom expr) expr)
                    ((and (eq (second expr) '*) (or (eq (first expr) 0) (eq (third expr) 0))) 0)
                    ((and (numberp (first expr)) (numberp (third expr)))
                     (eval (list (second expr) (first expr) (third expr)))))))
    (list 'isa 'verbage 'arg1 box 'result val)))

(defun create-mathematical-error ()
  (setf *mathematical-busy* nil)
  (setf *mathematical-error* t))

;; MATHEMATICAL-QUERY
;; This is the query of the matematical buffer.
;; The buffer is busy whenever the variable *mathematical-busy* is T, and
;; Free otherwise.  The query returns Error whenever the *mathematical-error* 
;; parameter is T. Both of the paramters are handled by the mathematical request
;; function.
;; 
(defun mathematical-query (instance buffer-name slot value)
  (declare (ignore buffer-name instance))
  (case slot
    (state
     (case value
       (busy *mathematical-busy*)
       (free (not *mathematical-busy*))
       (error *mathematical-error*)
       (t (print-warning "Unknown state query ~S to goal module" value)
          nil)))))

;; The RESET function
;; Simply specifies the compilation buffer type.
;;
(defun reset-mathematical-2 (instance)
  (declare (ignore instance))
  (specify-compilation-buffer-type mathematical perceptual))

;; the MATHEMATICAL MODULE definition
;; As usual
;;
(define-module-fct 'mathematical
  '(mathematical) 
  nil
  :version "0.2a"
  :documentation "A module for performing fast mathematical computations"
  :query 'mathematical-query
  :reset '(nil reset-mathematical-2)
  :request 'request-mathematical-chunk)



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; THE SCREEN MODULE
;;; The most important visual input module. Contains info about the current screen.
;;; Input is gathers through the <DUMP-SCREEN> function, which is defined in the
;;; AL-model file.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun request-screen-chunk (instance buffer-name chunk-spec)  
  (declare (ignore instance buffer-name slot chunk-spec))
  nil)

(defun screen-query (instance buffer-name slot value)  
  (declare (ignore instance buffer-name slot value))
  nil)

(defun reset-screen-2 (instance)
	(declare (ignore instance))
	(specify-compilation-buffer-type screen goal))

;; SCREEN
;; 
(define-module-fct 'screen
  '(screen) 
  nil
  :version       "0.2"
  :documentation "The most important visual input module. Contains info about the current screen"
  :query #'goal-style-query
  :reset '(nil reset-screen-2)
  :request #'goal-style-request)


