;;;  -*- mode: LISP; Syntax: COMMON-LISP;  Base: 10 -*-
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Author      : John R. Anderson & Andrea Stocco
;;; Copyright   : (c) 2005 John R. Anderson
;;; Address     : Department of Psychology 
;;;             : Carnegie Mellon University
;;;             : Pittsburgh, PA 15213-3890
;;; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; 
;;; Filename    : AL-model.lisp
;;; Version     : 0.2a. Based on the previous "6box19" code by JRA.
;;; 
;;; Description : The Algebra model interacting with the CMU algebra tutor 
;;;
;;; Date        : Oct 15th, 2005.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Describes the Algebra model. 
;;; The model solves equations on a the tutor. The interaction with the tutor is
;;; managed by a set of model-specific buffers. 
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; HOW TO RUN THE CODE
;;;
;;; First: The model needs the Algebra Tutor by Shawn Betts---it can only 
;;; interact with it (sorry for that...).
;;;
;;; The model may run through all of the sections of the Foerster's book, or
;;; run only some of them.  It can run on *ALL* of the sections that are 
;;; currently available of the tutor system.  
;;; 
;;; To run the model through oll of them, you may simply call this functions:
;;;
;;; --> (full-test id verbose)
;;;
;;; Where ID is a number that will be used as Tutor's ID (to register model's
;;; actions on the log file) and VERBOSE is a flag indicating whether the ACT-R
;;; trace should be printed or not.
;;;
;;; This functions, like every other function described here, accepts two
;;; keyword args; They are :TUTOR and :REAL-TIME.  The first specifies which
;;; version of tutor will run with model, whethe version 1 or version 2. The
;;; accepted values are numbers 1 and 2, respectively. The default is 2.
;;; The second is the :REAL-TIME param of ACT-R 6, passed to ACT-R's RUN function.
;;; Defaults to NIL.
;;;
;;; To run only a specific section of the book, use this command:
;;;
;;; --> (test-problems chap sect id time verbose)
;;;
;;; Where CHAP is the chapter, SECT is the section, TIME is the amount of time
;;; the simulation should run (usually 1000s is sufficient).  ID and VERBOSE 
;;; aree the same params used in the FULL-TEST function.  This functions also
;;; accepts the :TUTOR and :REAL-TIME keyword args.
;;;
;;; Currently, the sections of the book that are modelled are the following:
;;; 
;;;    1.1    1.2    1.7
;;;    2.1    2.3    2.4    2.5    2.6    2.7
;;;    3.1    3.2    3.4
;;;    4.1    4.2    4.3
;;;
;;; You may need to preload knowledge intro the model to run a specific section.
;;; This can be done by using the prest function.
;;;
;;; --> (preset block tutor)
;;;
;;; Where BLOCK is one of the preset knowledge blocks, and TUTOR is the type
;;; of tutor you want to use (1 or 2, as always).  The predefined knowledge
;;; blocks are parameters in the code, and must be invoked by name.  The name
;;; reflects the section they must be used for.  There are six of them:
;;;
;;;    PRE1-7    PRE2-6    PRE2-7
;;;    PRE3-1    PRE3-2    PRE4-1
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; History     : Oct 20th, 2005. Added support for dinamic loading of operators
;;;               This means that finally the two tutos can be run from the
;;;               very same code, without bothering in commenting and 
;;;               uncommenting operators. The key to dynamic loading is the
;;;               addition of a tutor TAG to the operator definition.
;;;               What before was 
;;;
;;;                  '(OP26 ISA OPERATOR...) 
;;;
;;;               now is defined as an assoc-list of a tag and an operator
;;;                definition:
;;;
;;;                  '(<TAG> . (OP26 ISA OPERATOR)). Bacause the CDR of the assoc
;;;   
;;;               is a list, this notation is equivalent to:
;;;   
;;;                  '(<TAG> OP26 ISA OPERATOR)
;;;
;;;               A <TAG> may be any symbol indicating a specific kind of tutor,
;;;               or teh constant T, meaning that the particular operator can
;;;               be used on all of the tutors.
;;;               When an operator is requested (because a hint or an
;;;               instruction is read), the operators in the list are filtered
;;;               according to the value of *current-tutor*, detagged, and
;;;               finally loaded into DM.
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; To do's
;;;
;;; * Add a dual-tasking capability.
;;; * Add real arithmetic knowledge
;;; * Have it report some basic protocols   
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;; The default tutor. If not otherwise specified, the model
;; will run with the Tutor 2.
;; The Tutors are specified with an integer reflecting their
;; respective version, i.e.  "1" for tutor1 and "2" for Tutor2.
;;
(defparameter *default-tutor* 2)

;; The current tutor the model is dealing with.
;; Must be specified because some operators are slightly different across
;; the two versions of the tutor, and in deciding which of the two
;; versions to apply, the current tutor must be  taken into consideration.
;; (Alternatively, one may use different specifications of the *state*
;; parameter, or include the tutor version in it. This would probably be
;; a more elegant solution. However, in the future, when all of the
;; operators will be abstracted from instructions, the different tutors
;; will not be a concern).  
;;
(defparameter *current-tutor* *default-tutor*)

;; A function that returns which is the current tutor.
;; May be redundant, but in the future one may consider not relying of tutor-version
;; parameter, so that modifying the function would be an easier step to modify all
;; the model's behavior.
;;
(defun current-tutor ()
  *current-tutor*)

;; The dafault real-time parameter. This is the value that is passed as a default
;; to the RUN function in the :REAL-TIME keyword arg, unless otherwise specified.
;; 
(defparameter *default-real-time* nil)

;; Whether the model runs in real-time simulation (T) or not (NIL). Its is the
;; argument that is really passed to the :REAL-TIME param when calling the RUN
;; function.  Defaults to the value stored in the *DEFAULT-REAL-TIME* param. 
;;
(defparameter *real-time* *default-real-time*) 


(defparameter instructions11
  '((t . (find a box that can be evaluated))))
;    (t . (op11-2 isa operator action copy arg1 evaluated-results arg2 arrow post start command-type unknown))
;    (t . (op11-3 isa operator action becomes-a-number arg1 box arg2 screen  post lone-number command-type unknown))))

(defparameter instructions12
  '((t . (FIND A BOX AT TOP WITH JUST A NUMBER))))

(defparameter instructions17
  '((t . (FIND A FIRST BOX AT BOTTOM WITH JUST A NUMBER))))
;    (t . (op17-3 isa operator action work-on-boxes arg1 box1 arg2 box2 post all-numbers command-type unknown)))

(defparameter instructions26
  '((t . (op26-1 isa operator action find-matching-boxes  arg1 box1 arg2 screen post got-matching-box command-type find))
    (t . (op26-2 isa operator action find-below  arg1 box2 arg2 box1 post two-boxes command-type find pre got-matching-box))))

(defparameter instructions27 
  '((t . (op27-1 isa operator pre green-clicked action check-invert-op arg1 box2 arg2 op-special post op-normal  command-type find-choose))
    (t . (op27-2 isa operator pre op-normal action read-number  arg1 Box1 arg2 type-part post Arg1-Copied command-type find&act))
    (t . (op27-3 isa operator pre op-special action read-number  arg1 box2 arg2 type-part post arg1-copieda command-type find&act))
    (t . (op27-4 isa operator pre arg1-copieda action read-op arg1 box2 arg2 type-part post op2-copied command-type find&act))
    (t . (op27-5 isa operator pre op2-copied action read-number arg1 box1 arg2 type-part post invert-done command-type find&act))))

(defparameter instructions31
  '((t . (op31-1 isa operator action find-plus-box  arg1 box1 arg2 screen post got-plus-box command-type find))
    (t . (op31-2 isa operator action find-times-below  arg1 box2 arg2 box1 post two-boxes command-type find pre got-plus-box))))

(defparameter instructions34
  '((t . (op34-1 isa operator action find-collect-boxes  arg1 box1 arg2 screen post got-collect-box command-type bind&click))
    (t . (op34-2 isa operator action find-left*  arg1 box2 arg2 box1 post state24 command-type bind&click pre got-collect-box))
    (t . (op34-3 isa operator action find-left+  arg1 box3 arg2 box1 post state25 command-type bind&click pre state24))
    (t . (op34-4 isa operator action find-right*  arg1 box4 arg2 box1 post state26 command-type bind&click pre state25))
    (t . (op34-5 isa operator action find-right+  arg1 box5 arg2 box1 post state27 command-type bind&click pre state26))))

(defparameter instructions43
  '((t . (op43-1 isa operator action find-bottom-equal  arg1 box1 arg2 screen post got-bottom-equal-box command-type bind&click))
    (t . (op43-2 isa operator action find-right-above  arg1 box2 arg2 box1 post state56 command-type bind&click pre got-bottom-equal-box))))


(defparameter hint11a
  '((example ((t . (click (evaluable-box) arrow))))
    (instruction ((1 . (find a box that can be evaluated))
                  (2 . (find a operation that can be evaluated))                  
                  (1 . (then click the box))
                  (2 . (then click the operation))))))

(defparameter hint11b 
  '((example ((t . (click "evaluate" arrow))))
    (instruction ((t . (THEN CLICK THE BUTTON LABELED "evaluate"))))))


(defparameter hint11c 
  '((example ((t . (click (green-little-box) arrow))
              (t . (enter (get-number-evaluation) arrow))))
    (instruction ((1 . (then click the green box pointed to by the box))
                  (1 . (then find a answer by evaluating  the box))
                  (1 . (then enter the answer))
                  (2 . (then click the green box that has replaced the operation))
                  (2 . (then find a answer by evaluating  the operation))
                  (2 . (then enter the answer))))))

(defparameter hint12c1 
  '((example ((t . (click (green-little-box) arrow))
              (t . (enter (get-number-evaluation) arrow))))
    (instruction ((1 . (then click the green box pointed to by the box))
                  (1 . (then find a answer by evaluating  the box))
                  (1 . (then enter the answer))
                  (2 . (then click the green box that has replaced the variable))
                  (2 . (then enter the number that variable is equal to))))))


(defparameter hint11d 
  '((example ((t . (click "next" arrow))))
    (instruction ((t . (THEN CLICK THE BUTTON LABELED "next"))))))

(defparameter hint12a
  '((example ((t . (click (evaluable-box) arrow))))
    (instruction ((1 . (FIND A BOX AT TOP WITH JUST A NUMBER))
                  (1 . (then click the box))
                  (2 . (FIND A variable in a expression))
                  (2 . (then click the variable))))))

(defparameter hint17a 
  '((example ((t . (click (bottom-number-box) arrow))
              (t . (click (box-above-bottom) arrow))))
    (instruction ((1 . (FIND A FIRST BOX AT BOTTOM WITH JUST A NUMBER))
                  (1 . (THEN CLICK THE FIRST BOX))
                  (1 . (THEN CLICK THE SECOND BOX THAT IS ABOVE THE FIRST BOX))
                  (2 . (FIND A equal sign WITH JUST A NUMBER on one side))
                  (2 . (THEN CLICK THE equal sign))
                  (2 . (THEN CLICK THE operator that is nearest tHE equal sign ))))))

;; Modified, AS, Oct 20th, 2005.
;;
;; Now this hint has two different operators tagged for the two different tutors.
(defparameter hint17b
  '((example ((1 . (click "invert" arrow))
              (2 . (click "unwind" arrow))))
    (instruction ((1 . (THEN CLICK THE BUTTON LABELED "invert"))
                  (2 . (THEN CLICK THE BUTTON LABELED "unwind"))))))

(defparameter hint17c
  '((example ((t . (CLICK (GREEN-box) arrow))
              (t . (key (bottom-number) arrow))
              (t . (key (invert-operator-above-bottom) arrow))
              (t . (enter (number-above-bottom) arrow))))
    (instruction ((1 . (THEN CLICK THE NEW BOX THAT IS GREEN))
                  (1 . (THEN key THE number IN THE FIRST BOX))
                  (1 . (THEN find a operator IN THE second BOX))
                  (1 . (Then key the NEW OPERATOR which is the inverse of THE OPERATOR))
                  (1 . (THEN enter THE OTHER NUMBER IN THE SECOND BOX))
                  (2 . (THEN CLICK THE BOX THAT IS GREEN))
                  (2 . (THEN Key the number isolated by the equal sign))
                  (2 . (Then key the NEW OPERATOR which is the inverse of THE OPERATOR))
                  (2 . (Then Enter the number associated with the operator))))))

(defparameter new-hint (append (remove (third hint17c) hint17c) 
                               instructions27))


;; Modified: AS, Oct 12, 2005
;; Combine --> Collect
;;
(defparameter hint26a 
  '((1 . (op26-3 isa OPERATOR action Click arg1 "combine" arg2 string post state10 command-type acting))
    (2 . (op26-3 isa OPERATOR action Click arg1 "collect" arg2 string post state10 command-type acting))))

(defparameter hint26b
  '((t . (op26-4 isa OPERATOR  action find-three arg1 box-three arg2 screen post three-founda command-type find))
    (t . (op26-5 isa OPERATOR action Click arg1 Box-three arg2 variable post state11 command-type acting))
    (t . (op26-6 isa operator action assign-left arg1 left arg2 box2 post left-done  command-type find))
    (t . (op26-6a isa operator action assign-right arg1 right arg2 box2 post right-done  command-type find))
    (t . (op26-7 isa OPERATOR action read-number arg1 left  arg2 type-part post 3Arg1-Copieda command-type find&act))
    (t . (op26-8 isa OPERATOR action read-op1 arg1 box1 arg2 box2 post state12 command-type combine-act))
    (t . (op26-9 isa OPERATOR action read-number arg1 right  arg2 type-part post enter-three command-type find&act))
    (t . (op26-10 isa OPERATOR action type arg1 "" arg2 string post state13 command-type acting))
    (t . (op26-11 isa OPERATOR action find-one arg1 box-one arg2 screen post one-founda command-type find))
    (t . (op26-12 isa OPERATOR action Click arg1 Box-one arg2 variable post state14 command-type acting))
    (t . (op26-13 isa OPERATOR action read-op2 arg1 left arg2 right post state15 command-type combine-act))
    (t . (op26-14 isa OPERATOR action type  arg1 "" arg2 string post state16 command-type acting))))

(defparameter hint26c
  '((example ((t . (click "next" arrow))))
    (instruction ((t . (THEN CLICK THE BUTTON LABELED "next"))))))


;; Modified, AS, Oct 12, 2005
;; Added a twin hint, specifc for tutor 2:
;; where reorder--> distribute
;; Commented the previos hint (otherwise would be retrieved, causing troubles)
;;
(defparameter hint31a 
  '((1 . (op31-4a isa OPERATOR action Click arg1 "reorder" arg2 string post state10 command-type acting))
    (2 . (op31-4b isa OPERATOR action Click arg1 "distribute" arg2 string post state10 command-type acting))))

;;; HERE!!! ;;;

(defparameter hint31b
  '((t . (op31-5 isa OPERATOR action find-two arg1 box-two arg2 screen post two-found command-type find))
    (t . (op31-6 isa OPERATOR action Click arg1 Box-two arg2 variable post state17 command-type acting))
    (t . (op31-7 isa OPERATOR action read-number arg1 Box2 arg2 type-part post 2Arg1-Copied command-type find&act))
    (t . (op31-8 isa OPERATOR action type arg1 "*" arg2 string post state18 command-type acting))
;   (t . (op31-9 isa OPERATOR action type arg1 "" arg2 string contents post state18 command-type acting))
    (t . (op31-10 isa OPERATOR action find-three arg1 box-three arg2 screen post three-found command-type find))
    (t . (op31-11 isa OPERATOR action Click arg1 Box-three arg2 variable post state19 command-type acting))
    (t . (op31-12 isa OPERATOR action read-number arg1 Box2  arg2 type-part post 3Arg1-Copied command-type find&act))
    (t . (op31-13 isa OPERATOR action type-part arg1 "*" arg2 string post state20 command-type acting-part))
    (t . (op31-14 isa OPERATOR action read-number arg1 Box1  arg2 type post state21 command-type find&act))
;   (t . (op31-15 isa OPERATOR action type arg1 "" arg2 string post state21 command-type acting))
    (t . (op31-16 isa OPERATOR action find-one arg1 box-one arg2 screen post one-found command-type find))
    (t . (op31-17 isa OPERATOR action Click arg1 Box-one arg2 variable post state22 command-type acting))
    (t . (op31-18 isa OPERATOR action read-op arg1 Box1  arg2 type post start command-type find&act))))
;   (t . (op31-19 isa OPERATOR action type arg1 "" arg2 string post state23 command-type acting))))

(defparameter hint34a 
  '((1 . (op34-6 isa OPERATOR action Click arg1 "combine" arg2 string post state28 command-type acting))
    (2 . (op34-6b isa OPERATOR action Click arg1 "collect" post arg2 string state28 command-type acting))))

(defparameter hint34b
  '((t . (op34-7 isa OPERATOR  action find-three arg1 box-three arg2 screen post three-foundb command-type find))
    (t . (op34-8 isa OPERATOR action Click arg1 Box-three arg2 variable post state28 command-type acting))
    (t . (op34-9 isa OPERATOR action read-signed-number arg1 box2  arg2 box3 post 3Arg1-Copiedb command-type combine-act))
    (t . (op34-10 isa OPERATOR action read-op arg1 box1 arg2 type-part post state29 command-type find&act))
    (t . (op34-11 isa OPERATOR action read-signed-number arg1 box4  arg2 box5 post enter-threeb command-type combine-act))
    (t . (op34-12 isa OPERATOR action type arg1 "" arg2 string post state30 command-type acting))
    (t . (op34-13 isa OPERATOR action find-one arg1 box-one arg2 screen post one-foundb command-type find))
    (t . (op34-14 isa OPERATOR action Click arg1 Box-one arg2 variable post state31 command-type acting))
    (t . (op34-15 isa OPERATOR action type arg1 "*" arg2 string post state32 command-type acting))))

(defparameter hint34-right1
   '((t . (op34-17 isa OPERATOR pre right-combine1 action find-two arg1 box-two arg2 screen post two-foundb command-type find))
     (t . (op34-18 isa OPERATOR action Click arg1 Box-two arg2 variable post state34 command-type acting))
     (t . (op34-20 isa OPERATOR action read-number arg1 box5  arg2 type-part post state35 command-type find&act))
     (t . (op34-19 isa OPERATOR action read-op arg1 box1  arg2 type  post start command-type find&act))))

(defparameter hint34-left2
   '((t . (op34-22 isa OPERATOR pre left-combine2 action find-twoa arg1 box-two arg2 screen post two-foundc command-type find))
     (t . (op34-23 isa OPERATOR action Click arg1 Box-two arg2 variable post state38 command-type acting))
     (t . (op34-24 isa OPERATOR action read-op arg1 box3  arg2 type-part  post state39 command-type find&act))
     (t . (op34-25 isa OPERATOR action read-number arg1 box3  arg2 type post start command-type find&act))))

(defparameter hint34-right2
   '((t . (op34-27 isa OPERATOR pre right-combine2 action find-twoa arg1 box-two arg2 screen post two-foundd command-type find))
     (t . (op34-28 isa OPERATOR action Click arg1 Box-two arg2 variable post state42 command-type acting))
     (t . (op34-29 isa OPERATOR action read-op1 arg1 box5 arg2 box1 post state43 command-type combine-act))
     (t . (op34-30 isa OPERATOR action read-number arg1 box5  arg2 type post start command-type find&act))))

(defparameter hint34-left1
   '((t . (op34-32 isa OPERATOR pre left-combine1 action find-two arg1 box-two arg2 screen post two-founde command-type find))
     (t . (op34-33 isa OPERATOR action Click arg1 Box-two arg2 variable post state46 command-type acting))
     (t . (op34-34 isa OPERATOR action read-number arg1 box3  arg2 type-part post state47 command-type find&act))
     (t . (op34-35 isa OPERATOR action type arg1 "+" arg2 string post state48 command-type acting))))

(defparameter hint34-combine
   '((t . (op34-37 isa OPERATOR pre both-combine action find-threea arg1 box-three arg2 screen post three-foundc command-type find))
     (t . (op34-38 isa OPERATOR action Click arg1 Box-three arg2 variable post state50 command-type acting))
     (t . (op34-39 isa OPERATOR action read-signed-numbera arg1 box3  arg2 type-part post 3Arg1-Copiedc command-type find&act))
     (t . (op34-40 isa OPERATOR action read-op arg1 box1 arg2 type-part post state51 command-type find&act))
     (t . (op34-41 isa OPERATOR action read-signed-numbera arg1 box5  arg2 type post state52 command-type find&act))
     (t . (op34-43 isa OPERATOR action find-onea arg1 box-one arg2 screen post one-foundc command-type find))
     (t . (op34-44 isa OPERATOR action Click arg1 Box-one arg2 variable post state53 command-type acting))
     (t . (op34-45 isa OPERATOR action type arg1 "+" arg2 string post state54 command-type acting))))

(defparameter hint43a 
  '((t . (op43-3 isa OPERATOR action Click arg1 "subtract" arg2 string post state57 command-type acting))))

(defparameter hint43b 
  '((t . (op43-4 isa OPERATOR action find-one arg1 box-one arg2 screen post one-foundd command-type find))
    (t . (op43-5 isa OPERATOR action Click arg1 Box-one arg2 variable post state58 command-type acting))
    (t . (op43-6 isa OPERATOR action type arg1 "-" arg2 string post state60 command-type acting))
    (t . (op43-8 isa OPERATOR action find-oneb arg1 box-one arg2 screen post one-founde command-type find))
    (t . (op43-9 isa OPERATOR action Click arg1 Box-one arg2 variable post state61 command-type acting))
    (t . (op43-10 isa OPERATOR action type arg1 "0" arg2 string post state62 command-type acting))))

(defparameter pre-hint26a "First  is added to a box then  is added to that.So add  and  first, then add the box.Click the top green box and type   .Click the bottom green box and type .")

(defparameter pre-hint26b "First  is multiplied by a box then  is multiplied by that.So multiply  and  first, then multiply the box.Click the left green box and type   .Click the lower green box and type .")

(defparameter pre-hint31a "Click the Reorder button(or press the 'r' key).")

;;;;;;;;;;;;;;
;; Modified, AS, Oct 12th, 2005
;; Added another hint to handle the distribute button
;;
(defparameter pre-hint31a2 "Click the Distribute button(or press the 'd' key).")

(defparameter pre-hint31b "Both operands in the  operator box are multiplied by .Click the left green box and type  .Click the right green box and type   .Click the bottom green box and type .")

;;;;;;;;;;;;;;
;; Modified, AS, Oct 12th, 2005
;; Added another hint more appropriate for tutor 2.
;;
(defparameter pre-hint31b2 "Both operands inside the parenthesis are multiplied by .Click the left green box and type   .Click the right green box and type  .Click the bottom green box and type   .")

;; Modified, AS, Oct 13th, 2005
;;
;; Modified to be consistent with the instructions printed on the screen.
;;
;; Added prehint43b2 
;;
(defparameter pre-hint43b "The right part of the diagram is subtractedfrom the left part of the diagram. is left on the right side.Click the top green box and type .Click the bottom green box and type .")

;;
;; added
(defparameter pre-hint43b2 "The right side of the equation is subtractedfrom the left side of the equation. is left on the right side.Click the top green box and type .Click the bottom green box and type .")


(defparameter pre1-7 
  '((t . (op11-1 ISA OPERATOR PRE BOX-TO-EVALUATE ACTION JUST-NUMBERS ARG1 BOX ARG2 SCREEN POST ALL-NUMBERS COMMAND-TYPE FIND))
    (t . (op11-4 ISA OPERATOR PRE ALL-NUMBERS ACTION CLICK ARG1 BOX ARG2 variable POST STATE1 COMMAND-TYPE ACTING))
    (t . (op11-5 ISA OPERATOR PRE RED-BOX ACTION CLICK ARG1 "evaluate" ARG2 string POST STATE2 COMMAND-TYPE ACTING))
    (t . (op11-6 ISA OPERATOR PRE GREEN-LITTLE-BOXA ACTION FIND-BOX-POINTER ARG1 GREEN-BOX ARG2 BOX POST GREEN-LITTLE-BOXB COMMAND-TYPE FIND))
    (t . (op11-7 ISA OPERATOR PRE GREEN-LITTLE-BOXB ACTION CLICK ARG1 GREEN-BOX ARG2 variable POST STATE3 COMMAND-TYPE ACTING))
    (t . (op11-8 ISA OPERATOR PRE TYPE-IN-BOX ACTION EVALUATE ARG1 BOX ARG2 ANSWER POST ANSWER COMMAND-TYPE MATH))
    (t . (op11-9 ISA OPERATOR PRE ANSWER ACTION TYPE ARG1 ANSWER ARG2 variable POST STATE4 COMMAND-TYPE ACTING))
    (t . (op11-10 ISA OPERATOR PRE DONE ACTION CLICK ARG1 "next" ARG2 string POST STATE5 COMMAND-TYPE acting))
    (t . (ACTING-1 ISA OPERATOR PRE INPUT-NUMBER ACTION INPUT-NUMBER ARG1 BOX ARG2 SCREEN POST ALL-NUMBERS COMMAND-TYPE FIND))))

(defparameter pre2-6 (append pre1-7
   '((t . (op17-1 ISA OPERATOR PRE BOTTOM-NUMBER ACTION FIND-BOTTOM ARG1 BOX1 ARG2 SCREEN POST BOTTOM-BOX COMMAND-TYPE FIND))
     (t . (op17-2 ISA OPERATOR PRE BOTTOM-BOX ACTION FIND-ABOVE ARG1 BOX2 ARG2 BOX1 POST TWO-BOXES COMMAND-TYPE FIND))
     (t . (op17-5a ISA OPERATOR PRE TWO-BOXES ACTION CLICK ARG1 BOX1 arg2 variable post state6a COMMAND-TYPE ACTING))
     (t . (op17-5b ISA OPERATOR PRE BOX1-CLICKED ACTION CLICK ARG1 BOX2 arg2 variable post state6a COMMAND-TYPE ACTING))
     (1 . (OP17-6 ISA OPERATOR PRE BOX12-CLICKED-INVERT ACTION CLICK ARG1 "invert" ARG2 string POST STATE7 COMMAND-TYPE ACTING))
     ;
     ; Modified, AS, Oct 12th, 2005
     ; Operator substitution: Added one specific for Tutor 2
     ; AS, Oct 19, 2005: Added the tutor flog to the operators
     ;
     (2 . (OP17-6 ISA OPERATOR PRE BOX12-CLICKED-INVERT ACTION CLICK ARG1 "unwind" ARG2 string POST STATE7 COMMAND-TYPE ACTING))
     (t . (op17-7 ISA OPERATOR PRE BOXES-INVERTED ACTION FIND-GREEN ARG1 BOX ARG2 SCREEN POST GREEN-FOUND COMMAND-TYPE FIND)) 
     (t . (op17-8 ISA OPERATOR PRE GREEN-FOUND ACTION CLICK ARG1 BOX ARG2 variable POST STATE7 COMMAND-TYPE ACTING))
     (t . (op17-9 ISA OPERATOR PRE GREEN-CLICKED ACTION READ-NUMBER ARG1 BOX1 ARG2 TYPE-PART POST ARG1-COPIED COMMAND-TYPE FIND&ACT))
     (t . (op17-10 ISA OPERATOR PRE ARG1-COPIED ACTION READ-OP ARG1 OP ARG2 BOX2 POST READY-TO-INVERT COMMAND-TYPE FIND))
     (t . (op17-11 ISA OPERATOR PRE READY-TO-INVERT ACTION INVERT-OP ARG1 OP ARG2 TYPE-PART POST STATE8 COMMAND-TYPE RETRIEVE&ACT))
     (t . (op17-12 ISA OPERATOR PRE STATE8 ACTION READ-NUMBER ARG1 BOX2 ARG2 TYPE-PART POST INVERT-DONE COMMAND-TYPE FIND&ACT))
     (t . (op17-13 ISA OPERATOR PRE INVERT-DONE ACTION type ARG1 "" arg2 string POST STATE9 COMMAND-TYPE ACTING)))))

(defparameter pre2-7 (append pre2-6
   '((t . (op26-1 ISA OPERATOR PRE MATCHING-BOXES ACTION FIND-MATCHING-BOXES ARG1 BOX1 ARG2 SCREEN POST GOT-MATCHING-BOX COMMAND-TYPE FIND))
     (t . (op26-2 ISA OPERATOR PRE GOT-MATCHING-BOX ACTION FIND-BELOW ARG1 BOX2 ARG2 BOX1 POST TWO-BOXES COMMAND-TYPE FIND))
     ;;
     ;; Modified, AS, Oct 13, 2005
     ;; The model does not have a OP26-3 operator any more.
     ;; Instead, the new OP26-3A has been used.
     ;;   OP26-3 (commented) --> OP26-3A
     ;;
     ;; Modified, AS, Oct 20th, 2005
     ;; Addded the tutor flag to each operator, so that (un)commenting is no more required  
     ;;
     (1 . (OP26-3 ISA OPERATOR PRE BOX12-CLICKED-COMBINE ACTION CLICK ARG1 "combine" ARG2 string POST STATE10 COMMAND-TYPE ACTING))
     (2 . (OP26-3a ISA OPERATOR PRE BOX12-CLICKED-COMBINE ACTION CLICK ARG1 "collect" ARG2 NIL POST STATE10 COMMAND-TYPE ACTING))
     (t . (op26-4 ISA OPERATOR PRE BOXES-COMBINED ACTION FIND-THREE ARG1 BOX-THREE ARG2 SCREEN POST THREE-FOUNDA COMMAND-TYPE FIND))
     (t . (op26-5 ISA OPERATOR PRE THREE-FOUNDA ACTION CLICK ARG1 BOX-THREE ARG2 variable POST STATE11 COMMAND-TYPE ACTING))
    (t . (op26-6  isa operator PRE GREEN-CLICKED-COMBINE action assign-left arg1 left arg2 box2 post left-done  command-type find))
    (t . (op26-6a  isa operator PRE left-done action assign-right arg1 right arg2 box2 post right-done  command-type find))
     (t . (op26-7 ISA OPERATOR PRE right-done ACTION READ-NUMBER ARG1 LEFT ARG2 TYPE-PART POST 3ARG1-COPIEDA COMMAND-TYPE FIND&ACT))
     (t . (op26-8 ISA OPERATOR PRE 3ARG1-COPIEDA ACTION READ-OP1 ARG1 BOX1 ARG2 BOX2 POST STATE12 COMMAND-TYPE COMBINE-ACT))
     (t . (op26-9 ISA OPERATOR PRE STATE12 ACTION READ-NUMBER ARG1 RIGHT ARG2 TYPE-PART POST ENTER-THREE COMMAND-TYPE FIND&ACT))
     (t . (op26-10 ISA OPERATOR PRE ENTER-THREE ACTION type ARG1 "" ARG2 string POST STATE13 COMMAND-TYPE ACTING))
     (t . (op26-11 ISA OPERATOR PRE STATE13 ACTION FIND-ONE ARG1 BOX-ONE ARG2 SCREEN POST ONE-FOUNDA COMMAND-TYPE FIND))
     (t . (op26-12 ISA OPERATOR PRE ONE-FOUNDA ACTION CLICK ARG1 BOX-ONE ARG2 variable POST STATE14 COMMAND-TYPE ACTING))
     (t . (op26-13 ISA OPERATOR PRE STATE14 ACTION READ-OP2 ARG1 LEFT ARG2 RIGHT POST STATE15 COMMAND-TYPE COMBINE-ACT))
     (t . (op26-14 ISA OPERATOR PRE STATE15 ACTION type ARG1 "" ARG2 string POST STATE16 COMMAND-TYPE ACTING))
     (t . (op26-15 ISA OPERATOR PRE DEAD-END ACTION CLICK ARG1 "next" ARG2 string POST STATE20 COMMAND-TYPE ACTING)))))


(defparameter pre3-1 (append (remove '(t . (op17-9 ISA OPERATOR PRE GREEN-CLICKED ACTION READ-NUMBER ARG1 BOX1 ARG2 TYPE-PART POST ARG1-COPIED COMMAND-TYPE FIND&ACT)) pre2-7 :test 'equal)
    '((t . (op27-1 ISA OPERATOR PRE GREEN-CLICKED ACTION CHECK-INVERT-OP ARG1 BOX2 ARG2 OP-SPECIAL POST OP-NORMAL COMMAND-TYPE FIND-CHOOSE))
      (t . (op27-3 ISA OPERATOR PRE OP-SPECIAL ACTION READ-NUMBER ARG1 BOX2 ARG2 TYPE-PART POST ARG1-COPIEDA COMMAND-TYPE FIND&ACT))
      (t . (op27-4 ISA OPERATOR PRE ARG1-COPIEDA ACTION READ-OP ARG1 BOX2 ARG2 TYPE-PART POST OP2-COPIED COMMAND-TYPE FIND&ACT))
      (t . (op27-5 ISA OPERATOR PRE OP2-COPIED ACTION READ-NUMBER ARG1 BOX1 ARG2 TYPE-PART POST INVERT-DONE COMMAND-TYPE FIND&ACT)) 
      (t . (op27-2 ISA OPERATOR PRE OP-NORMAL ACTION READ-NUMBER ARG1 BOX1 ARG2 TYPE-PART POST ARG1-COPIED COMMAND-TYPE FIND&ACT)))))


(defparameter pre3-2 (append pre3-1
    '((t . (op31-1 ISA OPERATOR PRE PLUS-BOX ACTION FIND-PLUS-BOX ARG1 BOX1 ARG2 SCREEN POST GOT-PLUS-BOX COMMAND-TYPE FIND))
      (t . (op31-2 ISA OPERATOR PRE GOT-PLUS-BOX ACTION FIND-TIMES-BELOW ARG1 BOX2 ARG2 BOX1 POST TWO-BOXES COMMAND-TYPE FIND))
      ;;
      ;; Modified, Oct 13th, 2005
      ;; Operator 31-4 does not exist any more. 
      ;; The original was renamed as op31-4a.
      ;; With tutor2, you should use op31-4b.
      ;;
      ;; Modified, Oct 20th, 2005
      ;; Both of the operators can coexists, because of the dinamic operator loading.
      ;; In addition, the operators may have the very same name, because they will be loaded
      ;; in different contexts.
      ;;
      (1 . (OP31-4 ISA OPERATOR PRE BOX12-CLICKED-REORDER ACTION CLICK ARG1 "reorder" ARG2 string POST STATE10 COMMAND-TYPE ACTING))
      (2 . (OP31-4 ISA OPERATOR PRE BOX12-CLICKED-REORDER ACTION CLICK ARG1 "distribute" ARG2 string POST STATE10 COMMAND-TYPE ACTING))
      
      (t . (op31-5 ISA OPERATOR PRE BOXES-REORDERED ACTION FIND-TWO ARG1 BOX-TWO ARG2 SCREEN POST TWO-FOUND COMMAND-TYPE FIND))
      (t . (op31-6 ISA OPERATOR PRE TWO-FOUND ACTION CLICK ARG1 BOX-TWO ARG2 variable POST STATE17 COMMAND-TYPE ACTING))
      (t . (op31-7 ISA OPERATOR PRE GREEN-CLICKED-REORDER ACTION READ-NUMBER ARG1 BOX2 ARG2 TYPE-PART POST 2ARG1-COPIED COMMAND-TYPE FIND&ACT))
      (t . (op31-8 ISA OPERATOR PRE 2ARG1-COPIED ACTION TYPE ARG1 "*" ARG2 string POST state18 COMMAND-TYPE ACTING))
;      (t . (op31-9 ISA OPERATOR PRE DONE*1 ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE18 COMMAND-TYPE ACTING)) 
      (t . (op31-10 ISA OPERATOR PRE STATE18 ACTION FIND-THREE ARG1 BOX-THREE ARG2 SCREEN POST THREE-FOUND COMMAND-TYPE FIND))
      (t . (op31-11 ISA OPERATOR PRE THREE-FOUND ACTION CLICK ARG1 BOX-THREE ARG2 variable POST STATE19 COMMAND-TYPE ACTING))
      (t . (op31-12 ISA OPERATOR PRE STATE19 ACTION READ-NUMBER ARG1 BOX2 ARG2 TYPE-PART POST 3ARG1-COPIED COMMAND-TYPE FIND&ACT))
      (t . (op31-13 ISA OPERATOR PRE 3ARG1-COPIED ACTION TYPE-PART ARG1 "*" ARG2 string POST STATE20 COMMAND-TYPE ACTING-PART))
      (t . (op31-14 ISA OPERATOR PRE STATE20 ACTION READ-NUMBER ARG1 BOX1 ARG2 TYPE POST state21 COMMAND-TYPE FIND&ACT))
;      (t . (op31-15 ISA OPERATOR PRE ENTER-THREEA ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE21 COMMAND-TYPE ACTING)) 
      (t . (op31-16 ISA OPERATOR PRE STATE21 ACTION FIND-ONE ARG1 BOX-ONE ARG2 SCREEN POST ONE-FOUND COMMAND-TYPE FIND))
      (t . (op31-17 ISA OPERATOR PRE ONE-FOUND ACTION CLICK ARG1 BOX-ONE ARG2 variable POST STATE22 COMMAND-TYPE ACTING))
      (t . (op31-18 ISA OPERATOR PRE STATE22 ACTION READ-OP ARG1 BOX1 ARG2 TYPE POST start COMMAND-TYPE FIND&ACT)))))
;      (t . (op31-19 ISA OPERATOR PRE 1OP-COPIED ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE23 COMMAND-TYPE ACTING)))))


(defparameter pre4-1 (append pre3-2
     '((t . (op34-1 ISA OPERATOR PRE COLLECT-BOXES ACTION FIND-COLLECT-BOXES ARG1 BOX1 ARG2 SCREEN POST GOT-COLLECT-BOX COMMAND-TYPE BIND&CLICK))
       (t . (op34-2 ISA OPERATOR PRE GOT-COLLECT-BOX ACTION FIND-LEFT* ARG1 BOX2 ARG2 BOX1 POST STATE24 COMMAND-TYPE BIND&CLICK))
       (t . (op34-3 ISA OPERATOR PRE STATE24 ACTION FIND-LEFT+ ARG1 BOX3 ARG2 BOX1 POST STATE25 COMMAND-TYPE BIND&CLICK))
       (t . (op34-4 ISA OPERATOR PRE STATE25 ACTION FIND-RIGHT* ARG1 BOX4 ARG2 BOX1 POST STATE26 COMMAND-TYPE BIND&CLICK)) 
       (t . (op34-5 ISA OPERATOR PRE STATE26 ACTION FIND-RIGHT+ ARG1 BOX5 ARG2 BOX1 POST STATE27 COMMAND-TYPE BIND&CLICK))
       ;;
       ;; Modified, AS, Oct 18, 2005
       ;; Commented a tutor1 operator that otherwise would shadow the tutor2 one.
       ;;
       ; (OP34-6 ISA OPERATOR PRE STATE27 ACTION CLICK ARG1 "combine" ARG2 string POST STATE28 COMMAND-TYPE ACTING)
       ;; 
       ;; Modified, AS, Oct 14, 2005
       ;; Added a tutor2-specific variant of the operator.  Known to work when running without presettings
       ;;
       ; (OP34-6b ISA OPERATOR PRE STATE27 ACTION CLICK ARG1 "collect" ARG2 string POST STATE28 COMMAND-TYPE ACTING)
       ;; End of variant
       ;;
       ;; Modified, AS, October 20th, 2005
       ;; Added both of the operators with tutor-specific tags
       ;;
       (1 . (OP34-6 ISA OPERATOR PRE STATE27 ACTION CLICK ARG1 "combine" ARg2 string POST STATE28 COMMAND-TYPE ACTING))
       (2 . (OP34-6 ISA OPERATOR PRE STATE27 ACTION CLICK ARG1 "collect" ARG2 string POST STATE28 COMMAND-TYPE ACTING))
       ;; End of variant
       ;;
       (t . (op34-7 ISA OPERATOR PRE BOXES-COLLECTED ACTION FIND-THREE ARG1 BOX-THREE ARG2 SCREEN POST THREE-FOUNDB COMMAND-TYPE FIND))
       (t . (op34-8 ISA OPERATOR PRE THREE-FOUNDB ACTION CLICK ARG1 BOX-THREE ARG2 variable POST STATE28 COMMAND-TYPE ACTING))
       (t . (op34-9 ISA OPERATOR PRE GREEN-CLICKED-COLLECT ACTION READ-SIGNED-NUMBER ARG1 BOX2 ARG2 BOX3 POST 3ARG1-COPIEDB COMMAND-TYPE COMBINE-ACT))
       (t . (op34-10 ISA OPERATOR PRE 3ARG1-COPIEDB ACTION READ-OP ARG1 BOX1 ARG2 TYPE-PART POST STATE29 COMMAND-TYPE FIND&ACT))
       (t . (op34-11 ISA OPERATOR PRE STATE29 ACTION READ-SIGNED-NUMBER ARG1 BOX4 ARG2 BOX5 POST ENTER-THREEB COMMAND-TYPE COMBINE-ACT))
       (t . (op34-12 ISA OPERATOR PRE ENTER-THREEB ACTION type ARG1 "" ARG2 string POST STATE30 COMMAND-TYPE ACTING))
       (t . (op34-13 ISA OPERATOR PRE STATE30 ACTION FIND-ONE ARG1 BOX-ONE ARG2 SCREEN POST ONE-FOUNDB COMMAND-TYPE FIND))
       (t . (op34-14 ISA OPERATOR PRE ONE-FOUNDB ACTION CLICK ARG1 BOX-ONE ARG2 variable POST STATE31 COMMAND-TYPE ACTING))
       (t . (op34-15 ISA OPERATOR PRE STATE31 ACTION TYPE ARG1 "*" ARG2 string POST STATE32 COMMAND-TYPE ACTING))
;       (t . (op34-16 ISA OPERATOR PRE STATE32 ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE33 COMMAND-TYPE ACTING))
       (t . (op34-17 ISA OPERATOR PRE RIGHT-COMBINE1 ACTION FIND-TWO ARG1 BOX-TWO ARG2 SCREEN POST TWO-FOUNDB COMMAND-TYPE FIND))
       (t . (op34-18 ISA OPERATOR PRE TWO-FOUNDB ACTION CLICK ARG1 BOX-TWO ARG2 variable POST STATE34 COMMAND-TYPE ACTING))
       (t . (op34-20 ISA OPERATOR PRE STATE34 ACTION READ-NUMBER ARG1 BOX5 ARG2 TYPE-PART POST STATE35 COMMAND-TYPE FIND&ACT))
       (t . (op34-19 ISA OPERATOR PRE STATE35 ACTION READ-OP ARG1 BOX1 ARG2 TYPE POST STArt COMMAND-TYPE FIND&ACT))
;       (t . (op34-21 ISA OPERATOR PRE STATE36 ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE37 COMMAND-TYPE ACTING))
       (t . (op34-22 ISA OPERATOR PRE LEFT-COMBINE2 ACTION FIND-TWOA ARG1 BOX-TWO ARG2 SCREEN POST TWO-FOUNDC COMMAND-TYPE FIND))
       (t . (op34-23 ISA OPERATOR PRE TWO-FOUNDC ACTION CLICK ARG1 BOX-TWO ARG2 variable POST STATE38 COMMAND-TYPE ACTING))
       (t . (op34-24 ISA OPERATOR PRE STATE38 ACTION READ-OP ARG1 BOX3 ARG2 TYPE-PART POST STATE39 COMMAND-TYPE FIND&ACT))
       (t . (op34-25 ISA OPERATOR PRE STATE39 ACTION READ-NUMBER ARG1 BOX3 ARG2 TYPE POST STArt COMMAND-TYPE FIND&ACT))
;       (t . (op34-26 ISA OPERATOR PRE STATE40 ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE41 COMMAND-TYPE ACTING))
       (t . (op34-32 ISA OPERATOR PRE LEFT-COMBINE1 ACTION FIND-TWO ARG1 BOX-TWO ARG2 SCREEN POST TWO-FOUNDE COMMAND-TYPE FIND))
       (t . (op34-33 ISA OPERATOR PRE TWO-FOUNDE ACTION CLICK ARG1 BOX-TWO ARG2 variable POST STATE46 COMMAND-TYPE ACTING))
       (t . (op34-34 ISA OPERATOR PRE STATE46 ACTION READ-NUMBER ARG1 BOX3 ARG2 TYPE-PART POST STATE47 COMMAND-TYPE FIND&ACT))
       (t . (op34-35 ISA OPERATOR PRE STATE47 action type arg1 "+" arg2 string post state48 command-type acting))
;       (t . (op34-36 ISA OPERATOR PRE STATE48 ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE49 COMMAND-TYPE ACTING))
       (t . (op34-27 ISA OPERATOR PRE RIGHT-COMBINE2 ACTION FIND-TWOA ARG1 BOX-TWO ARG2 SCREEN POST TWO-FOUNDD COMMAND-TYPE FIND))
       (t . (op34-28 ISA OPERATOR PRE TWO-FOUNDD ACTION CLICK ARG1 BOX-TWO ARG2 variable POST STATE42 COMMAND-TYPE ACTING))
       (t . (op34-29 ISA OPERATOR PRE STATE42 ACTION READ-OP1 ARG1 BOX5 ARG2 BOX1 POST STATE43 COMMAND-TYPE COMBINE-ACT))
       (t . (op34-30 ISA OPERATOR PRE STATE43 ACTION READ-NUMBER ARG1 BOX5 ARG2 TYPE POST STArt COMMAND-TYPE FIND&ACT))
;       (t . (op34-31 ISA OPERATOR PRE STATE44 ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE45 COMMAND-TYPE ACTING))
       (t . (op34-37 ISA OPERATOR PRE BOTH-COMBINE ACTION FIND-THREEA ARG1 BOX-THREE ARG2 SCREEN POST THREE-FOUNDC COMMAND-TYPE FIND))
       (t . (op34-38 ISA OPERATOR PRE THREE-FOUNDC ACTION CLICK ARG1 BOX-THREE ARG2 variable POST STATE50 COMMAND-TYPE ACTING))
       (t . (op34-39 ISA OPERATOR PRE STATE50 ACTION READ-SIGNED-NUMBERA ARG1 BOX3 ARG2 TYPE-PART POST 3ARG1-COPIEDC COMMAND-TYPE FIND&ACT))
       (t . (op34-40 ISA OPERATOR PRE 3ARG1-COPIEDC ACTION READ-OP ARG1 BOX1 ARG2 TYPE-PART POST STATE51 COMMAND-TYPE FIND&ACT))
       (t . (op34-41 ISA OPERATOR PRE STATE51 ACTION READ-SIGNED-NUMBERA ARG1 BOX5 ARG2 TYPE POST state52 COMMAND-TYPE FIND&ACT))
;       (t . (op34-42 ISA OPERATOR PRE ENTER-THREEC ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE52 COMMAND-TYPE ACTING))
       (t . (op34-43 ISA OPERATOR PRE STATE52 ACTION FIND-ONEA ARG1 BOX-ONE ARG2 SCREEN POST ONE-FOUNDC COMMAND-TYPE FIND))
       (t . (op34-44 ISA OPERATOR PRE ONE-FOUNDC ACTION CLICK ARG1 BOX-ONE ARG2 variable POST STATE53 COMMAND-TYPE ACTING))
       (t . (op34-45 ISA OPERATOR PRE STATE53 ACTION TYPE ARG1 "+" ARG2 string POST STATE54 COMMAND-TYPE ACTING)))))
;       (t . (op34-46 ISA OPERATOR PRE STATE54 ACTION SUBMIT ARG1 CONTENTS ARG2 NIL POST STATE55 COMMAND-TYPE ACTING)))))

(defvar *start*)
(defvar *confusions*)
(defvar *section*)
(defvar *input-length*)
(defparameter *kind* 'example)


(defun dump-it ()
  (do ((temp (no-output (sdm isa operator)) (cdr temp))
       (result nil (cons (dump-one (car temp)) result)))      
      ((null temp) (reverse result))))

(defun dump-one (item)
  (do ((temp '(pre action arg1 arg2 post command-type) (cdr temp))
       (result (list item 'isa 'operator) (append result (list (car temp) (chunk-slot-value-fct item (car temp))))))
      ((null temp) result)))

(defun get-count (item)
  (no-output (eval `(if (dm ,item) (caaar (sdp ,item :references))))))

(defun enable-confusions ()
  (let (result)
  (do ((temp (append instructions27 pre2-7) (cdr temp)))
      ((null temp) (setf *confusions* result))
    (let ((pre (get-value (car temp) 'pre))
          (post (get-value (car temp) 'post)))
      (cond ((not (assoc pre result))
             (setf result (cons (list pre 0) result))))
      (cond ((not (assoc post result))
             (setf result (cons (list post 0) result))))
      (setf result (append result (list (list (list pre post) -.025))))))))

(defun modifier (n1 n2)
  (sqrt (/ (* 2 n1 n2) 
           (+ n1 n2))))

(defun test-confusion (a b)
  (let ((item (assoc (list a b) *confusions* :test 'equal) )
        (count1 (assoc a *confusions*))
        (count2 (assoc b *confusions*)))
    (cond (item (rplaca (cdr count1) (1+ (second count1)))
                (rplaca (cdr count2) (1+ (second count2)))
                (* (second item) (modifier (second count1) (second count2)))))))

;; PRESET (LIST, TUTOR)
;; Preloads operators before skipping to a predefined section.
;; Basically, it is an intensivelearning course where all of the knowledge
;; prior to a certain section is implanted into the model.
;; 
;; From Oct 20th, 2005, this function requires the mandatory specification
;; of te tutor on which the section is intended to be run.  This is required
;; because operators may be sometimes different for the two tutors, and 
;; are tagged. The function also skims the operators selecting only those that
;; are appropriate for the tutor.
;;
(defun preset (lis tutor)
  ;
  ; First, the current tutor is temporarily bound to the tutor argument
  ;
  (let ()
    (setf *current-tutor* tutor)
    ;
    ; Then, operators are filtered and de-tagged
    ;
    (let ((filtered-ops (extract-proper-operators lis)))
      ;
      ; Finally, each operator is added to the DM and properly setted
      ;
      (do ((temp (eval (cons 'add-dm filtered-ops)) (cdr temp)))
          ((null temp) nil)
        (eval `(sdp ,(car temp) :creation-time -100 :references 20))))))


;;;  DIGITAL MODULE CODE REMOVED (NOW IN AL-BUFFERS.LISP).

(defun internal-state (state)
  (not (member state '(box12-clicked-invert Box2-Clicked Box1-Clicked red-box  BOXES-REORDERED
                       green-clicked Boxes-Inverted type-in-box green-little-boxa start done))))

(defun read-buffer-slot (buffer slot)
  (let ((chunk (car (no-output (buffer-contents buffer)))))
    (chunk-slot-value-fct chunk slot)))



(defun check-left (box)
  (let* ((contents (second (assoc box (cdr (assoc ':old-equation (dump-screen)))))))
    (if (numberp (car contents)) 
      (list 'isa 'box 'val 'success)
      (list 'isa 'box 'val 'failure))))

(defun check-error (arg)
   (declare (ignore arg))
   (let ((text (cdr (assoc ':text (dump-screen))))
         (general (second (assoc :text (second (assoc :instructions (dump-screen)))))))
  
  (cond ((equal text '("End of Experiment.")) (list 'isa 'box 'val 'end))
        ((equal text '("Take a break. Click Continue when you're ready to go on."))(list 'isa 'box 'val 'continue))
        (t 
  (if (or (and (equal general "There is a special case when the operator is - and the unknown is
on the right.")
                  (equal text '(("Your answer is incorrect." "The result of subtracting something from 10 is 2.
To find out the value of that something, subtract 2 from 10.
Click the green box and type 10 - 2."))))
          (and 
;(equal general "Solve the following problems.") 
           (equal text '("Your answer is incorrect.
Use the <- and -> buttons (or the left and right arrow keys)
and the First Mistake button
to review your work and correct the mistake."))))
     (list 'isa 'box 'val 'error)
     (list 'isa 'box 'val 'error-free))))))
     

(defun find-box-pointer (box)
  (let* ((screen (dump-screen))
         (pointer (find-pointer box (cdr (assoc ':old-equation screen))))
         (expr (second (assoc pointer (cdr (assoc ':equation screen)))))
         (arg2 (if (listp expr) (third expr)))
         (arg1 (if (listp expr) (first expr)))
         (val (if (atom expr) expr (if (wrong-one arg1 (cdr (assoc ':equation screen))) arg2 arg1))))
    (list 'isa 'box 'val (if (listp val) (car val) val))))

(defun wrong-one (arg boxes)
  (or (numberp arg) (assoc arg boxes)))

(defun find-pointer (box lis)
  (do ((temp lis (cdr temp)))
      ((null temp) nil)
    (cond ((atom (second (car temp))) (if (eq box (second (car temp))) (return (caar temp))))
           ((member box (second (car temp))) (return (caar temp))))))

(defun suitable-operator-p (op)
  (let ((context (car op)))
    (or (equal context t)
        (equal context *current-tutor*))))

;; Filters operators so that only the ones pertaining to
;; the current tutor may be used.
;;
(defun filter-proper-operators (lst)
  (remove-if-not #'suitable-operator-p lst))

;; Retrieves operators from a list of
;; context-operator associations
;;  
(defun extract-operators (lst)
  (mapcar #'cdr lst))

(defun extract-proper-operators (lst)
  (setf lst (second (assoc *kind* lst)))
  (extract-operators (filter-proper-operators lst)))

;; FOLLOW-HELP
;; Associated hints appearing to the screen to operators,
;; whihc will be loaded into DM.
;;
;; Modified, Oct, 2005, AS. Eliminated redundant code.
;; Added support for both tutors.
;;
(defun follow-help () 
  (let ((instructions (clean (second (assoc ':text (dump-screen))))))
    (cond ((null instructions) nil)
          ;;
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;; This section was modified by AS, Oct 11th., 2005
          ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
          ;; Instructions are slightly different than in Tutor 1.
          ;; Some text has changed to let the tutor understand the
          ;; hints (waiting for a simple language parser).
          ;; In particular, box --> operator
          ;; 
          ((or (equal instructions "Click the box pointed to by the arrow.")
               (equal instructions "Click the operator pointed to by the arrow."))
           (if (and (= *current-tutor* 1) (= (find-section) 12)) (extract-proper-operators hint12a)(extract-proper-operators hint11a)))
          ((equal instructions "Click the variable pointed to by the arrow.")(extract-proper-operators hint12a))
          ((equal instructions "Click the Evaluate button(or press the 'e' key).") 
           (extract-proper-operators hint11b))

          ((equal instructions "Substitute X for its valueClick the green box and type .") 
           (extract-proper-operators hint12c1))
          
          ((or (equal instructions "   = Click the green box and type .")
               (equal instructions "Copy the value according to the arrow.Click the green box and type ."))
           (extract-proper-operators hint11c))
          
          ((equal instructions "Your Answer is correct.Click the Next Prob. button (or press the 'n' key).") 
           (if (equal (perceive-state) 'dead-end) 
             (extract-proper-operators hint26c)
             (extract-proper-operators hint11d)))
          
          ;; Here: boxes --> operators
          ((equal instructions "Click the boxes pointed to by the arrows.") 
           (extract-proper-operators hint17a))
          ((equal instructions "Click the operators pointed to by the arrows.") 
           (extract-proper-operators hint17a))
          
          ;; Here: Invert  --> Unwind;
          ;; Here: 'i' key --> 'u' key 
          ((or (equal instructions "Click the Unwind button(or press the 'u' key).")
               (equal instructions "Click the Invert button(or press the 'i' key)."))
           (extract-proper-operators hint17b))
          
          ((or (equal instructions "The result of adding  to something is .To find the value of that something, subtract  from .Click the green box and type   .")
               (equal instructions "The result of subtracting something from  is .To find out the value of that something, subtract  from .Click the green box and type   .")
               (equal instructions "The result of multiplying  by something is .To find the value of that something, divide  by .Click the green box and type   .")
               (equal instructions "The result of dividing  into something is .To find out the value of that something, multiply  by .Click the green box and type   .")
               (equal instructions "The result of subtracting  from something is .To find out the value of that something, add  to .Click the green box and type   .")) 
           (if (< (find-section) 27) 
             (extract-proper-operators hint17c)
             (extract-proper-operators new-hint)))

          ((or (equal (perceive-state) 'BOXES-COMBINED)
               (equal (perceive-state) 'GREEN-CLICKED-COMBINE)) 
           (extract-proper-operators hint26b))

          ((equal instructions "Click the Subtract button(or press the 's' key).") 
           (extract-proper-operators hint43a))

          ;((equal instructions "Click the Combine button(or press the 'c' key).")
          ;;
          ;; Modified, AS, Oct 13th, 2005
          ;; Added to right hint for section 34.
          ;;   hint34a --> hint34a2
          ;;
          ;; Modified, AS, Oct 20th, 2005
          ;; Allowed both expression, depending on the tutor.
          ;;
          ((or (equal instructions "Click the Collect button(or press the 'c' key).")
               (equal instructions "Click the Combine button(or press the 'c' key)."))
           (if (eq (perceive-state) 
                   'boxes-to-combine)
             (extract-proper-operators hint34a)
             (extract-proper-operators hint26a)))    ;;; HERE!!!
          
          ((member (perceive-state) '(GREEN-CLICKED-COLLECT boxes-collected)) 
           (cond ((and (present-test 'box3) 
                       (present-test 'box5)) 
                  (extract-proper-operators (append hint34b HINT34-combine)))
                 ((present-test 'box3)
                  (extract-proper-operators (append hint34b (if (find-two nil) HINT34-left1 hint34-left2))))
                 
                 ((present-test 'box5) 
                  (extract-proper-operators (append hint34b (if (find-two nil) HINT34-right1 hint34-right2))))

                 (t (extract-proper-operators hint34b))))
          ;;
          ;; Modified, AS, Oct 12th, 2005
          ;; Both prehints 31a and 31a2 point to the hint31a operator
          ;;
          ;; ((equal instructions pre-hint31a) hint31a)
          ;;
          ((or (equal instructions pre-hint31a)
               (equal instructions pre-hint31a2))
           (extract-proper-operators hint31a))
          
          ;; Modified, AS, Oct 12th, 2005
          ;; Added one hint for tutor 2
          ;
          ; Modified, AS, Oct 20th, 2005
          ; Uncommented---now operators are tagged
          ;
          ((equal instructions pre-hint31b) 
	   (extract-proper-operators hint31b))
          ((equal instructions pre-hint31b2) 
	   (extract-proper-operators hint31b))
          ((equal instructions pre-hint43b) 
	   (extract-proper-operators hint43b))
          ;;
          ;; Modified, AS, Oct 13, 2005
          ;; Added a clause to handle special instructions for Tutor 2
          ;;
          ((equal instructions pre-hint43b2) 
	   (extract-proper-operators hint43b)))))

(defun clean (seq)
  (if (listp seq) (setf seq (car (last seq))))
  (do ((temp '(#\0 #\1 #\2 #\3 #\4 #\5 #\6 #\7 #\8 #\9 #\+ #\- #\* #\/ #\newline) (cdr temp))
      (result seq (remove (car temp) result)))
      ((null temp) result)))

(defun get-time ()
 (* 1000 (mp-time)))


(defun do-it-all (subj v-flag &key (tutor *default-tutor*) (real-time *real-time*))
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 1 1 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 1 2 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 1 7 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 2 6 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 2 7 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 3 1 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 3 2 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 3 4 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 4 1 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 4 2 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time)))
  (test-problems 4 3 subj 6000 v-flag :tutor tutor :real-time real-time)
  (print (list 'times (get-internal-real-time) (get-time))))

(defun tune-it (subj v-flag &key (tutor *default-tutor*) (real-time *real-time*))
  (test-problems 1 1 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 1 2 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 1 7 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 2 7 subj 6000 v-flag :tutor tutor :real-time real-time))

(defun full-test (subj v-flag &key (tutor *default-tutor*) (real-time *real-time*))
  (test-problems 1 1 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 1 2 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 1 7 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 2 6 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 2 7 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 3 1 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 3 2 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 3 4 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 4 1 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 4 2 subj 6000 v-flag :tutor tutor :real-time real-time)
  (test-problems 4 3 subj 6000 v-flag :tutor tutor :real-time real-time))

(defun test-problems (ch sect subj time v-flag &key (tutor *default-tutor*) (real-time *real-time*)(source *kind*) )
  (let ()
    (setf *current-tutor* tutor)
    (setf *section* (+ (* ch 10) sect))
    (if v-flag
      (sgp :v t)
      (sgp :v nil))
    (setf *conceptual-busy* nil
           *conceptual-error* nil
          *verbal-busy* nil 
          *graphical-busy* nil 
          *verbal-error* nil 
          *graphical-error* nil 
          *instructions* nil 
          *assess-success* nil
          *digital-busy* nil 
          *digital-error* nil 
          *input-length* 0
          *mathematical-busy* nil 
          *mathematical-error* nil 
          *ignore-error* nil 
          *kind* source
          *hint* nil
          *state* nil 
          *start* t)
    (case tutor
      (1 (tutor::setup-tutor1))
      (2 (tutor::setup-tutor2))
      (otherwise (error "Non-existing type of tutor: ~A" tutor)))
    (do-section ch sect subj)
    (let ((goal (new-name-fct 'goal)))
      (eval `(add-dm (,goal isa task state start step nil watch none)))
      (eval `(goal-focus ,goal))
      (run .05 :real-time real-time)
      (update-state))
    (run time :real-time real-time)))

(defun startp (x)
  (equal x 'start))

(defun graphical-command (x) (not (member x '(evaluate inverse))))

(defun make-state () (gentemp "STATE"))
(defun make-label ()   (gentemp "LABEL"))

(defun old-state (state screen) (if (equal state 'start) screen state))

(defun make-action (arg) 
  (cond ((equal (chunk-slot-value-fct arg 'fourth) 'evaluated) 'just-numbers)
        ((or (equal (chunk-slot-value-fct arg 'second) 'bottom)
            (equal (chunk-slot-value-fct arg 'seventh) 'side)) 'find-bottom)
        ((equal (chunk-slot-value-fct arg 'second) 'inverting) 'invert-op)
        ((equal (chunk-slot-value-fct arg 'third) 'green) 'find-green)
        ((or (equal (chunk-slot-value-fct arg 'third) 'above) 
            (equal (chunk-slot-value-fct arg 'third) 'nearest)) 'find-above)
        ((equal (chunk-slot-value-fct arg 'second) 'evaluating) 'evaluate)
        ((or (equal (chunk-slot-value-fct arg 'second) 'top)
            (equal (chunk-slot-value-fct arg 'third) 'expression))'INPUT-NUMBER)
        ((or (equal (chunk-slot-value-fct arg 'first) 'pointed)
            (equal (chunk-slot-value-fct arg 'third) 'replaced)) 'find-box-pointer)))

(defun count-words (arg) 
  (do ((count 1 (1+ count))
        (temp '(second third fourth fifth sixth seventh eighth ninth tenth eleventh twelfth)  (cdr temp)))
       ((member (chunk-slot-value-fct arg (car temp)) '(the nil)) count)))

(defun nounp (x) (member x '(box operator number sign)))
(defun adjectivep (x) (member x '(green first second new other equal)))
(defun combine (adj noun) (intern (concatenate 'string (string adj) "-" (string noun))))

(defun unique (object)
  (let* ((equation (cdr (assoc :equation (dump-screen))))
         (old-equation (cdr (assoc :old-equation (dump-screen))))
        (box (assoc object equation)))
    (cond ((equal object (bottom-number-box)) '(find-bottom screen))
          ((equal object (BOX-ABOVE-BOTTOM)) (list 'find-above (find-label (bottom-number-box))))          
          ((equal object (green-box)) '(find-green screen))                          
          ((equal object (invert-operator-above-bottom)) (list 'inverse  (list 'find-operator-above (find-label (above (caar old-equation))))))   
          ((equal object (bottom-number)) (list 'find-number-old (find-label (caar old-equation))))   
          ((equal object (NUMBER-ABOVE-BOTTOM)) (list 'find-number-old (find-label (above (caar old-equation)))))
          ((equal object (green-little-box)) (list 'FIND-BOX-POINTER (get-pointer object 
                                                                                  (car (member object equation :test 'match-contents))
                                                                       old-equation)))
          ((and box (numberp (second box)))  '(input-number screen))
  ;        ((and (equal object (get-number-evaluation))
   ;             (member object equation :key 'second)) '(input-number screen))
 ;         ((equal object (evaluable-box))  '(input-number screen))
          ((and box (numberp (first (second box))) (numberp (third (second box)))) '(just-numbers screen))
          ((numberp object) (find-number object old-equation)))))
          

(defun find-number (number equation)
  (do ((temp equation (cdr temp)))
      ((null temp) nil)
    (if (equal number (get-math-value (second (car temp)))) (return (list 'evaluate (find-label (caar temp)))))))

(defun get-math-value (expr)
  (cond ( (and (listp expr) (numberp (first expr)) (numberp (third expr))) (eval (list (second expr) (first expr) (third expr))))
        ((numberp expr) expr)))



(defun get-pointer (object box old-equation)
  (cond ((null box) (find-label (second (car old-equation)))) (t
  (let* ((oldbox (assoc (car box) old-equation))
         (from (if (and (equal object (if (listp (car (second box))) (caar (second box)) (car (second box))))
                        (really (assoc (first (second oldbox)) old-equation)))
                 (first (second oldbox))(third (second oldbox)))))
    (find-label from)))))

(defun really (y) (listp (second y)))

(defun find-label (item)
  (do ((temp (reverse (chunk-type-slot-names assoc)) (cdr temp)))
      ((null temp) nil)
    (if (equal (chunk-slot-value-fct (buffer-read 'label) (car temp)) item) (return (car temp)))))

(defun bottom-number ()
  (let* ((expr (cdr (assoc ':old-equation (dump-screen))))
         (bottom (car expr))
         (contents (second bottom))
         (box (first bottom))
         (above (above box))
         (rest (second (assoc above expr))))
    (if  (listp contents) (if (numberp (first contents)) (first contents) (third contents)))))

(defun INVERT-OPERATOR-ABOVE-BOTTOM ()
  (let* ((expr (cdr (assoc ':old-equation (dump-screen))))
         (bottom (car expr))
         (box (first bottom))
         (above (above box))
         (above-expr (second (assoc above expr)))
         (op (if (listp above-expr) (second above-expr))))
    (case op (+ '-) (- '+) (* '/) (/ '*))))

(defun NUMBER-ABOVE-BOTTOM ()
  (let* ((expr (cdr (assoc ':old-equation (dump-screen))))
         (bottom (car expr))
         (box (first bottom))
         (above (above box))
         (target (second (assoc above expr))))
    (if (listp target) (if (numberp (first target)) (first target) (third target)))))

(defun above (bot)
  (let* ((expr (second (assoc bot (cdr (assoc ':old-equation (dump-screen)))))))
    (if (listp expr) (if (numberp (car expr)) (third expr) (car expr)))))
                        

(defun match-contents (item box)
  (and (listp (second box)) (member item (second box) :key 'access-car)))

(defun access-car (x) (if (listp x) (car x) x))

(defun unique-action (object) (first (unique object)))
(defun unique-referent (object) (second (unique object)))

(defun get-number-evaluation ()
  (do ((temp (cdr (assoc :old-equation (dump-screen))) (cdr temp)))
      ((null temp) nil)
    (cond ((numberp (second (car temp))) (return (second (car temp))))
          ((get-math-value (second (car temp))) (return (get-math-value (second (car temp))) )))))

(defun sign-it (item)
  (let ((target (second (second (assoc item (cdr (assoc ':old-equation (dump-screen))))))))
    (if target target item)))
   



(clear-all)


(define-model box

(eval `(sgp :esc t :egs 0.05  :ol 1 :rt -1.6 :bll .5 :pl t  :epl t :lf 0.3 :g 200  :ut -100 :v t  
            :model-warnings nil :do-not-harvest label :do-not-harvest conceptual
;  :sim-hook test-confusion
            :ga 1 :ans 0 :mp 10 :pm t :trace-detail low :pct t :do-not-harvest screen :alpha .2))

(chunk-type operator pre action arg1 arg2 post command-type)
(chunk-type task state step arg1 arg2 return post watch action op)
(chunk-type box command val val1 state)
(chunk-type sentence first second third fourth fifth sixth seventh eighth ninth tenth eleventh twelfth)
(chunk-type verbage command op result arg1 arg2)
(chunk-type digital command item state)
(chunk-type assoc screen)
(chunk-type temporal state)
(chunk-type screen state hint instructions equation error)
(chunk-type inverse first second)



(add-dm (note isa temporal state ok) (finger isa digital)
        (i1 isa inverse first + second -)
        (i2 isa inverse first - second +)
        (i3 isa inverse first * second /)
        (i4 isa inverse first / second *))

(sdp i1 :references 1000 :creation-time -100)
(sdp i2 :references 1000 :creation-time -100)
(sdp i3 :references 1000 :creation-time -100)
(sdp i4 :references 1000 :creation-time -100)


;(enable-confusions)
;(setf *similarity-hook-fn* 'test-confusion)

;overall control

(p start-section
   =goal>
      isa          task
      step         nil
      state        =state
==>
   +label>
      isa assoc
      screen screen1
  +verbal>
      isa verbage
      command read-hint
      arg1 =state
   =goal>
       step reading)


(p check-old
   =goal>
      isa task
      step reading
  =verbal>
      isa sentence
      first nil
==>
   =goal>
       step check-old)

(p skip-then
   =goal>
      isa task
      step reading
   ?conceptual>
       state empty
    =verbal>
       isa sentence
       first then
==>
   +verbal>
       isa verbage
       command advance
       arg1 1)

(p read-referent
   =goal>
      isa task
      step reading
   =conceptual>
       isa operator
       arg1 =val
       action =filled
       arg2 screen
       command-type find
    =verbal>
       isa sentence
       first the
       second =label
==>    
   =conceptual>
       arg2 =label
   +verbal>
       isa verbage
       command advance
       arg1 2)

(p five-word-embedding
   =goal>
      isa task
      state =state1
      step reading
   =conceptual>
       isa operator
       arg1 =label
       pre =pre
       post =post
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first =first
       second =second
       third =third
     - third inverting
       fourth the
       fifth =referent
!safe-eval! (not (nounp =first))
==>
!safe-bind! =new (make-state)
!safe-bind! =action (make-action =verbal)
   =conceptual>
      pre =post
      post =new
   +conceptual>
       isa operator
       pre =pre
       action =action
       arg1 =label
       arg2 =referent
       command-type find
       post =post
   +verbal>
       isa verbage
       command advance
       arg1 5)

(p find-relation
   =goal>
      isa task
      step reading
   =conceptual>
       isa operator
       arg1 =label
       pre =pre
       post =post
    =verbal>
       isa sentence
       first which
       second is
       third the
       fourth =relation
       fifth of
       sixth the
       seventh =referent
==>
!safe-bind! =new (make-state)
!safe-bind! =action (make-action =verbal)
   =conceptual>
      pre =post
      post =new
   +conceptual>
       isa operator
       pre =pre
       action =relation
       arg1 =label
       arg2 =referent
       command-type find
       post =post
   +verbal>
       isa verbage
       command advance
       arg1 7)

(p three-word-embedding1
   =goal>
      isa task
      state =state1
      step reading
   =conceptual>
       isa operator
       arg1 =label
       pre =pre
       post =post
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first that
       second is
       third =third
       fourth nil
==>
!safe-bind! =new (make-state)
!safe-bind! =action (make-action =verbal)
   =conceptual>
      pre =post
      post =new
   +conceptual>
       isa operator
       pre =pre
       action =action
       arg1 =label
       arg2 screen
       command-type find
       post =post
   +verbal>
       isa verbage
       command advance
       arg1 3)

(p four-word-number-associated
   =goal>
      isa task
      state =state1
      step reading
   =conceptual>
       isa operator
     - arg1 operator
       arg1 =label
       pre =pre
       post =post
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first associated
       second with
       third the
       fourth =fourth
==>
!safe-bind! =new (make-state)
!safe-bind! =action (make-action =verbal)
   =conceptual>
      pre =post
      post =new
   +conceptual>
       isa operator
       pre =pre
       action find-number-old
       arg1 =label
       arg2 =fourth
       command-type find
       post =post
   +verbal>
       isa verbage
       command advance
       arg1 4)

(p four-word-number-isolated
   =goal>
      isa task
      state =state1
      step reading
   =conceptual>
       isa operator
     - arg1 operator
       arg1 =label
       pre =pre
       post =post
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first isolated
       second by
       third the
       fourth =fourth
==>
!safe-bind! =new (make-state)
!safe-bind! =action (make-action =verbal)
   =conceptual>
      pre =post
      post =new
   +conceptual>
       isa operator
       pre =pre
       action find-number-old
       arg1 =label
       arg2 =fourth
       command-type find
       post =post
   +verbal>
       isa verbage
       command advance
       arg1 4)

(p three-word-number-in
   =goal>
      isa task
      state =state1
      step reading
   =conceptual>
       isa operator
     - arg1 operator
       arg1 =label
       pre =pre
       post =post
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first in
       second the
       third =third
==>
!safe-bind! =new (make-state)
!safe-bind! =action (make-action =verbal)
   =conceptual>
      pre =post
      post =new
   +conceptual>
       isa operator
       pre =pre
       action find-number-old
       arg1 =label
       arg2 =third
       command-type find
       post =post
   +verbal>
       isa verbage
       command advance
       arg1 3)

(p is-equal-to
   =goal>
      isa task
      state =state1
      step reading
   =conceptual>
       isa operator
     - arg1 operator
       arg1 =label
       pre =pre
       post =post
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first that
       second  =third
       third is 
       fourth equal
       fifth to
==>
!safe-bind! =new (make-state)
!safe-bind! =action (make-action =verbal)
   =conceptual>
      pre =post
      post =new
   +conceptual>
       isa operator
       pre =pre
       action find-number-old
       arg1 =label
       arg2 =third
       command-type find
       post =post
   +verbal>
       isa verbage
       command advance
       arg1 5)

(p three-word-find-in
   =goal>
      isa task
      state =state1
      step reading
   =conceptual>
       isa operator
       action nil
       arg1 operator
       pre =pre
       post =post
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first in
       second the
       third =third
==>
!safe-bind! =new (make-state)
!safe-bind! =action (make-action =verbal)
   =conceptual>
       action find-operator-above
       arg2 =third
   +verbal>
       isa verbage
       command advance
       arg1 3)

(p add-noun1
    =conceptual>
        isa operator
        arg1 =adj
    =verbal>
       isa sentence
       first =noun
!safe-eval! (adjectivep =adj)
!safe-eval! (nounp =noun)

==>
!safe-bind! =new (combine =adj =noun)
   =conceptual>
       arg1 =new
   +verbal>
       isa verbage
       command advance
       arg1 1)

(p add-noun2
    =conceptual>
        isa operator
        arg2 =adj
    =verbal>
       isa sentence
       first =noun
!safe-eval! (adjectivep =adj)
!safe-eval! (nounp =noun)

==>
!safe-bind! =new (combine =adj =noun)
   =conceptual>
       arg2 =new
   +verbal>
       isa verbage
       command advance
       arg1 1)

(p string-label
   =goal>
      isa task
      step reading
   =conceptual>
       isa operator
       arg1 =label
       arg2 variable
    =verbal>
       isa sentence
       first labeled
       second =string
!safe-eval! (stringp =string)
==>
   =conceptual>
       arg1 =string
       arg2 string
   +verbal>
       isa verbage
       command advance
       arg1 2)

(p simple-action
   =goal>
      isa task
      state =state1
      step reading
   ?conceptual>
       state empty
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first =action
     - first find
       second the
       third =label
==>
!safe-bind! =new (make-state)
!safe-bind! =old (old-state =state1 =state2)
   +conceptual>
       isa operator
       pre =old
       action =action
       arg1 =label
       arg2 variable
       command-type acting
       post =new
   +verbal>
       isa verbage
       command advance
       arg1 3)

(p find
   =goal>
      isa task
      state =state1
      step reading
   ?conceptual>
       state empty
    =screen>
       isa screen
       state =state2
    =verbal>
       isa sentence
       first find
==>
!safe-bind! =new (make-state)
!safe-bind! =old (old-state =state1 =state2)
   +conceptual>
       isa operator
       pre =old
       arg2 screen
       command-type find
       post =new
   +verbal>
       isa verbage
       command advance
       arg1 1)

(p read-arg1
   =goal>
      isa task
      step reading
   =conceptual>
       isa operator
       arg1 nil
       command-type find
    =verbal>
       isa sentence
       first a
       second =term
==>
   =conceptual>
       arg1 =term
   +verbal>
       isa verbage
       command advance
       arg1 2)

(p read-action
   =goal>
      isa task
      step reading
   =conceptual>
       isa operator
       arg1 =val
    -  arg1 operator
       action nil
       command-type find
    =verbal>
       isa sentence
       first =something
!eval! (not (nounp =something))
==>
!safe-bind! =term (make-action =verbal)
!safe-bind! =number (count-words =verbal)    
   =conceptual>
       action =term
   +verbal>
       isa verbage
       command advance
       arg1 =number)

(p interpret-example-variable
   =goal>
      isa task
      step reading
    =screen>
       isa screen
       state =state
   ?conceptual>
       state empty
    =verbal>
       isa sentence
       first =action
       second =term
       third arrow
!safe-eval! (not (stringp =term))
==>
!safe-bind! =new1 (make-state)
!safe-bind! =new2 (make-state)
!safe-bind! =label (make-label)
   +conceptual>
       isa operator
       pre =new1
       action =action
       arg1 =label
       arg2 variable
       command-type acting
       post =new2
   =goal>
       step interpret-referent
       arg1 =term)

(p interpret-example-string
   =goal>
      isa task
      state =state1
      step reading
    =screen>
       isa screen
       state =state2
   ?conceptual>
       state empty
    =verbal>
       isa sentence
       first =action
       second =term
       third arrow
!safe-eval! (stringp =term)
==>
!safe-bind! =old (old-state =state1 =state2)
!safe-bind! =new2 (make-state)
   +conceptual>
       isa operator
       pre =old
       action =action
       arg1 =term
       arg2 string
       command-type acting
       post =new2
   =goal>
       step check-old)

(p interpret-referent
   =goal>
      isa task
      state =state1
      step interpret-referent
       arg1 =term
   =conceptual>
       isa operator
       pre =new
       arg1 =arg
    =screen>
       isa screen
       state =state2
!eval! (not (listp (unique-referent =term)))
==>
!safe-bind! =action (unique-action =term)
!safe-bind! =referent (unique-referent =term)
!safe-bind! =old (old-state =state1 =state2)
   +conceptual>
       isa operator
       pre =old
       action =action
       arg1 =arg
       arg2 =referent
       command-type find
       post =new
   =goal>
       step check-old)

(p finish-embedded
   =goal>
      isa task
      state =state1
      step embedded-referent
       arg1 =term
   =conceptual>
       isa operator
       pre =new
       arg2 =arg
    =screen>
       isa screen
       state =state2
!eval! (not (listp (second =term)))
==>
!safe-bind! =action (first =term)
!safe-bind! =referent (second =term)
!safe-bind! =old (old-state =state1 =state2)
   +conceptual>
       isa operator
       pre =old
       action =action
       arg1 =arg
       arg2 =referent
       command-type find
       post =new
   =goal>
       step check-old)


(p interpret-referent-embedded
   =goal>
      isa task
      state =state1
      step interpret-referent
       arg1 =term
   =conceptual>
       isa operator
       pre =new
       arg1 =arg
    =screen>
       isa screen
       state =state
!eval! (listp (unique-referent =term))
==>
!safe-bind! =action (unique-action =term)
!safe-bind! =referent (unique-referent =term)
!safe-bind! =old (make-state)
!SAFE-BIND! =LABEL (MAKE-LABEL)
   +conceptual>
       isa operator
       pre =old
       action =action
       arg1 =arg
       arg2 =label
       command-type find
       post =new
   =goal>
       step embedded-referent
       arg1 =referent)

(P check-if-known
   =GOAL>
       ISA TASK
       STEP check-old
   =conceptual>
       isa operator
       pre =pre
       arg1 =arg1
       arg2 =arg2
       action =action
       command-type =type
 ==>
   +retrieval>
       isa operator
       pre =pre
       arg1 =arg1
       arg2 =arg2
       action =action
       command-type =type
   =GOAL>
       STEP retrieving
)

(p finish-reading-new
   =goal>
      isa task
      step retrieving
   ?retrieval>
      state   error
==>
   -conceptual>
   =goal>
      step ready)

(p finish-reading-old
   =goal>
      isa task
      step retrieving
   =retrieval>
      isa operator
      post =new
   =conceptual>
      isa operator
==>
   =conceptual>
      post =new
   -conceptual>
   =goal>
      step ready)



       

(p read-with-emphasis
   =goal>
     isa task
   =screen>
     isa screen
     instructions change
     state =state
==>
   =screen>
     instructions t
  +verbal>
       isa verbage
       command read-special
       arg1 =state
   =goal>
       step correct-error
       state start)

(p read-instructions
   =goal>
     isa task
     step retrieving-operator
     state =state
   =screen>
     isa screen
     instructions t
   ?retrieval>
      state   error
==>
  +verbal>
       isa verbage
       command read-instructions
       arg1 =state
   =goal>
       step reading)

(p read-hint
   =goal>
     isa task
     step retrieving-operator
     state =state
   =screen>
     isa screen
     instructions nil
     hint t
   ?verbal>
      state free
   ?retrieval>
      state   error
==>
  +verbal>
       isa verbage
       command read-hint
       arg1 =state
   =goal>
       step reading)

(p seek-help
   =goal>
     isa task
     step retrieving-operator
   =screen>
     isa screen
     instructions nil
     hint nil
     equation t
   ?retrieval>
      state   error
  ?digital>
       state free
==>
   +digital>
     isa digital
     command click
     item "hint")

(p next-hint
   =goal>
      isa task
      step ready
   ?verbal>
         state error
   ?digital>
         state free
==>
!eval! (free-verbal-error)
   +digital>
     isa digital
     command click
     item "hint")

(p success
   =goal>
      isa task
      step assess
   =screen>
      isa screen
      error nil
==>
!eval! t
   =goal>
      step ready)

(spp success :success t)

(p assess-error
   =goal>
      isa task
      step assess
      state =state
   =screen>
      isa screen
      error t
   ==>
!eval! (free-screen-error)
    +verbal>
       isa VERBAGE
       command correct-error
       arg1 =state
   =goal>
      step correct-error)

(p correct-error
   =goal>
      isa task
      step ready
      state =state
   =screen>
      isa screen
      error t
   ==>
!eval! (free-screen-error)
    +verbal>
       isa VERBAGE
       command correct-error
       arg1 =state
   =goal>
      step correct-error)

(p error-rehearsal
   =goal>
      isa task
      step correct-error
  =verbal>
      isa verbage
      command read-instructions
      result =op
==>
   =goal>
      step 1
      arg1 =op
   +retrieval> =op)

(p continue-rehearsal
   =goal>
      isa task
      step =num
     arg1 =op
   ?retrieval>
      state free
!eval! (and (numberp =num) (< =num 10))
==>
!bind! =new (1+ =num)
   =goal>
      step =new
   +retrieval> =op)

(p finish-rehearsing
   =goal>
      isa task
      step 10
      arg1 =op
  =retrieval>
     isa operator
     pre =watch
   =screen>
     isa screen
     state =state
==>
   =goal>
      step ready
      watch =watch
      op =op
   +retrieval> =op)

(p force-operator1
   =goal>
      isa task
      state start
      watch =watch
      step ready
      op =op
   =screen>
      isa screen
      error nil
      equation =any
      state =watch
   ?verbal>
         state free
         state error-free
   ?digital>
         state free
==>
   =goal>
      step retrieving-operator
      watch none
      state =watch
   +retrieval> =op)

(p force-operator2
   =goal>
      isa task
      state =watch
      step ready
      watch =watch
      op =op
   ?verbal>
         state free
         state error-free
   ?digital>
         state free
!safe-eval! (not (startp =watch))
==>
   =goal>
      step retrieving-operator
      watch none
      state =watch
   +retrieval> =op)


(p find-operator1
   =goal>
      isa task
      state start
      watch =watch
      step ready
   =screen>
      isa screen
      error nil
      equation =any
      state =state
   ?digital>
         state free
!safe-eval! (not (equal =watch =state))
==>
   =goal>
      step retrieving-operator
      state =state
   +retrieval>
     isa operator
   - post =state
     pre =state)

(p find-operator2
   =goal>
      isa task
      state =state
      step ready
      watch =watch
   ?digital>
         state free
!safe-eval! (not (equal =watch =state))
!safe-eval! (not (startp =state))
==>
   =goal>
      step retrieving-operator
      state =state
   +retrieval>
     isa operator
   - post =state
     pre =state)



(p* generic-bind&click
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action =command
           arg1 =var1
           arg2 =var2
           post =post
           command-type bind&click   
    =label>
       isa assoc
       =var2 =val2
  ?digital>
       state free
   ==>
   +graphical>
     isa box
     command =command
     val =val2
   =goal>
     step bind&click
     arg1 =var1
     post =post)

(p* collect&click
   =goal>
      isa   task
      post  =post
      step  bind&click
      arg1  =var
   =graphical>
      isa   box
      val   =val
   =label>
      isa  assoc 
!safe-eval! (not (equal =val 'failure))
==>
   =label>
      =var      =val
   +digital>
     isa        digital
     command    click
     item       =val
   =goal>
      step      ready
      state     =post
      post      nil
      arg1      nil
      arg2      nil)

(p* forget-click
   =goal>
       isa task
       post =post
       step bind&click
       arg1 =var
   =graphical>
       isa box
       val failure
   =label>
       isa assoc
==>
  =label>
    =var  empty
   =goal>
      step ready
      state =post
      post nil
      arg1 nil
      arg2 nil)


(p* generic-find
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action =command
           arg1 =var1
           arg2 =var2
           post =post
           command-type find   
  =label>
     isa assoc
     =var2 =val2  
  ?digital>
       state free
!safe-eval! (graphical-command =command)
   ==>
   +graphical>
     isa box
     command =command
     val =val2
   =goal>
     step looking
     arg1 =var1
     post =post)

(p* generic-find-choose
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action =command
           arg1 =var1
           arg2 =path1
           post =path2
           command-type find-choose   
    =label>
           isa assoc
           =var1 =val1        
  ?digital>
       state free
   ==>
   +graphical>
     isa box
     command =command
     val =val1
   =goal>
     step looking
     arg1 dummy
     arg2 =path1
     post =path2)



(p* collect-graphical-result
   =goal>
       isa task
       post =post
       step looking
       arg1 =var
   =graphical>
       isa box
       val =val
   =label>
        isa assoc
!safe-eval! (not (equal =val 'failure))
==>
 =label>
     =var =val
   =goal>
      step ready
      state =post
      post nil
      arg1 nil
      arg2 nil)

(p other-path
   =goal>
       isa task
       post =post
       step looking
       arg1 =var
       arg2 =path
   =graphical>
       isa box
       val failure
==>
   =goal>
      step ready
      state =path
      post nil
      arg1 nil
      arg2 nil)

(p* digital-act-label
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action =action
           arg1 =var1
           arg2 variable
           post =post
           command-type acting
    =label>
        isa assoc
        =var1 =val1
    ?digital>
       state free
   ==>
   +digital>
     isa digital
     command =action
     item =val1
     state =post
   =goal>
      step ready
      state start
      post nil
      arg1 nil)

(p digital-act-no-label
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action =action
           arg1 =val1
           arg2 string
           post =post
           command-type acting
    ?digital>
       state free
   ==>
   +digital>
     isa digital
     command =action
     item =val1
     state =post
   =goal>
      step ready
      state start
      post nil
      arg1 nil)

(p digital-act-partr
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action =action
           arg1 =val1
           arg2 string
           post =post
           command-type acting-part
    ?digital>
       state free
   ==>
   +digital>
     isa digital
     command =action
     item =val1
   =goal>
      step ready
      state =post
      post nil
      arg1 nil)


(p* generic-find&act
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action =command
           arg1 =var1
           arg2 =action
           post =post
           command-type find&act   
    =label>
         isa assoc
        =var1 =val1
  ?digital>
       state free
   ==>
   +graphical>
     isa box
     command =command
     val =val1
   =goal>
      step do-digital-buffer
      action =action
      post =post
      arg1 graphical)

(p* generic-combine-act
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action =command
           arg1 =var1
           arg2 =var2
           post =post
           command-type combine-act      
   =label>
      isa assoc
      =var1 =val1
      =var2 =val2
  ?digital>
       state free
   ==>
   +graphical>
     isa box
     command =command
     val =val1
     val1 =val2
   =goal>
      step do-digital-buffer
      action type-part
      post =post
      arg1 graphical)

(p* retrieve-inverse
   =goal>
      isa task
      step retrieving-operator
    =retrieval>
           isa operator
           action inverse
           arg1 =var1
           arg2 =var2
           post =post
           command-type find
  =label>
      isa assoc
      =var2 =val
  ?digital>
       state free
   ==>
!safe-bind! =newval (sign-it =val)
   +retrieval>
     isa inverse
     first =newval
   =goal>
      step harvest-retrieval
      arg1 =var1
      post =post)

(P* COLLECT-retrieval-RESULT
   =GOAL>
       ISA TASK
       POST =POST
       STEP harvest-retrieval
       ARG1 =VAR
   =retrieval>
       ISA inverse
       second =VAL
   =LABEL>
       ISA ASSOC
   !SAFE-EVAL! (NOT (EQUAL =VAL 'FAILURE))
 ==>
   =LABEL>
       =VAR =VAL
   =GOAL>
       STEP READY
       STATE =POST
       POST NIL
       ARG1 NIL
       ARG2 NIL
)

(p do-digital
   =goal>
      isa task
      step do-digital
      action =action
      arg1 =var1
      post =post
    ?digital>
       state free
   ==>
   +digital>
     isa digital
     command =action
     item =var1
     state =post
   =goal>
      step ready
      state start
      post nil
      action nil
      arg1 nil)

(p do-digital-graphical
   =goal>
      isa task
      step do-digital-buffer
      action =action
      arg1 graphical
      post =post
   =graphical> 
      isa box
      val =var
    ?digital>
       state free
   ==>
   +digital>
     isa digital
     command =action
     item =var
   =goal>
      step ready
      state =post
      post nil
      action nil)

(p do-digital-retrieval
   =goal>
      isa task
      step do-digital-buffer
      action =action
      arg1 retrieval
      post =post
   =retrieval>
       isa inverse
       second =var
    ?digital>
       state free
   ==>
   +digital>
     isa digital
     command =action
     item =var
   =goal>
      step ready
      state =post
      post nil
      action nil)

;; EVALUATE
;; Sends a request to the mathematical buffer to evaluate the content of
;; a box.  The request is interpreted and the result of the evaluation is
;; stored into the 'result' store of the chunk.
;; 
(p* evaluate
   =goal>
      isa     task
      step    retrieving-operator
   =retrieval>
      isa     operator
      action  evaluate
      arg1    =var1
      arg2    =var2
      post    =post
   =label>
      isa assoc
      =var2 =val2
   ==>
   =goal>
      step    evaluating
      arg1    =var1
      post    =post
   +mathematical>
      isa     verbage 
      command evaluate-box
      arg1    =val2)

(p* collect-mathematical-result
   =goal>
      isa task
      post =post
      step evaluating
      arg1 =var
   =mathematical>
      isa verbage
      result =val
   =label>
      isa assoc
   ==>
   =label>
      =var =val
   =goal>
      step ready
      state =post
      post nil
      arg1 nil
      arg2 nil)

(spp :successes 50000 :failures 0 :efforts 5000000)
)



