;;; Utility functions for converting a box tree to a list, walking a box tree, finding children, etc.

(in-package :tutor)

(defgeneric find-tree (tree fn)
  (:documentation "checks each node in box-tree. Returns the node that satisfies fn."))

(defmethod find-tree ((tree x-box) fn)
  (if (funcall fn tree)
      tree))

(defmethod find-tree ((tree number) fn)
  (declare (ignore tree fn))
  nil)

(defmethod find-tree ((tree arg-box) fn)
  (if (funcall fn tree)
      tree
    (or (find-tree (arg-value (box-arg1 tree)) fn)
	(find-tree (arg-value (box-arg2 tree)) fn))))

(defmethod find-tree ((tree answer-box) fn)
  (if (funcall fn tree)
      tree
    (find-tree (arg-value (answer tree)) fn)))

(defmethod find-tree ((tree number-box) fn)
  (when (funcall fn tree)
    tree))

(defgeneric accum-tree (box fn)
  (:documentation "Return all nodes that satisfy fn"))

(defmethod accum-tree ((box number) fn)
  (declare (optimize (debug 3)))
  (declare (ignore box fn))
  nil)

(defmethod accum-tree ((box x-box) fn)
  (declare (optimize (debug 3)))
  (declare (ignore box fn))
  nil)

(defmethod accum-tree ((box answer-box) fn)
  (declare (optimize (debug 3)))
  (accum-tree (arg-value (answer box)) fn))

(defmethod accum-tree ((box number-box) fn)
  (declare (optimize (debug 3)))
  (if (ignore-errors (funcall fn box))
      (cons box (accum-tree (arg-value (number box)) fn))
    (accum-tree (arg-value (number box)) fn)))

(defmethod accum-tree ((box arg-box) fn)
  (declare (optimize (debug 3)))
  (labels ((rstep (box fn)
		  ;; The recursive step
		  (nconc (and (typep (arg-value (box-arg1 box)) 'box)
			      (accum-tree (arg-value (box-arg1 box)) fn))
			 (and (typep (arg-value (box-arg2 box)) 'box)
			      (accum-tree (arg-value (box-arg2 box)) fn)))))
    (if (funcall fn box)
	(cons box (rstep box fn))
      (rstep box fn))))

(defun accum-all-tree (root test)
  "Accumulate boxes from the tree that satisfy TEST. This is run on all the boxes."
  (when (funcall test root)
    (cons root
          (apply 'nconc (mapcar (lambda (b)
                                  (accum-all-tree b test))
                                (only-boxes (children root)))))))

(defun find-eqn (box-tree point)
  "Find the equation that contains point"
  (find-tree box-tree (lambda (x) (rlet ((r :rect :topleft (make-point (box-x x) 
								       (box-y x))
					    :bottomright (make-point (+ (box-x x) +box-width+)
								     (+ (box-y x) +box-height+))))
					(point-in-rect-p r point)))))

(defun parent (box)
  "Return a single parent of BOX."
  (first (box-parents box)))


(defun root-node (box)
  "Return the root node of the tree BOX is part of."
  (if (box-parents box)
      (root-node (first (box-parents box)))
    box))

(defun only-boxes (list)
  "Remove all elements that are not boxes."
  (remove-if-not (lambda (x) (typep x 'box)) list))

(defun box-which-arg (box fn)
  "Return the arg that satisfies fn first. the other arg is the 2nd returned."
  (if (funcall fn (box-arg1 box))
      (values (box-arg1 box) (box-arg2 box))
    (values (box-arg2 box) (box-arg1 box))))

(defun box-other-arg (box arg)
  "Return the argument that isn't arg."
  (if (eq (box-arg1 box) arg)
      (box-arg2 box)
    (box-arg1 box)))

(defgeneric box-to-tree (box &optional mark-selected box-ids))

(defmethod box-to-tree ((box arg-box) &optional mark-selected box-ids)
  "Create a tree out of box and its children. If MARK-SELECTED is t
then add a fourth list item, 'SELECTED, to selected boxes."
  (nconc (when box-ids
	   (list (box-id box)))
	 (list (arg-value (box-op box))
	       (box-to-tree (arg-value (box-arg1 box)) mark-selected box-ids)
	       (box-to-tree (arg-value (box-arg2 box)) mark-selected box-ids))
	 (when (and mark-selected
		    (box-selected box))
	   (list :selected))))

(defmethod box-to-tree ((box x-box) &optional mark-selected  box-ids)
  (declare (ignore mark-selected box-ids))
  'x)

(defmethod box-to-tree ((box answer-box) &optional mark-selected box-ids)
  "Create a tree out of box and its children. If MARK-SELECTED is t
then add a fourth list item, 'SELECTED, to selected boxes."
  ;; A bit gross.
  (nconc (when box-ids 
	   (list (box-id box)))
	 (list (box-to-tree (arg-value (answer box)) mark-selected box-ids))))

(defmethod box-to-tree ((box number-box) &optional mark-selected box-ids)
  ;; A bit gross.
  (nconc (when box-ids 
	   (list (box-id box)))
	 (list (box-to-tree (arg-value (number box)) mark-selected box-ids))))

(defmethod box-to-tree ((box number) &optional mark-selected box-ids)
  (declare (ignore mark-selected box-ids))
  box)

(defmethod box-to-tree ((box symbol) &optional mark-selected box-ids)
  (declare (ignore mark-selected box-ids))
  box)

(defgeneric box-to-tree-with-editables (box))

(defmethod box-to-tree-with-editables ((box arg-box))
  "Create a tree out of box and its children. If MARK-SELECTED is t
then add a fourth list item, 'SELECTED, to selected boxes."
  (if (and (arg-editable (box-arg1 box))
           (arg-editable (box-arg2 box))
           (arg-editable (box-op box))
           (box-user-input box))
      (box-user-input box)
    (list (arg-value-or-editable (box-op box))
          (box-to-tree-with-editables (arg-value-or-editable (box-arg1 box)))
          (box-to-tree-with-editables (arg-value-or-editable (box-arg2 box))))))

(defmethod box-to-tree-with-editables ((box x-box))
 'x)

(defmethod box-to-tree-with-editables ((box answer-box))
  "Create a tree out of box and its children. If MARK-SELECTED is t
then add a fourth list item, 'SELECTED, to selected boxes."
  ;; A bit gross.
  (list (box-to-tree-with-editables (arg-value-or-editable (answer box)))))

(defmethod box-to-tree-with-editables ((box number-box))
  (list (box-to-tree-with-editables (arg-value-or-editable (number box)))))

(defmethod box-to-tree-with-editables ((box number))
  box)

(defmethod box-to-tree-with-editables ((box symbol))
  box)


(defgeneric copy-boxes (box parent &optional x-box))

(defmethod copy-boxes ((box arg-box) parent &optional (x-box (make-instance 'x-box)))
  ;; Use type-of so it creates the box as an equal-box if it is one
  ;; (which is also an arg-box).
  (let ((b (make-instance (type-of box)
			  :parents (when parent (list parent))
			  :x (box-x box)
			  :y (box-y box)
			  :nleaves (box-nleaves box)
			  :depth (box-depth box)
			  :lchild-width (box-lchild-width box)
			  :id (box-id box)
			  :op (make-instance 'operator-arg :value (arg-value (box-op box)))
			  :arg1 (make-instance 'argument)
			  :arg2 (make-instance 'argument))))
    (setf (arg-value (box-arg1 b)) (copy-boxes (arg-value (box-arg1 box)) b x-box)
	  (arg-value (box-arg2 b)) (copy-boxes (arg-value (box-arg2 box)) b x-box))
    b))

(defmethod copy-boxes ((box x-box) p &optional (x-box (make-instance 'x-box)))
  (declare (ignore box))
  (push p (box-parents x-box))
  x-box)

(defmethod copy-boxes ((box answer-box) p &optional (x-box (make-instance 'x-box)))
  (let ((b (make-instance 'answer-box
			  :parents (when p (list p))
			  :x (box-x box)
			  :y (box-y box)
			  :id (box-id box)
			  :nleaves (box-nleaves box)
			  :depth (box-depth box)
			  :answer (make-instance 'argument))))
    (setf (arg-value (answer b)) (copy-boxes (arg-value (answer box)) b x-box))
    b))

(defmethod copy-boxes ((box number-box) p &optional (x-box (make-instance 'x-box)))
  (let ((b (make-instance 'number-box
			  :parents (list p)
			  :x (box-x box)
			  :y (box-y box)
			  :id (box-id box)
			  :nleaves (box-nleaves box)
			  :depth (box-depth box)
			  :number (make-instance 'argument))))
    (setf (arg-value (number b)) (copy-boxes (arg-value (number box)) b x-box))
    b))

(defmethod copy-boxes ((box number) p &optional (x-box (make-instance 'x-box)))
  (declare (ignore p x-box))
  box)

			  
(defun find-root (boxes)
  "Given a list of boxes return the parent of them all. Assumes
they're all connected."
  (labels ((root (b boxes)
		 (cond ((null boxes) b)
		       (t
			(let ((p (car (member (parent b) boxes))))
			  (if p 
			      (root p (remove p boxes))
			    b))))))
    (root (car boxes) (cdr boxes))))

(defun find-root-verify-rest (boxes)
  "Return the box that is the root for all other nodes. Verify that
the other boxes are children of one another or the root node."
  (labels ((verify (r boxes)
		   (cond ((null boxes) nil)
			 (t
			  (let ((c1 (car (member (arg-value (box-arg1 r)) boxes)))
				(c2 (car (member (arg-value (box-arg2 r)) boxes))))
			    (setf boxes (remove c1 boxes))
			    (setf boxes (remove c2 boxes))
			    (when c1
			      (setf boxes (intersection boxes (verify c1 boxes))))
			    (when c2
			      (setf boxes (intersection boxes (verify c2 boxes))))
			    boxes)))))

    (let ((r (find-root boxes)))
      (when (null (verify r (remove r boxes)))
	r))))


(defun arg-input-filled-in-p (arg)
  "Helper fn for input-filled-in-p."
  (if (arg-editable arg)
      (when (arg-user-answer arg)
	(input-filled-in-p (arg-value arg)))
    (input-filled-in-p (arg-value arg))))

(defgeneric input-filled-in-p (root)
  (:documentation "Return true if all input fields have data in them."))

(defmethod input-filled-in-p ((box number))
  (declare (ignore box))
  t)

(defmethod input-filled-in-p ((box symbol))
  (declare (ignore box))
  t)

(defmethod input-filled-in-p ((box x-box))
  (declare (ignore box))
  t)

(defmethod input-filled-in-p ((box arg-box))
  (or (box-user-input box)
      (and (arg-input-filled-in-p (box-arg1 box))
           (arg-input-filled-in-p (box-op box))
           (arg-input-filled-in-p (box-arg2 box)))))

(defmethod input-filled-in-p ((box equal-box))
  (and (arg-input-filled-in-p (box-arg1 box))
       (arg-input-filled-in-p (box-arg2 box))))

(defmethod input-filled-in-p ((box answer-box))
  (arg-input-filled-in-p (answer box)))

(defmethod input-filled-in-p ((box number-box))
  (arg-input-filled-in-p (number box)))

(defun selected-boxes ()
  "Returns a list of selected boxes"
  (accum-tree (tutor-eqn-root *tutor*) #'box-selected))

(defun tree-x-box (root)
  "Return the x-box of the tree with the root node, ROOT."
  (if (typep root 'x-box)
      root
    (dolist (c (only-boxes (children root)))
      (let ((box (tree-x-box c)))
	(when box
	  (return-from tree-x-box box))))))

(defgeneric listify-tree (box)
  (:documentation "Return all boxes in a list."))

(defmethod listify-tree ((box x-box))
  (list box))

(defmethod listify-tree ((box answer-box))
  (cons box (and (typep (arg-value (answer box)) 'box)
		 (listify-tree (arg-value (answer box))))))

(defmethod listify-tree ((box number-box))
  (cons box (and (typep (arg-value (number box)) 'box)
		 (listify-tree (arg-value (number box))))))

(defmethod listify-tree ((box arg-box))
  (labels ((rstep (box)
		  ;; The recursive step
		  (nconc (and (typep (arg-value (box-arg1 box)) 'box)
			      (listify-tree (arg-value (box-arg1 box))))
			 (and (typep (arg-value (box-arg2 box)) 'box)
			      (listify-tree (arg-value (box-arg2 box)))))))
    (cons box (rstep box))))


(defgeneric symbolize-box (box))

(defmethod symbolize-box ((box number))
  box)

(defmethod symbolize-box ((box box))
  (box-id box))

(defun arg-container (arg root)
  "Find the box that arg is contained in."
  (cond ((typep root 'answer-box)
	 (if (eq arg (answer root))
	     root
	   (arg-container arg (arg-value (answer root)))))
	((typep root 'arg-box)
	 (if (or (eq arg (box-arg1 root))
		 (eq arg (box-arg2 root))
		 (eq arg (box-op root)))
	     root
	   (or (arg-container arg (arg-value (box-arg1 root)))
	       (arg-container arg (arg-value (box-arg2 root))))))))

(defun box-containers (box)
  "Find the arguments that contain box."
  (labels ((doit (p)
             (cond ((typep p 'answer-box)
		    (answer p))
		   ((typep p 'arg-box)
		    (cond ((eq box (arg-value (box-arg1 p)))
			   (box-arg1 p))
		    ((eq box (arg-value (box-arg2 p)))
		     (box-arg2 p)))))))
    (let ((p (box-parents box)))
      (mapcar #'doit p))))

(defun dump-box (box)
  "Return box as a list of the form (ID ... (properties))."
  (let (props)
    (list (box-id box)
          (cond ((typep box 'arg-box)
                 (let ((a1 (box-arg1 box))
                       (a2 (box-arg2 box))
                       (op (box-op box)))
                   (cond ((and (arg-editable a1)
                               (arg-editable a2)
                               (arg-editable op))
                          (setf props '(:box-editable))
                          (list (arg-user-answer a1)
                                (arg-user-answer op)
                                (arg-user-answer a2)))
                         ((and (arg-editable a1)
                               (arg-editable op))
                          (setf props '(:arg-op-editable))
                          (list (arg-user-answer a1)
                                (arg-user-answer op)
                                (symbolize-box (arg-value a2))))
                         ((and (arg-editable op)
                               (arg-editable a2))
                          (setf props '(:op-arg-editable))
                          (list (symbolize-box (arg-value a1))
                                (arg-user-answer op)
                                (arg-user-answer a2)))
                         (t
                          (list (if (arg-editable a1)
                                    (progn
                                      (push :arg1-editable props)
                                      (list (arg-id a1)
                                            (arg-user-answer a1)))
                                  (symbolize-box (arg-value a1)))
                                (if (arg-editable op)
                                    (progn
                                      (push :op-editable props)
                                      (list (arg-id op)
                                            (arg-user-answer op)))
                                  (arg-value op))
                                (if (arg-editable a2)
                                    (progn
                                      (push :arg2-editable props)
                                      (list (arg-id a2)
                                            (arg-user-answer a2)))
                                  (symbolize-box (arg-value a2))))))))
                 ((typep box 'answer-box)
                 (let ((a (answer box)))
                   (if (arg-editable a)
                       (progn
                         (push :answer-editable props)
                         (arg-id a))
                     (symbolize-box (arg-value (answer box))))))
                ((typep box 'x-box)
                 :x)
                ((typep box 'number-box)
                 (symbolize-box (arg-value (number box))))
                (t (error "~a isn't covered" (type-of box))))
          (nconc (when (box-selected box)
                   (list :selected))
                 (when (box-pointed-to box)
                   (list :arrow))
                 props))))

(defun dump-equation (root)
  (let ((flat (listify-tree root)))
    (mapcar 'dump-box flat)))

(defun clear-selected ()
  (mapc (lambda (b) (setf (box-selected b) nil))
                  (selected-boxes)))

(defun box-left (box)
  (arg-value (box-arg1 box)))

(defun box-right (box)
  (arg-value (box-arg2 box)))

(defun selected-subtree (root)
  "Return the subtree of ROOT that is selected in list form."
  (labels ((skip-to-selection (tree)
                              (cond ((and (consp tree)
                                          (= (length tree) 1))
                                     (skip-to-selection (first tree)))
                                    ((consp tree)
                                     (if (find :selected tree)
                                         tree
                                       (or (skip-to-selection (second tree))
                                           (skip-to-selection (third tree)))))))

           (rip-selected-subtree (tree)
                                 (when (and (listp tree)
                                            (find :selected tree))
                                   (list (first tree)
                                         (if (numberp (second tree))
                                             (second tree)
                                           (rip-selected-subtree (second tree)))
                                         (if (numberp (third tree))
                                             (third tree)
                                           (rip-selected-subtree (third tree)))))))
    (let ((tree (box-to-tree root t)))
      (rip-selected-subtree (skip-to-selection tree)))))

(defun make-subtree (tree)
  "Create a subtree made of boxes."
  (labels ((make (tree parent)
                 (cond ((null tree) nil)
                       ((numberp tree) tree)
                       ((and (listp tree) (= (length tree) 1))
                        ;; This is a special case. its a number box.
                        (let ((nb (make-instance 'number-box
                                                 :parents (list parent)
                                                 :number (make-instance 'argument))))
                          ;; Sorta hacky. How else can we do it though?
                          (setf (arg-value (number nb)) (make (first tree) nb))
                          nb))
                       (t
                        (let ((b (make-box (first tree) nil nil parent)))
                          (setf (arg-value (box-arg1 b)) (make (second tree) b)
                                (arg-value (box-arg2 b)) (make (third tree) b))
                          b)))))
    (make tree nil)))


(defun box-editable-p (box)
  "box must be an arg box."
  (and (arg-editable (box-arg1 box))
       (arg-editable (box-op box))
       (arg-editable (box-arg2 box))))

(defun arg-op-editable-p (box)
  "only the left arg and op are editable. box must be an arg box."
  (and (arg-editable (box-arg1 box))
       (arg-editable (box-op box))
       (not (arg-editable(box-arg2 box)))))

(defun op-arg-editable-p (box)
  "only the op and right arg are editable.. box must be an arg box."
  (and (not (arg-editable (box-arg1 box)))
       (arg-editable (box-op box))
       (arg-editable(box-arg2 box))))

(defun op-editable-p (box)
  "only the op is editable. box must be an arg box."
  (and (not (arg-editable (box-arg1 box)))
       (arg-editable (box-op box))
       (not (arg-editable(box-arg2 box)))))

(defun all-editable-p (box)
  (and (arg-editable (box-arg1 box))
       (arg-editable (box-op box))
       (arg-editable (box-arg2 box))))