;;; the algebra isomorph

;; Each box is of the form XX + YY where XX can be a number or ?. When
;; it's a ? it means it needs to be filled in.
;;
;; Lines connecting boxes go from the the center of the operator to
;; the center of the ?.
;;
;; Maintain a history of the graph so users can go forward and back.

;; Sample eqn: (* (+ 4 5) 6)

;;        +-------+
;;        | 4 + 5 |
;;        +-------+
;;            \
;;             \
;;              \
;;             +-------+
;;             | ? * 6 |
;;             +-------+

;; How Scoring Works
;;
;; Each operation "cycle" is worth 7 cents. a problem is worth 7 *
;; no. of operations. during an operation cycle if the user
;; incorrectly fills in any of the green boxes they lose 7 cents. If
;; they click an operation that can't be done with the selected boxes
;; they lose 3 cents. If they use a hint they lose 4 cents.

(in-package :tutor)
  
(defun box-p (arg)
  (typep (arg-value arg) 'box))

(defun arg-transfer (arg)
  "transfer the input to the value."
  (when (arg-editable arg)
    (setf (arg-editable arg) nil
	  (arg-value arg) (arg-user-answer arg))
    (when (arg-widget arg)
      (remove-subviews (view-container (arg-widget arg)) (arg-widget arg)))
;;     (event-dispatch)
    (setf (arg-widget arg) nil)))

;; (defun blah (arg1 op arg2)
;;   (mapcar #'list arg1 op arg2))

(defun make-answers (&rest answers)
  "convert between '((1 + 2) ...) and '((1 ...) (+ ...) (2 ...)) and back."
  (apply #'mapcar #'list (mapcar (lambda (e) (if (atom e) (list e) e)) answers)))

(defgeneric transfer-from-widget (box)
  (:documentation "transfer the value of the edit box to the arg-value for any editable arguments in the box."))

(defmethod transfer-from-widget ((box answer-box))
  ;; Check if they got it right
  (arg-transfer (answer box)))

(defmethod transfer-from-widget ((box arg-box))
  (if (box-user-input box)
      (let* ((p (parent box)))
        (etypecase p
          (arg-box
           (if (box-left-child-p box p)
               (setf (arg-value (box-arg1 p)) (box-user-input box))
             (setf (arg-value (box-arg2 p)) (box-user-input box))))
          (answer-box
           (setf (arg-value (answer p)) (box-user-input box))))
        (setf (box-user-input box) nil))
    (progn
      (arg-transfer (box-arg1 box))
      (arg-transfer (box-arg2 box))
      (arg-transfer (box-op box)))))

(defgeneric transfer-editables (box))
(defmethod transfer-editables ((box number)))
(defmethod transfer-editables ((box x-box)))
(defmethod transfer-editables ((box number-box)))
(defmethod transfer-editables ((box answer-box))
  (transfer-from-widget box)
  (transfer-editables (arg-value (answer box))))
(defmethod transfer-editables ((box arg-box))
  (transfer-from-widget box)
  (transfer-editables (arg-value (box-arg1 box)))
  (transfer-editables (arg-value (box-arg2 box))))
	     
(defun transfer-and-verify-args ()
  "For each box, transfer the users input and and demerits."
  (transfer-editables (tutor-eqn-root *tutor*))
  (setf (tutor-incorrect-submit *tutor*) (not (correct-answer-p))))

(defun correct-answer-p ()
  "Search each of the answer-sets for the current tree. If the
  current tree exists in an answer-set then the user's answer was
  correct."
  ;; (dformat "~a ~a~%"(tutor-submit-answers *tutor*)
;;            (box-to-tree-with-editables (tutor-eqn-root *tutor*)))
  (dolist (answers (tutor-submit-answers *tutor*))
    (when (find (box-to-tree-with-editables (tutor-eqn-root *tutor*)) answers :test #'equal)
      (return-from correct-answer-p t))))

(defun last-eqn ()
  (when (first (tutor-eqn-history-before *tutor*))
    (history-eqn (first (tutor-eqn-history-before *tutor*)))))

(defun refresh-eqn (&optional both-sides)
  "Refresh the display of the equation."
  (if (tutor1-p)
      (tutor1-refresh-eqn both-sides)
    (tutor2-refresh-eqn both-sides)))

(defun draw-simplified-ui-buttons (container)
  "Draw the buttons in the view and resize the view to fit them."
    (let* ((size #@(100 25))
           (view (make-instance 'view
                                :view-container container))
           (row1 (mapcar (lambda (o)
                               (make-instance 'button-dialog-item
                                              :view-container view
                                              :view-position #@(0 0)
                                              :view-size size
                                              :dialog-item-text (op-name o)
                                              :dialog-item-action (button-fn (handle-operation o t))))
                             (list (or (find-operation 'unwind)
                                       (find-operation 'invert))
                                   (find-operation 'evaluate)
                                   (or (find-operation 'collect)
                                       (find-operation 'combine)))))
           (dis (or (find-operation 'distribute)
                    (find-operation 'reorder)))
           (sub (find-operation 'subtract))
           (row2 (list (make-instance 'button-dialog-item
                                      :view-container view
                                      :view-position #@(0 0)
                                      :view-size size
                                      :dialog-item-text (op-name dis)
                                      :dialog-item-action (button-fn (handle-operation dis t)))
                       (make-instance 'button-dialog-item
                                      :view-container view
                                      :view-position #@(0 0)
                                      :view-size size
                                      :dialog-item-text (op-name sub)
                                      :dialog-item-action (button-fn (handle-operation sub t)))
                       (make-instance 'button-dialog-item
                                      :view-container view
                                      :view-position #@(0 0)
                                      :view-size size
                                      :dialog-item-text "Hint"
                                      :dialog-item-action (button-fn (handle-button :hint)))
                       (make-instance 'button-dialog-item
                                      :view-container view
                                      :view-position #@(0 0)
                                      :view-size size
                                      :dialog-item-text "Next Prob."
                                      :dialog-item-action (button-fn (handle-button :next-problem))))))
      (order-views row1 8 0)
      (order-views row2 8 35)
      (set-view-size view
                     (apply 'max (mapcar 'view-right (append row1 row2)))
                     (+ (apply 'max (mapcar 'view-bottom (append row1 row2))) 5))
      view))

(defvar *height* 0)
(defvar *width* 0)

(defun tutor2-refresh-eqn (both-sides)
  "Refresh the display of the equation."
  (declare (ignore both-sides))
  ;; last view
  (clear-click-regions)
  (event-dispatch)
  ;; (clear-message)
  (delete-subviews (tutor-eqn-view *tutor*))
  (let ((eqns (loop for i in (nconc (nreverse (mapcar 'history-eqn (tutor-eqn-history-before *tutor*)))
                                    (list (tutor-eqn-root *tutor*)))
                    collect (prog1
                                ;; Yet another hack. uugh.
                                (tutor2-redraw-tree i (tutor-eqn-view *tutor*) (eq i (tutor-eqn-root *tutor*)) (and (eq i (tutor-eqn-root *tutor*))
                                                                                                                    (not (tutor-submit-enabled *tutor*))))
                              ;; A hack to keep only the current
                              ;; equation's click regions
                              ;; active. Otherwise the user can
                              ;; highlight parts of previous equation
                              ;; states.
                              (unless (eq i (tutor-eqn-root *tutor*))
                                (clear-click-regions))))))
    (order-views-vertically eqns)
    (resize-view-for-vertically (tutor-eqn-view *tutor*) eqns)
    (tutor2-recenter-tree nil (tutor-eqn-view *tutor*))
;;     (if (tutor-submit-enabled *tutor*)
;;         (set-view-position (tutor-buttons *tutor*) 1000 1000)
    (when *simplified-ui*
      (set-view-position (tutor-buttons *tutor*)
                         (- (+ (view-x (tutor-eqn-view *tutor*)) (truncate (view-width (tutor-eqn-view *tutor*)) 2))
                            (truncate (view-width (tutor-buttons *tutor*)) 2))
                         (if *views-with-arrows*
                             (view-bottom (tutor-eqn-view *tutor*))
                           (+ (view-bottom (tutor-eqn-view *tutor*)) 20)))
      (set-view-position *message-view*
                         (view-x (tutor-buttons *tutor*))
                         (+ (view-bottom (tutor-buttons *tutor*)) 15)))
    ;; Calculate how big the window has to be.
    (setf *height* (max *height* (view-height (tutor-eqn-view *tutor*)))
          *width* (max *width* (view-width (tutor-eqn-view *tutor*))))
    (dformat "~a ~a~%" *height* *width*)))

(defun tutor1-refresh-eqn (both-sides)
  "Refresh the display of the equation."
  (clear-click-regions)
  ;; This logic is pretty messy. We want to either delete the last
  ;; view or draw the last selected boxes in simplified-ui mode. And,
  ;; otherwise, we want to keep the last-eqn there unless both-sides
  ;; is true, in which case we want to redraw it (it's changed).
  (when (or both-sides
            (and *simplified-ui*
                 (not (tutor-submit-enabled *tutor*))))
    (delete-subviews (tutor-last-eqn-view *tutor*))
    ;; Last view.  for simplified ui, only draw it when we're in
    ;; submit mode.
    (unless (and *simplified-ui*
                 (not (tutor-submit-enabled *tutor*)))
      (when (tutor-eqn-history-before *tutor*)
        (when (tree-x-box (last-eqn))
          (setf (x-box-reposition (tree-x-box (last-eqn))) t))
        (uber-respace-tree (last-eqn))
        (tutor1-recenter-tree (last-eqn) (tutor-last-eqn-view *tutor*))
        ;; simplified-ui only displays the selected boxes
        (when *simplified-ui*
          (resize-view-for-boxes (tutor-last-eqn-view *tutor*) (tutor-last-selected-boxes *tutor*)))
        (tutor1-redraw-tree (last-eqn) (tutor-last-eqn-view *tutor*)))))
    
  ;; Current view
  (event-dispatch)
  ;;   (erase-rect *window* +side-bar-x+ 339 (+ +side-bar-x+ +side-bar-width+) 358)
  ;;(clear-message t)
  ;;(clear-arrows)
  (delete-subviews-if (tutor-eqn-area-view *tutor*) (lambda (v)
                                                      (not (find v (list (tutor-eqn-view *tutor*)
                                                                         (tutor-buttons *tutor*)
                                                                         *message-view*)))))
  (when (tree-x-box (tutor-eqn-root *tutor*))
    (setf (x-box-reposition (tree-x-box (tutor-eqn-root *tutor*))) t))
  (uber-respace-tree (tutor-eqn-root *tutor*))
  (delete-subviews (tutor-eqn-view *tutor*))
  (if *simplified-ui*
      ;; this is a centering hack to make sure elements don't overlap.
      (tutor1-recenter-tree (tutor-eqn-root *tutor*) (tutor-eqn-view *tutor*) t (- (view-width (tutor-eqn-area-view *tutor*)) 150) (- (view-height (tutor-eqn-area-view *tutor*)) 50))
    (tutor1-recenter-tree (tutor-eqn-root *tutor*) (tutor-eqn-view *tutor*)))
  (tutor1-redraw-tree (tutor-eqn-root *tutor*) (tutor-eqn-view *tutor*))
  ;;   (event-dispatch)
  (unless (tutor-submit-enabled *tutor*)
    (register-boxes (tutor-eqn-root *tutor*)))

  (when *simplified-ui*
    ;; only adjust the position when the eqn pushes the buttons down.
;;     (when (<= (view-y (tutor-buttons *tutor*))
;;               (view-bottom (tutor-eqn-view *tutor*)))
      (set-view-position (tutor-buttons *tutor*)
                         (- (+ (view-x (tutor-eqn-view *tutor*)) (truncate (view-width (tutor-eqn-view *tutor*)) 2))
                            (truncate (view-width (tutor-buttons *tutor*)) 2))
                         (view-bottom (tutor-eqn-view *tutor*)))
      (set-view-position *message-view*
                         (view-x (tutor-buttons *tutor*))
                         (+ (view-bottom (tutor-buttons *tutor*)) 15))
    (set-view-position (tutor-last-eqn-top-view *tutor*)
                       (view-right (tutor-eqn-view *tutor*))
                       ;; top align the last-eqn, or if the eqn isn't
                       ;; tall enough, bottom align it.
                       (min
                        (let* ((p (convert-coordinates (make-point 0 (apply 'min (mapcar 'box-y (accum-all-tree (tutor-eqn-root *tutor*) 'identity))))
                                                       (tutor-eqn-view *tutor*)
                                                       (view-container (tutor-last-eqn-top-view *tutor*)))))
                          (- (point-v p) +pad+))
                        ;;(view-y (tutor-eqn-view *tutor*))
                        (let ((p (convert-coordinates (make-point 0 (+ (box-y (tutor-eqn-root *tutor*)) +box-height+))
                                                      (tutor-eqn-view *tutor*)
                                                      (view-container (tutor-last-eqn-top-view *tutor*)))))
                          (+ (- (point-v p) (view-height (tutor-last-eqn-top-view *tutor*)))
                             +pad+))))
    )
)

(defun toggle-select-box (box)
  (setf (box-selected box) (not (box-selected box)))
  (if (box-selected box)
      (history-select box)
    (history-unselect box)))

(defun redraw-selected-box (box)
  (if (tutor1-p)
      (draw-box box (tutor-eqn-view *tutor*))
    (refresh-eqn)))

(defun click-box (box)
  (when (and box (not (tutor-submit-enabled *tutor*)))
    (toggle-select-box box)
    (redraw-selected-box box)
    ;; call what's next
    (when (auto-whats-next-mode)
      (do-whats-next (auto-whats-next-mode)))))

;; (defun tutor1-click-box (box)
;;   (when (and box (not (tutor-submit-enabled *tutor*)))
;;     (toggle-select-box box)
;;     (draw-box box (tutor-eqn-view *tutor*))
;;     ;; call what's next
;;     (when (auto-whats-next-mode)
;;       (do-whats-next))))

(defmethod view-click-event-handler ((view box-view) where)
  (dformat "click box-view ~a!~%" view)
  (if (eq view (tutor-eqn-view *tutor*))
      (handle-click-event (point-h where) (point-v where))))
    ;;(call-next-method)))

(defmethod view-click-event-handler ((view region-view) where)
  (declare (ignore where))
  (dformat "click region-view!~%")
  (handle-click-region view))

(defun handle-button (but)
  (dformat "handle-button: ~a~%" but)
  (case but
    ;;     (submit
    ;;      (handle-submit nil))
    (:first-mistake
     (handle-first-mistake))
    (:back
     (handle-history-nav t nil))
    (:forward
     (handle-history-nav t t))
    (:next-problem
     (handle-problem-end t))
    (:hint
     ;; FIXME: should minus be included in hints?
     (do-whats-next :all))
    (t (error "Unknown button")))
  ;; call what's next
  (when (and (not (eq but :next-problem))
             (auto-whats-next-mode))
    (do-whats-next (auto-whats-next-mode))))

(defun advance-after-fill ()
  "Call this after a user filled in a value. It will advance the
  problem if all values have been filled in."
  ;; If all the green boxes are filled in, submit it. Unless we're
  ;; in auto-whats-next mode. In that case they can't move on until
  ;; the've typed the correct answer.
  (event-dispatch)
  (when (input-filled-in-p (tutor-eqn-root *tutor*))
    (if (or (and (not (auto-whats-next-mode))
                 (not *force-correct-fill-in*))
            (and (or (auto-whats-next-mode)
                     *force-correct-fill-in*)
                 (correct-answer-p)))
        (progn
          (handle-submit)
          ;; We need to wrap the equation refresh in this because
          ;; the get-string-from-user uses a seperate process
          ;; which for some reason makes the refresh go really
          ;; slow. So we've got to make sure we're the only
          ;; process running.
          (without-interrupts
           (without-event-processing
            (refresh-eqn nil)))
          (when *0back-after-submit*
            (0back *window* *0back-iterations* *0back-on-delay* *0back-off-delay*))
          t)
      (progn
        (history-add-bad-submit)
        (when (and (not (auto-whats-next-mode))
                   *force-correct-fill-in*)
          (set-message-text "Your answer is incorrect."))
        nil))))

(defun refresh-after-input (input)
  "Depending on hte tutor, call the appropriate update function."
  (if (tutor1-p)
      (etypecase input
        (box (redraw-box-answer input))
        (argument (redraw-arg-answer input)))
    (refresh-eqn)))

(defun post-fill (input box)
  "Given input for a box, fill in the box args with it and do the Next Thing."
  (dformat "input: ~a~%" input)
  (history-add-box-input box input)
  (when (and (not (eq input :cancel))
             (or ;; special case
                 (numberp input)
                 (= (length input) (length (editables box)))))
    (dformat "here ~a~%" input box)
    (when (if (numberp input)
              (setf (box-user-input box) input)
            (progn
              ;; we have to clear this if they later input a full expression.
              (setf (box-user-input box) nil)
              (loop named fill
                    for arg in (editables box)
                    for b in input
                    do (if (typep arg 'operator-arg)
                           (if (find b '(+ - * / =))
                               (setf (arg-user-answer arg) b)
                             (return-from fill nil))
                         (if (numberp b)
                             (setf (arg-user-answer arg) b)
                           (return-from fill nil)))
                    finally (return-from fill t))))
      (unless (advance-after-fill)
        (refresh-after-input box))))
  ;; call what's next
  (when (auto-whats-next-mode)
    (do-whats-next (auto-whats-next-mode))))

(defun handle-fill-box (box)
  (history-add-open-dialog 'box (box-id box))
  (read-box (lambda (input) 
              (post-fill input box))
            box))

(defun handle-fill-arg-op (box)
  (history-add-open-dialog 'arg-op (box-id box))
  (read-arg-op (lambda (input)
                 ;;(format t "arg-op: ~a~%" input)
                 (post-fill input box))
               box))

(defun handle-fill-op-arg (box)
  (history-add-open-dialog 'op-arg (box-id box))
  (read-op-arg (lambda (input)
                 ;;(format t "arg-op: ~a~%" input)
                 (post-fill input box))
               box))

(defun handle-fill-arg (arg)
  "If VAL is nil then prompt the user. Otherwise, use it."
  (labels ((post-fill (input)
                      ;; 
                      (dformat "input: ~a~%" input)
                      (when input
                        (setf (arg-user-answer arg) input))
                      (history-add-input arg input)
                      ;; Check if the fill-in stage is done
                      (unless (advance-after-fill)
                        (refresh-after-input arg)
;;                         (refresh-eqn)
                        ;; (redraw-arg-answer arg)
                        )
                      ;; call what's next
                      (when (auto-whats-next-mode)
                        (do-whats-next (auto-whats-next-mode)))))
    (history-add-open-dialog 'arg (arg-id arg))
    (when (arg-editable arg)
      (if (typep arg 'operator-arg)
          (read-operator #'post-fill arg)
        (read-number #'post-fill arg)))))

(defun handle-first-mistake ()
  "Go back to the eqn flagged as incorrect."
  (if (tutor-submit-enabled *tutor*)
      (history-first-mistake-fail)
    (progn
      (if (or (tutor-incorrect-submit *tutor*)
              (find-if 'history-incorrect (tutor-eqn-history-before *tutor*)))
          (progn
            (history-first-mistake t)
            (eqn-history-beginning)
            (loop until (tutor-incorrect-submit *tutor*)
                  do (eqn-history-forward))
            (refresh-eqn t)
            (set-message-text 
             (if (tutor1-p)
                 (format nil "Here is the diagram after your first mistake.~%Once you have found your error click the <- button,~%then redo the operation correctly.")
               (format nil "Here is the expression after your first mistake.~%Once you have found your error click the <- button,~%then redo the operation correctly."))))
        (progn
          (history-first-mistake nil)
          (set-message-text "No mistakes were made up to this point."))))))

(defun store-selected-boxes ()
  (dformat "storing: ~a~%" (selected-boxes))
  (setf (tutor-last-selected-boxes *tutor*) (selected-boxes)))

(defun handle-operation (op clicked)
  ;; We need to backup the tree because after op is called the tree
  ;; will be different.
  (typecase op
    (symbol (setf op (find-operation (intern (symbol-name op) "TUTOR")))))
  (unless (typep op 'operation)
    (error "Unknown operation"))
  (if (tutor-submit-enabled *tutor*)
      (history-add-opn-during-submit (operation-name op) clicked)
    (let ((tree (box-to-tree (tutor-eqn-root *tutor*) t t)))
      (store-selected-boxes)
      (if (funcall (operation-fn op))
          (progn
            (clear-message t)
            (clear-arrows)
            (eqn-history-push)
            (history-add-operation (operation-name op) clicked tree)
            (clear-selected)
            (refresh-eqn t))
        ;; 	(event-dispatch))
        ;; If the operation failed, dock points
        (progn
          (if (selected-boxes)
              (set-message-text (format nil "~a can't be done with the selected ~a." 
                                        (op-name op)
                                        (box-text (= (length (selected-boxes)) 1)
                                                  (typep (first (selected-boxes)) 'number-box))))
            (set-message-text (format nil "~a can't be done without selecting something." (op-name op))))
          (unless (auto-whats-next-mode)
            (dformat "bad operation!~%")
            (decf (tutor-problem-score *tutor*) +bad-operation-points+))
          (history-add-bad-operation (operation-name op) clicked)))
      ;; call what's next
      (when (auto-whats-next-mode)
        (do-whats-next (auto-whats-next-mode))))))

(defun invert-op (op)
  (let ((a '((/ . *) (+ . -) (- . +) (* . /))))
    (cdr (assoc op a))))

(defun handle-submit ()
  ;;   (if *submit-enabled*
  ;;       (progn
  (transfer-and-verify-args)
  ;; incentives
  (when (and (tutor-incorrect-submit *tutor*)
             (not (auto-whats-next-mode)))
    (dformat "incorrect submit!~%")
    (decf (tutor-problem-score *tutor*) +bad-fill-in+))
  ;; Some operations (evaluate) need some cleanup to be done
  (when (functionp (tutor-submit-post-fn *tutor*))
    (funcall (tutor-submit-post-fn *tutor*))
    (setf (tutor-submit-post-fn *tutor*) nil))
  (clear-message t)
  (eqn-history-backup)
  (history-add-submit)
  (setf (tutor-submit-enabled *tutor*) nil))
        
;;     (progn
;;       ;; Clicked when submit wasn't enabled. Add demerits
;;       (inc-demerits +submit-demerits+)
;;       (history-add-bad-submit))))

(defun handle-history-nav (clicked &optional forward)
  "Move backward in the history, unless FORWARD is t in which case,
move forward. This is disabled if the user is submitting values."
  (if forward
      (if (tutor-submit-enabled *tutor*)
          (history-add-failed-nav clicked forward)
	(when (tutor-eqn-history-after *tutor*)
	  (history-go-forward clicked)
	  (eqn-history-forward)
          (clear-message t)
          (clear-arrows)
	  (refresh-eqn t)))
    (when (tutor-eqn-history-before *tutor*)
      (history-go-back clicked)
      (eqn-history-back)
      (clear-message t)
      (clear-arrows)
      (refresh-eqn t))))

(defun handle-key-press (ch)
  (let ((op (find-operation ch)))
    (if op 
        (handle-operation op nil)
      (case ch
        (#\n
         (handle-problem-end nil))
        (#\BackArrow
         (handle-history-nav nil nil))
        (#\ForwardArrow
         (handle-history-nav nil t))
        (t
         ;; TODO: if its a number and we're in submit mode and there's
         ;; only 1 editable box then pop open the dialog box for them
         ;; (save them clicking).
         (history-add-keypress ch))))))

(defmethod view-key-event-handler :after ((w window) ch)
  (when (eq w *window*)
    (handle-key-press ch)))

;;; Exam stuff

(defun next-problem ()
  (if (problems (tutor-problem-set *tutor*))
      (progn
	(setf *input-dialog* nil
              (tutor-incorrect-submit *tutor*) nil
	      (tutor-eqn-history-before *tutor*) nil
	      (tutor-eqn-history-after *tutor*) nil
	      (tutor-current-problem *tutor*) (pop (problems (tutor-problem-set *tutor*)))
              (tutor-problem-score *tutor*) (* (problem-points (tutor-current-problem *tutor*)) +operation-points+)
	      (tutor-eqn-root *tutor*) (make-tree (problem-eqn (tutor-current-problem *tutor*)))
	      (tutor-auto-whats-next-mode *tutor*) (problem-auto-hint (tutor-current-problem *tutor*))
	      (tutor-eqn-history-backup *tutor*) (make-history :eqn (copy-boxes (tutor-eqn-root *tutor*) nil)
                                                 :incorrect nil))
	(history-add-new-prob (box-to-tree (tutor-eqn-root *tutor*) nil t))
        (set-view-size (tutor-eqn-view *tutor*) 0 0)
        (set-view-size (tutor-last-eqn-view *tutor*) 0 0)
        (dformat "2a: ~a~%" (view-container *message-view*))
	(refresh-eqn t)
        (dformat "3: ~a~%" (view-container *message-view*))
        (draw-stats)
	;; call what's next
	(if (auto-whats-next-mode)
            (do-whats-next (auto-whats-next-mode))
          (clear-message t)))
    (progn
      (fsm-next))))

(defun correct-problem-answer-p ()
  "Return T if the problem is equivalent to the answer."
;;   (and (loop for i in (tutor-eqn-history-before *tutor*)
;;              never (history-incorrect i))
;;        (not (tutor-incorrect-submit *tutor*))
;;        (not (can-do-more t))))
  (let ((attempt (box-to-tree (tutor-eqn-root *tutor*)))
        (answers (problem-answer (tutor-current-problem *tutor*))))
    (find attempt answers :test #'equal)))
  

(defun handle-problem-end (clicked)
  (unless (tutor-submit-enabled *tutor*)
    (if (correct-problem-answer-p)
        (progn
          ;; Give them the score for the problem.
          (dformat "score: ~a ~a ~%" (tutor-score *tutor*) (tutor-problem-score *tutor*))
          (unless (auto-whats-next-mode)
            (incf (tutor-score *tutor*) (max 0 (tutor-problem-score *tutor*))))
          (history-add-next-prob t clicked)
          (next-problem))
      (progn
        (history-add-next-prob nil clicked)
        (if (can-do-more)
            (set-message-text +not-done-text+)
          (set-message-text +incorrect-text+))))
    ;; if there aren't any left display the end test page, dump stats?
    (when (null (tutor-current-problem *tutor*))
      (dformat "You're done.~%"))))

(defun dump-buttons ()
  (nconc (all-operations)
         '(:back :forward :first-mistake :hint :next-problem)))

(defmethod fsm-dump-screen ((blk (eql 'problem)))
  (list (cons :old-equation (and (last-eqn) (dump-equation (last-eqn))))
	(cons :equation (dump-equation (tutor-eqn-root *tutor*)))
	(cons :buttons (dump-buttons))
        (list :input-dialog *input-dialog*)
        (list :instructions (instructions (tutor-problem-set *tutor*)))
;; 	(list :demerits *demerits*)
	(list :text (when *message-widget*
		      (if (typep *message-widget* 'static-text-dialog-item)
                          (dialog-item-text *message-widget*)
                        (map 'list 'dialog-item-text (view-subviews *message-widget*)))))))
	

(defmethod fsm-execute-command ((blk (eql 'problem)) (cmd (eql :click)) target)
  (let ((symbol (intern (symbol-name target) :tutor)))
    (cond ((find-operation symbol)
           (handle-operation (find-operation symbol) t))
          ((member target '(:back :forward :first-mistake :hint :next-problem))
           (handle-button target))
          ((find-if (lambda (region)
                      (eq target (click-region-id region)))
                    (tutor-click-regions *tutor*))
           ;; Fake a click in the region
           (let ((region (find-if (lambda (region)
                                    (eq target (click-region-id region)))
                                  (tutor-click-regions *tutor*))))
             (if (click-region-view region)
                 (handle-click-region (click-region-view region))
               (handle-click-event (1+ (click-region-x1 region)) (1+ (click-region-y1 region))))))
          (t
           (error "~a is not clickable" target)))))

(defmethod fsm-execute-command ((blk (eql 'problem)) (cmd (eql :type)) key)
  "Type the key. If *input-dialog* is non-nil then send the key
  to it."
  (if *input-dialog*
      (type-in-dialog *input-dialog* key)
;;     (let ((opn (cdr (find-if (lambda (x)
;;                                (eq (operation-quick-key (cdr x)) key))
;;                              *operations*))))
;;       (if opn
;;           (handle-operation (operation-name opn))))))
    (handle-key-press key)))

(defvar *focus-view* nil)
(defvar *focus-size* 60)

(defun delete-focus-view ()
  (when *focus-view*
    (remove-subviews *window* *focus-view*))
  (setf *focus-view* nil))

(defun update-focus-view (x y)
  (when *focus-view*
    (remove-subviews *window* *focus-view*))
  (let ((s (truncate *focus-size* 2)))
  (setf *focus-view* (draw-oval *window* (- x s) (- y s) (+ x s) (+ y s) *red-color*))))

(defun focus-on-button (txt)
  (let* ((views (all-subviews *window* 'button-dialog-item))
         (b (find-if (lambda (v)
                       (string= (dialog-item-text v) txt))
                     views)))
    (when b
      (let ((coord (convert-coordinates (view-center b) b *window*)))
      (update-focus-view (point-h coord) (point-v coord))))))

(defmethod fsm-execute-command ((blk (eql 'problem)) (cmd (eql :focus)) target)
  (let ((symbol (intern (symbol-name target) :tutor)))
    (cond ((null target)
           (delete-focus-view))
          ((find-operation symbol)
           ;; focus the button
           (focus-on-button (op-name (find-operation symbol))))
          ((member target '(:back :forward :first-mistake :hint :next-problem))
           ;; other button
           (focus-on-button (nth (position target '(:back :forward :first-mistake :hint :next-problem)) '("<-" "->" "First Mistake" "Hint" "Next Problem"))))
          ((find-tree (tutor-eqn-root *tutor*)
                      (lambda (b)
                        (eq (box-id b) target)))
           (if (tutor1-p)
               (let* ((box (find-tree (tutor-eqn-root *tutor*)
                                      (lambda (b)
                                        (eq (box-id b) target))))
                      (coord (convert-coordinates (rect-center (box-x box) (box-y box) +box-width+ +box-height+) (tutor-eqn-view *tutor*) *window*)))
                 (update-focus-view (point-h coord) (point-v coord)))
             (error "tutor2 TODO.")))
           ((eq target :instructions)
           (let ((coord (convert-coordinates (view-center (tutor-instruction-view *tutor*)) (tutor-instruction-view *tutor*) *window*)))
             (update-focus-view (point-h coord) (point-v coord))))
          ((eq target :text)
           (let ((coord (convert-coordinates (view-center *message-view*) *message-view* *window*)))
             (update-focus-view (point-h coord) (point-v coord))))
          (t (error "~a is not a focusable element." target)))))

(defmethod fsm-execute-command ((blk (eql 'problem)) (cmd (eql :submit)) args)
  (declare (ignore args))
  (if *input-dialog*
      (submit-dialog *input-dialog*)
    (error "No input dialogs.")))

;; FIXME: should we just let CLOS barf an error?
(defmethod fsm-execute-command ((blk (eql 'problem)) cmd args)
  (declare (ignore blk args))
  (error "Unknown command: ~a" cmd))

(defun tutor1-draw-eqn-layout (simplified-p)
  ;;(delete-subviews *window*)
  (let ((opns (mapcar (lambda (o)
                        (list :button :text (op-name o) :action (button-fn (handle-operation o t))))
                      *operations*)))
    (if simplified-p
        (create-layout `(:vbox :contents ( ;;(:space :height ,+window-pad+)
                                          (:hbox :contents ((:space :width ,+window-pad+)
                                                            (:vbox :contents ((:space :height ,+window-pad+)
                                                                              (:hbox :contents ((:vbox :contents ((:view :name eqn))
                                                                                                       :border t
                                                                                                       :name main-area)
;;                                                                                                 (:space :width 5)
;;                                                                                                 (:vbox :contents ((:view :height 200
;;                                                                                                                          :name last-eqn
;;                                                                                                                          :border t)
;;                                                                                                                   (:space))
;;                                                                                                        :width 200)
                                                                                                )
                                                                                     )
                                                                              (:space :height ,+window-pad+))
                                                                   :pad 3)

                                                            (:space :width ,+window-pad+))
                                                 :pad 3)
                                          (:space :height ,(- +window-pad+ 25))))
                       *window*)
      (create-layout `(:vbox :contents ( ;;(:space :height ,+window-pad+)
                                        (:hbox :contents ((:space :width ,+window-pad+)
                                                          (:vbox :contents ((:view :name instruction :border nil :height 200)
                                                                            (:vbox :contents ((:text :text " Last Diagram State" :height 17)
                                                                                              (:view :name last-eqn))
                                                                                   :border t))
                                                                 :pad 3)
                                                          (:vbox :contents ((:view :name message :border nil :height 200)
                                                                            (:vbox :contents ((:text :text " Current Diagram State" :height 17)
                                                                                              (:view :name eqn))
                                                                                   :border t))
                                                                 :pad 3)
                                                          (:vbox :contents ((:space :height 200)
                                                                            (:vbox :contents (,@opns)
                                                                                   :height 174)
                                                                            (:space :height 30)
                                                                            (:vbox :contents ((:hbox :contents ((:button :text "<-" :action ,(button-fn (handle-button :back)))
                                                                                                                (:button :text "->" :action ,(button-fn (handle-button :forward))))
                                                                                                     :pad 5
                                                                                                     :height 30)
                                                                                              (:vbox :contents ((:button :text "First Mistake" :action ,(button-fn (handle-button :first-mistake)) :height nil)
                                                                                                                (:button :text "Hint" :action ,(button-fn (handle-button :hint)) :height nil)
                                                                                                                (:button :text "Next Problem" :action ,(button-fn (handle-button :next-problem)) :height nil))
                                                                                                     :height 90)
                                                                                              (:view :name problems :height 30)
                                                                                              (:view :name score :height 30)
                                                                                              (:space))
                                                                                   :pad 5)
                                                                            )
                                                                 :width 150)
                                                          (:space :width 2))
                                               :pad 3)
                                        (:space :height ,+window-pad+)))
                     *window*))))

(defun draw-just-instr ()
  (create-layout `(:vbox :contents ((:hbox :contents ((:space)
                                                      (:vbox :contents ((:space)
                                                                        (:view :name instruction :height 200)
                                                                        (:space))
                                                             :width 600)
                                                      (:space)))))
                 *window*))

(defun tutor2-draw-eqn-layout (simplified-p)
  ;;(delete-subviews *window*)
  (let ((opns (mapcar (lambda (o)
                        (list :button :text (op-name o) :action (button-fn (handle-operation o t))))
                      *operations*)))
    (if simplified-p
        (create-layout `(:vbox :contents ( ;;(:space :height ,+window-pad+)
                                          (:hbox :contents ((:space :width ,+window-pad+)
                                                            (:vbox :contents ((:space :height ,+window-pad+)
                                                                              (:vbox :contents ((:view :name eqn))
                                                                                     :border t)
                                                                              (:space :height ,+window-pad+)
                                                                              ;; (:view :name message :border nil :height 120)
                                                                              )
                                                                   :pad 3)
;;                                                             (:vbox :contents ((:space :height 200)
;;                                                                               (:vbox :contents (,@opns)
;;                                                                                      :height 174)
;;                                                                               (:space :height 30)
;;                                                                               (:vbox :contents ((:vbox :contents ((:button :text "Hint" :action ,(button-fn (handle-button :hint)) :height nil)
;;                                                                                                                   (:space)
;;                                                                                                                   (:space))
;;                                                                                                        :height 90)
;;                                                                                                 (:space :height 30)
;;                                                                                                 (:view :name problems :height 30)
;;                                                                                                 (:view :name score :height 30)
;;                                                                                                 (:space))
;;                                                                                      :pad 5)
;;                                                                               )
;;                                                                    :width 150)
                                                            (:space :width ,+window-pad+))
                                                 :pad 3)
                                          (:space :height ,(- +window-pad+ 25))))
                       *window*)
      (create-layout `(:vbox :contents ( ;;(:space :height ,+window-pad+)
                                        (:hbox :contents ((:space :width ,+window-pad+)
                                                          (:vbox :contents ((:view :name instruction :border nil :height 200)
                                                                            (:vbox :contents ((:view :name eqn))
                                                                                   :border t)
                                                                            (:view :name message :border nil :height 150))
                                                                 :pad 3)
                                                          (:vbox :contents ((:space :height 200)
                                                                            (:vbox :contents (,@opns)
                                                                                   :height 174)
                                                                            (:space :height 30)
                                                                            (:vbox :contents ((:hbox :contents ((:button :text "<-" :action ,(button-fn (handle-button :back)))
                                                                                                                (:button :text "->" :action ,(button-fn (handle-button :forward))))
                                                                                                     :pad 5
                                                                                                     :height 30)
                                                                                              (:vbox :contents ((:button :text "First Mistake" :action ,(button-fn (handle-button :first-mistake)) :height nil)
                                                                                                                (:button :text "Hint" :action ,(button-fn (handle-button :hint)) :height nil)
                                                                                                                (:button :text "Next Problem" :action ,(button-fn (handle-button :next-problem)) :height nil))
                                                                                                     :height 90)
                                                                                              (:view :name problems :height 30)
                                                                                              (:view :name score :height 30)
                                                                                              (:space))
                                                                                   :pad 5)
                                                                            )
                                                                 :width 150)
                                                          (:space :width 2))
                                               :pad 3)
                                        (:space :height ,+window-pad+)))
                     *window*))))

(defun simplified-ui-instructions (problem-set)
  (let ((views (draw-just-instr)))
    (dformat "~s ~s~%" (cdr (assoc 'instruction views)) (instructions (first problem-set)))
    (draw-block (instructions (first problem-set)) (cdr (assoc 'instruction views)))
    (history-show-instructions)
    (tutor-sleep *instruction-delay-amount*)
    (delete-subviews *window*)
    (history-end-instructions)
    (tutor-fsm-post-init problem-set)))

(defun tutor-fsm-post-init (problem-set)
  (let* ((views (if (tutor1-p)
                   (tutor1-draw-eqn-layout *simplified-ui*)
                 (tutor2-draw-eqn-layout  *simplified-ui*)))
         (main-area (cdr (assoc 'main-area views))))
    ;; We get passed a list with the problem set in it.
    (setf problem-set (first problem-set))

    (when *simplified-ui*
      (setf (tutor-last-eqn-top-view *tutor*) (let* ((x (make-point (- (view-width main-area) 240)))
                                                     (y 20))
                                                (make-instance 'view
                                                               :view-position (make-point x y)
                                                               :view-size #@(222 202)
                                                               :view-container main-area))))

    (setf (tutor-eqn-area-view *tutor*) (cdr (assoc 'eqn views))
          (tutor-last-eqn-area-view *tutor*) (if (and (tutor1-p)
                                                      *simplified-ui*)
                                                 (let* ((top (tutor-last-eqn-top-view *tutor*)))
                                                   (draw-rect top 0 0 221 201 *light-gray-color*)
                                                   (make-instance 'view
                                                                  :view-position #@(1 1)
                                                                  :view-size #@(220 200)
                                                                  :view-container top))
                                               (cdr (assoc 'last-eqn views)))
          (tutor-score-view *tutor*) (cdr (assoc 'score views))
          (tutor-problems-view *tutor*) (cdr (assoc 'problems views))
          *message-view* (cdr (assoc 'message views))
          (tutor-instruction-view *tutor*) (cdr (assoc 'instruction views))
          (tutor-eqn-view *tutor*) (make-instance 'box-view
                                                  :view-position #@(0 0)
                                                  :view-size #@(0 0)
                                                  :view-container (tutor-eqn-area-view *tutor*))
          (tutor-last-eqn-view *tutor*) (make-instance 'box-view
                                                       :view-position #@(0 0)
                                                       :view-size #@(0 0)
                                                       :view-container (tutor-last-eqn-area-view *tutor*)))
    (when *simplified-ui*
      (setf (tutor-buttons *tutor*) (draw-simplified-ui-buttons (tutor-eqn-area-view *tutor*)))
      (setf *message-view* (make-instance 'view
                                          :view-container (tutor-eqn-area-view *tutor*)
                                          :view-size #@(500 150))))
    ;; Show the instructions
    ;;(dformat "inst: ~a~%" views)
    (unless *simplified-ui*
      (draw-block (instructions problem-set) (tutor-instruction-view *tutor*)))
    ;; some widgets
    (init-message-widget)
    (init-demerits-widget)
    (init-whatsnext-arrows)
    (init-problems-left-widget)
    (setf (tutor-submit-enabled *tutor*) nil)
    ;;(make-button-ops)
    (setf (tutor-problem-set *tutor*) problem-set
          (tutor-num-problems *tutor*) (length (problems problem-set)))
    (dformat "1: ~a~%" (view-container *message-view*))
    (next-problem)
    (dformat "2: ~a~%" (view-container *message-view*))))

(defmethod fsm-init ((blk (eql 'problem)) problem-set)
  (if *instruction-delay*
      (simplified-ui-instructions problem-set)
    (tutor-fsm-post-init problem-set)))

(defmethod fsm-end :around ((blk (eql 'problem)))
  (clear-message t)
  (call-next-method))

(defun setup-tutor1 ()
  (setf *which-tutor* 'tutor1))

(defun setup-tutor2 ()
  (setf *which-tutor* 'tutor2))

(defun tutor1-p ()
  (eq *which-tutor* 'tutor1))

(defun tutor2-p ()
  (eq *which-tutor* 'tutor2))

(defmethod view-key-event-handler :before ((w ccl::string-dialog) ch)
  (history-log-input ch))

;;;;;; TEST TEST
;; Lets see if we can record keystrokes and mouse clicks

(defmethod view-key-event-handler :after ((w tutor-window) ch)
  (playback-log-keypress ch))


(defmethod view-click-event-handler :after ((view tutor-window) where)
  (playback-log-click where))
