(declaim (optimize (debug 3) (speed 0) (size 0)))

(in-package :tutor)

(defvar *input-dialog* nil
  "The dialog box for typing input.")

(defun clear-input-dialog ()
  "Call this if the fill-in stage must be aborted. This happens if the user presses <-."
  (when *input-dialog*
    (window-close *input-dialog*)
    (handle-window-close *input-dialog*)))

(defun read-template (title text post-fn)
  (let (dialog)
    ;; If the user clicked a green box while the dialog was still
    ;; open, close it then proceed. Otherwise they may get confused
    ;; because they click the green box and nothing happens. Little do
    ;; they know a dialog window is hiding behind the tutor window.
    (when *input-dialog*
      (window-close *input-dialog*)
      (setf *input-dialog* nil))
    (labels ((on-submit (input)
                        (window-close dialog)
                        (funcall post-fn input)
                        (handle-window-close dialog)))
      (setf dialog (get-string-from-user text
                                         :position (make-point (truncate (tutor-width) 2) (tutor-height))
                                         :window-title title
                                         :size #@(400 100)
                                         :modeless t
                                         :action-function #'on-submit))
      (setf *input-dialog* dialog))))

;;;; pop up a window and ask for a number or an operator


;; EQN = NUMBER OP NUMBER
;; 

(defmacro expect (test &body body)
  `(if ,test
       (progn ,@body)
     (error "Parse failure: ~a" ,test)))

(defun parse-number (stream)
  (let ((acc (make-array 1 :element-type 'character :adjustable t :fill-pointer 0)))
    ;; Read a possible -
    (when (char= #\- (peek-char nil stream))
      (vector-push-extend (read-char stream) acc))
    ;; Read the number
    (expect (digit-char-p (peek-char nil stream))
            (vector-push-extend (read-char stream) acc))
    (loop while (and (peek-char nil stream nil)
                     (digit-char-p (peek-char nil stream)))
          do (vector-push-extend (read-char stream) acc))
    (read-from-string acc)))

(defun parse-operator (stream)        
  ;; Read the operator
  (expect (find (peek-char nil stream) "+-*/")
          (read-from-string (string (read-char stream)))))

(defun parse-fraction (stream)
  (let ((acc (make-array 1 :element-type 'character :adjustable t :fill-pointer 0)))
    ;; Read a possible -
    (when (char= #\- (peek-char nil stream))
      (vector-push-extend (read-char stream) acc))
    ;; Read the number
    (expect (digit-char-p (peek-char nil stream))
            (vector-push-extend (read-char stream) acc))
    (loop while (digit-char-p (peek-char nil stream))
          do (vector-push-extend (read-char stream) acc))
    ;; read fraction
    (expect (char= #\/ (peek-char nil stream))
            (vector-push-extend (read-char stream) acc))
    ;; Read the number
    (expect (digit-char-p (peek-char nil stream))
            (vector-push-extend (read-char stream) acc))
    (loop while (and (peek-char nil stream nil)
                     (digit-char-p (peek-char nil stream)))
          do (vector-push-extend (read-char stream) acc))
    (read-from-string acc)))

;; (defmacro define-input-parser (name &rest fields)
;;   (with-gensyms (str parse-templ p1 stream)
;;     `(defun ,name (str)
;;        (macrolet ((,parse-templ ,(loop for f in fields collect (second f))
;;                      (let ((,stream (make-string-input-stream str)))
;;                        ;; read space
;;                        (loop while (find (peek-char nil ,stream) '(#\Space #\Tab)) do (read-char ,stream))
;;                        (let ,(loop for f in fields
;;                                    collect `(,(first f) (prog1
;;                                                             (,(second f) ,stream)
;;                                                           (loop while (find (peek-char nil ,stream) '(#\Space #\Tab)) do (read-char ,stream)))))
;;                          ;; read trailing space
;;                          (loop while (find (peek-char nil ,stream) '(#\Space #\Tab)) do (read-char ,stream))
;;                          (if (eq (peek-char nil ,stream nil ,stream) ,stream)
;;                              (list ,@(loop for f in fields collect (first f)))
;;                            (error "trailing garbage"))))))
;;          (or ,@(loop for 

(defun parse-seq (str &rest fns)
  "use each fn in fns to parse string. Eat whitespace before and
  after each fn. Return the result in a list or signal an
  error if it wasn't able to parse."
  (labels ((eat-space (stream)
             (loop while (find (peek-char nil stream) '(#\Space #\Tab)) do (read-char stream)))
           (maybe-eat-space (stream)
            (loop while (and (peek-char nil stream nil)
                             (find (peek-char nil stream) '(#\Space #\Tab))) do (read-char stream))))
    (let ((stream (make-string-input-stream str)))
      (prog1
          (loop for fn in fns
                collect (progn
                          (eat-space stream)
                          (funcall fn stream)))
        (maybe-eat-space stream)
        (unless (eq (peek-char nil stream nil stream) stream)
          (error "trailing garbage"))))))

(defun parse-arg-op-input (str)
  (or (ignore-errors (parse-seq str 'parse-number 'parse-operator))
      (ignore-errors (parse-seq str 'parse-fraction 'parse-operator))))
  
(defun parse-op-arg-input (str)
  (or (ignore-errors (parse-seq str 'parse-operator 'parse-number))
      (ignore-errors (parse-seq str 'parse-operator 'parse-fraction))))

(defun parse-box-input (str)
  (macrolet ((line (fn1 fn2)
               `(ignore-errors (parse-seq str ,fn1 'parse-operator ,fn2))))
    (or (line 'parse-number 'parse-number)
        (line 'parse-number 'parse-fraction)
        (line 'parse-fraction 'parse-number)
        (line 'parse-fraction 'parse-fraction))))

(defun read-box (post-fn box)
  "Call post-fn with a list of up to 3 atoms or nil if it failed."
  (let ((fn (if *use-mouse-input*
                (lambda (title text fn)
                  (let ((p (convert-coordinates (make-point 0 (view-height (box-answer-view box)))
                                               (box-answer-view box)
                                               *window*)))
                    (read-mouse-input (point-h p) (point-v p)
                                      title text fn)))
              'read-template)))
    (funcall fn (if (tutor1-p) "Box Contents" "Expression")
                   (format nil "Type the contents of the ~a.~%Fractions are written like this: 1/2, 13/8, etc." (if (tutor1-p) "operator box" "expression"))
                   (lambda (input)
                     (let ((a (parse-box-input input))
                           (b (parse-number-input input)))
                     (funcall post-fn (or a b)))))))

(defun read-arg-op (post-fn box)
  "Call post-fn with a list of up to 3 atoms or nil if it failed."
  (let ((fn (if *use-mouse-input*
                (lambda (title text fn)
                  (let ((p (convert-coordinates (make-point 0 (view-height (box-answer-view box)))
                                               (box-answer-view box)
                                               *window*)))
                    (read-mouse-input (point-h p) (point-v p)
                                      title text fn)))
              'read-template)))
    (funcall fn (if (tutor1-p) "Box Contents" "Expression")
             (format nil "Type the left number and operator of the ~a.~%Fractions are written like this: 1/2, 13/8, etc." (if (tutor1-p) "box" "expression"))
             (lambda (input)
               (funcall post-fn (parse-arg-op-input input))))))

(defun read-op-arg (post-fn box)
  "Call post-fn with a list of up to 3 atoms or nil if it failed."
  (let ((fn (if *use-mouse-input*
                (lambda (title text fn)
                  (let ((p (convert-coordinates (make-point 0 (view-height (box-answer-view box)))
                                               (box-answer-view box)
                                               *window*)))
                    (read-mouse-input (point-h p) (point-v p)
                                      title text fn)))
              'read-template)))
    (funcall fn (if (tutor1-p) "Box Contents" "Expression")
             (format nil "Type the operator and right number of the ~a.~%Fractions are written like this: 1/2, 8/13, etc." (if (tutor1-p) "box" "expression"))
             (lambda (input)
               (funcall post-fn (parse-op-arg-input input))))))

;; (let* ((stream (make-string-input-stream input))
;;                                            (tokens (list (ignore-errors (read stream))
;;                                                          (ignore-errors (read stream))
;;                                                          (ignore-errors (read stream)))))
;;                                       (when (and (numberp (first tokens))
;;                                                  (find (second tokens) '(+ - * /))
;;                                                  (numberp (third tokens)))
;;                                         tokens))

(defun parse-number-input (input)
  (let ((n (ignore-errors 
             (multiple-value-bind (n p) (read-from-string input)
               (assert (= p (length input)))
               n))))
    ;;(format t "~a~%" n)
    (when (numberp n)
      n)))

(defun parse-operator-input (input)
  (let ((op (ignore-errors 
             (multiple-value-bind (op p) (read-from-string input)
               (assert (= p (length input)))
               op))))
    ;;(format t "~a~%" n)
    (when (find op '(+ - * /))
      op)))

(defun read-number (post-fn arg)
  "call POST-FN with a symbol or nil if it failed."
  (let ((fn (if *use-mouse-input*
                (lambda (title text fn)
                  (let ((p (convert-coordinates (make-point 0 (view-height (arg-widget arg)))
                                               (arg-widget arg)
                                               *window*)))
                    (read-mouse-input (point-h p) (point-v p)
                                      title text fn)))
              'read-template)))
    (funcall fn "Number"
             (format nil "Type a number.~%Fractions are written like this: 1/2, 8/13, etc.")
             (lambda (input)
               (funcall post-fn (parse-number-input input))))))

(defun read-operator (post-fn arg)
  "call post-fn with a symbol or nil if it failed."
  (let ((fn (if *use-mouse-input*
                (lambda (title text fn)
                  (let ((p (convert-coordinates (make-point 0 (view-height (arg-widget arg)))
                                               (arg-widget arg)
                                               *window*)))
                    (read-mouse-input (point-h p) (point-v p)
                                      title text fn)))
              'read-template)))
    (funcall fn "Operator"
             "Type an operator."
             (lambda (input)
               (funcall post-fn 
                        (parse-operator-input input))))))

(defmethod window-close-event-handler :after ((window dialog))
  (handle-window-close window))

(defun handle-window-close (window)
  (history-add-close-dialog)
  (when (eq window *input-dialog*)
    (setf *input-dialog* nil)))

(defun type-in-dialog (dialog char)
  (let ((view (find-if (lambda (d)
                         (typep d 'editable-text-dialog-item))
                       (subviews dialog))))
    (cond ((graphic-char-p char) (ed-insert-char view char))
          ((char= char #\backspace) (ed-rubout-char view)))
    (view-draw-contents view)))

(defun submit-dialog (dialog)
  (let ((view (find-if (lambda (d)
                         (typep d 'default-button-dialog-item))
                       (subviews dialog))))
    (press-button view)))


;;;;;; input in the MRI 

(defclass input-window (view)
  ())

(defvar *input-window* nil)
(defvar *input-text* nil)

(defparameter +input-window-width+ 150)
(defparameter +input-window-height+ 125)

(defun add-input (str)
  (set-dialog-item-text *input-text* (concatenate 'string (dialog-item-text *input-text*) str)))

(defun read-mouse-input (x y title text post-fn)
  (declare (ignore text title))
  (labels ((submit-it ()
                      (let ((txt (dialog-item-text *input-text*)))
                        ;;(window-close *input-window*)
                        (remove-subviews *window* *input-window*)
                        ;; restore the operation buttons
                        ;; (unhide-view (tutor-buttons *tutor*))
                        (setf *input-window* nil
                              *input-text* nil)
                        (history-input-click-button 'submit)
                        (funcall post-fn txt)))
           (input-clear ()
                        (set-dialog-item-text *input-text* "")
                        (history-input-click-button 'clear)))
  (macrolet ((input (s)
                    `(lambda (foo) 
                       (declare (ignore foo))
                       (add-input ,s)
                       (history-input-click-button ,s))))
;;     (hide-view (tutor-buttons *tutor*))
    (when *input-window*
      (remove-subviews *window* *input-window*)
      (history-input-close))
    (history-input-open)
    (setf *input-window* (make-instance 'input-window
;;                                         :window-title title
;;                                         :window-type :document
;;                                         :close-box-p nil
                                        :view-font '("Monaco" 14)
                                        :view-position (make-point x y)
                                        :view-container *window*
                                        :view-size (make-point +input-window-width+ +input-window-height+)))
    (let ((views (create-layout `(:vbox :border t
                                        :contents ((:hbox :contents ((:button :width 37 :text "^" :action ,(button-fn (submit-it)))
                                                                     (:text :text "" :width 4)
                                                                     (:text :text "" :name text)))
                                                   (:hbox :contents ((:button :text "7" :action ,(input "7"))
                                                                     (:button :text "8" :action ,(input "8"))
                                                                     (:button :text "9" :action ,(input "9"))
                                                                     (:button :text "+" :action ,(input "+"))))
                                                   (:hbox :contents ((:button :text "4" :action ,(input "4"))
                                                                     (:button :text "5" :action ,(input "5"))
                                                                     (:button :text "6" :action ,(input "6"))
                                                                     (:button :text "-" :action ,(input "-"))))
                                                   (:hbox :contents ((:button :text "1" :action ,(input "1"))
                                                                     (:button :text "2" :action ,(input "2"))
                                                                     (:button :text "3" :action ,(input "3"))
                                                                     (:button :text "*" :action ,(input "*"))))
                                                   (:hbox :contents ((:button :text "0" :action ,(input "0"))
                                                                     (:button :text "C" :action ,(button-fn (input-clear)))
                                                                     (:button :text "" :action ,(input "/"))
                                                                     (:button :text "/" :action ,(input " / "))))))
                                *input-window*)))
      (setf *input-text* (cdr (assoc 'text views)))))))
